@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 
  
 
-/* $Id: render_handler_agg.cpp,v 1.95 2007/07/18 23:17:54 strk Exp $ */
+/* $Id: render_handler_agg.cpp,v 1.127.2.2 2008/03/04 11:03:59 strk Exp $ */
 
 // Original version by Udo Giacomozzi and Hannes Mayr, 
 // INDUNET GmbH (www.indunet.it)
@@ -48,16 +48,17 @@ Status
     solid fills       COMPLETE
     linear gradients  COMPLETE
     radial gradients  COMPLETE
-    focal gradients   NOT IMPLEMENTED *
+    focal gradients   COMPLETE
+    ext. spread modes NOT IMPLEMENTED *
+    linear RGB mode   NOT IMPLEMENTED *
     bitmaps, tiled    COMPLETE
     bitmaps, clipped  COMPLETE
     bitmaps, smooth   COMPLETE
     bitmaps, hard     COMPLETE    
     color xform       COMPLETE
     
-    * focal gradients (introduced in Flash 7, I think) are not yet supported 
-    by Gnash itself AFAIK, but AGG supports them and it should be easy to add 
-    them.    
+    * special fill styles are not yet supported by Gnash itself AFAIK, but AGG 
+    supports them and it should be easy to add them.    
     
   fonts               COMPLETE
     
@@ -67,8 +68,7 @@ Status
   
   video               COMPLETE
   
-  Currently the renderer should be able to render everything correctly,
-  except videos.
+  Currently the renderer should be able to render everything correctly.
   
   
 What could and should be /optimized/
@@ -110,7 +110,7 @@ AGG ressources
 */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 
@@ -172,10 +172,6 @@ AGG ressources
 #include "render_handler_agg_bitmap.h"
 #include "render_handler_agg_style.h"
 
-#ifndef trunc
-#define trunc(x) ( x < 0 ?  -floor(-x) : floor(x) )
-#endif
-
 #ifndef round
 #define round(x) rint(x)
 #endif
@@ -187,6 +183,7 @@ using namespace gnash;
 
 namespace gnash {
 
+
 // --- CACHE -------------------------------------------------------------------
 /// This class holds a completely transformed path (fixed position). Speeds
 /// up characters that stay fixed on a certain position on the stage. 
@@ -194,21 +191,48 @@ namespace gnash {
 
 class agg_transformed_path 
 {
+public:
   /// Original transformation matrix 
   matrix m_mat;  
   
+  /// Normal or rounded coordinates?
+  bool m_rounded;
+  
   /// Number of cache hits 
-  int hits;
+  int m_hits;
   
-  /// Number of cache misses
-  int misses;
-  
-  /// Contents of this cache item. First dimension is fill style 
-  std::vector <std::vector <agg::path_storage> > data;
+  /// Contents of this cache item (AGG path).  
+  std::vector <agg::path_storage> m_data;
 };
 
 class agg_cache_manager : private render_cache_manager
 {
+
+  std::vector <agg_transformed_path> m_items;
+
+  /// Looks for a matching pre-computed path in the cache list
+  /// Returns NULL if no cache item matches 
+  std::vector <agg::path_storage>* search(const matrix& mat, bool rounded) {
+  
+    size_t ccount = m_items.size();
+    
+    for (size_t cno=0; cno<ccount; cno++) {    
+      agg_transformed_path& item = m_items[cno];
+          
+      if ((item.m_mat == mat) && (item.m_rounded == rounded)) {
+      
+        // Found it!
+        return &item.m_data;
+      
+      }    
+    }
+    
+    // could not find a matching item
+    return NULL;
+  
+  }
+  
+
 };
 
 
@@ -248,7 +272,7 @@ public:
     // * = m_rbuf is first initialized with a NULL buffer so that m_pixf and
     // m_rbase initialize with the correct buffer extents
   
-    m_buffer = new uint8_t[width*height];
+    m_buffer = new boost::uint8_t[width*height];
     
     m_rbuf.attach(m_buffer, width, height, width);
     
@@ -295,7 +319,7 @@ public:
   
 private:
   // in-memory buffer
-  uint8_t* m_buffer;
+  boost::uint8_t* m_buffer;
   
   // agg class to access the buffer
   agg::rendering_buffer m_rbuf;
@@ -339,8 +363,6 @@ private:
 
 
 public:
-  //int              m_view_width;      // TODO: remove these??
-  //int              m_view_height;
 
   // Enable/disable antialiasing.
   bool  m_enable_antialias;
@@ -349,9 +371,6 @@ public:
   float m_display_width;
   float m_display_height;
 
-  gnash::matrix m_current_matrix;
-  gnash::cxform m_current_cxform;
-
   void set_antialiased(bool enable) {
     // enable=false *forces* all bitmaps to be rendered in low quality
     m_enable_antialias = enable;
@@ -372,9 +391,9 @@ public:
   // that can later be passed to fill_styleX_bitmap(), to set a
   // bitmap fill style.
   {    
-    return new agg_bitmap_info<agg::pixfmt_rgb24_pre> (im->m_width, im->m_height,
-      im->m_pitch, im->m_data, 24);
-    assert(0); 
+    return new agg_bitmap_info<agg::pixfmt_rgb24_pre> (im->width(), im->height(),
+      im->pitch(), im->data(), 24);
+    abort(); 
   }
 
 
@@ -385,8 +404,8 @@ public:
   //
   // This version takes an image with an alpha channel.
   {
-    return new agg_bitmap_info<agg::pixfmt_rgba32_pre> (im->m_width, im->m_height,
-      im->m_pitch, im->m_data, 32); 
+    return new agg_bitmap_info<agg::pixfmt_rgba32_pre> (im->width(), im->height(),
+      im->pitch(), im->data(), 32); 
   }
 
 
@@ -402,19 +421,6 @@ public:
     return new agg_bitmap_info<agg::pixfmt_rgb24_pre> (0, 0, 0, &dummy, 24);
   }
 
-  gnash::bitmap_info* create_bitmap_info_alpha(int /*w*/, int /*h*/, uint8_t* /*data*/)
-  // Create a bitmap_info so that it contains an alpha texture
-  // with the given data (1 byte per texel).
-  {
-    //return new agg_bitmap_info<agg::pixfmt_gray8> (w, h, w, data, 8);
-
-    // where is this used, anyway??
-    log_msg("create_bitmap_info_alpha() currently not supported");
-    
-    return new bitmap_info();
-  }
-
-
   void  delete_bitmap_info(gnash::bitmap_info* bi)
   // Delete the given bitmap info class.
   {
@@ -454,8 +460,8 @@ public:
     mat.concatenate(*source_mat);
     
     // compute video scaling relative to video obejct size
-    double vscaleX = TWIPS_TO_PIXELS(bounds->width())  / frame->m_width;
-    double vscaleY = TWIPS_TO_PIXELS(bounds->height()) / frame->m_height;
+    double vscaleX = TWIPS_TO_PIXELS(bounds->width())  / frame->width();
+    double vscaleY = TWIPS_TO_PIXELS(bounds->height()) / frame->height();
     
     // convert Gnash matrix to AGG matrix and scale down to pixel coordinates
     // while we're at it
@@ -482,8 +488,8 @@ public:
     typedef agg::image_accessor_clone<baseformat> img_source_type;
     
     // rendering buffer is used to access the frame pixels here        
-    agg::rendering_buffer img_buf(frame->m_data, frame->m_width, frame->m_height,
-      frame->m_width*3);
+    agg::rendering_buffer img_buf(frame->data(), frame->width(), frame->height(),
+      frame->pitch());
          
     baseformat img_pixf(img_buf);
     
@@ -508,11 +514,11 @@ public:
     mat.transform(&d, point(bounds->get_x_min(), bounds->get_y_max()));
     
     agg::path_storage path;
-    path.move_to(a.m_x, a.m_y);
-    path.line_to(b.m_x, b.m_y);
-    path.line_to(c.m_x, c.m_y);
-    path.line_to(d.m_x, d.m_y);
-    path.line_to(a.m_x, a.m_y);
+    path.move_to(a.x, a.y);
+    path.line_to(b.x, b.y);
+    path.line_to(c.x, c.y);
+    path.line_to(d.x, d.y);
+    path.line_to(a.x, a.y);
 
     if (m_alpha_mask.empty()) {
     
@@ -570,19 +576,11 @@ public:
       bpp(bits_per_pixel),
       /*xscale(1.0/20.0),
       yscale(1.0/20.0),*/
-      stage_matrix(),
       scale_set(false),
       m_enable_antialias(true),
       m_display_width(0.0),
       m_display_height(0.0),
-      m_current_matrix(),
-      m_current_cxform(),
-      m_rbuf(),
-      m_pixf(NULL), // TODO: use an auto_ptr
-      _clipbounds(),
-      _clipbounds_selected(),
-      m_drawing_mask(false),
-      m_alpha_mask()
+      m_drawing_mask(false)
   {
     // TODO: we really don't want to set the scale here as the core should
     // tell us the right values before rendering anything. However this is
@@ -595,19 +593,15 @@ public:
   // Destructor
   ~render_handler_agg()
   {
-    // don't need to check m_pixf != NULL
-    // as that check is already implemented
-    // in the 'delete' statement
-    // if (m_pixf != NULL)
-      delete m_pixf;    // TODO: is this correct??
   }
 
   /// Initializes the rendering buffer. The memory pointed by "mem" is not
   /// owned by the renderer and init_buffer() may be called multiple times
   /// when the buffer size changes, for example. However, bits_per_pixel must
   /// remain the same. 
+  /// rowstride is the size, in bytes, of one row.
   /// This method *must* be called prior to any other method of the class!
-  void init_buffer(unsigned char *mem, int size, int x, int y)
+  void init_buffer(unsigned char *mem, int size, int x, int y, int rowstride)
   {
         assert(x > 0);
         assert(y > 0);
@@ -617,24 +611,17 @@ public:
     xres    = x;
     yres    = y;
     
-        // don't need to check m_pixf != NULL
-        // as that check is already implemented
-        // in the 'delete' statement
-    //if (m_pixf != NULL)
-      delete m_pixf;    // TODO: is this correct??
-
-    int row_size = xres*((bpp+7)/8);
-    m_rbuf.attach(memaddr, xres, yres, row_size);
+    m_rbuf.attach(memaddr, xres, yres, rowstride);
 
     // allocate pixel format accessor   
-    m_pixf = new PixelFormat(m_rbuf);
+    m_pixf.reset(new PixelFormat(m_rbuf));
     //m_rbase = new renderer_base(*m_pixf);  --> does not work!!??
     
     // by default allow drawing everywhere
     set_invalidated_region_world();
     
-    log_msg("initialized AGG buffer <%p>, %d bytes, %dx%d, rowsize is %d bytes", 
-      mem, size, x, y, row_size);
+    log_debug("initialized AGG buffer <%p>, %d bytes, %dx%d, rowsize is %d bytes", 
+      mem, size, x, y, rowstride);
   }
   
 
@@ -656,21 +643,14 @@ public:
   // coordinates of the movie that correspond to the viewport
   // bounds.
   {
-    assert(m_pixf != NULL);
+    assert(m_pixf.get());
     
     assert(scale_set);
-
     // clear the stage using the background color
     for (unsigned int i=0; i<_clipbounds.size(); i++) 
       clear_framebuffer(_clipbounds[i], agg::rgba8_pre(background_color.m_r,
         background_color.m_g, background_color.m_b,
-        background_color.m_a));
-        
-    // calculate final pixel scale
-    /*double scaleX, scaleY;
-    scaleX = (double)xres / (double)viewport_width / 20.0;  // 20=TWIPS
-    scaleY = (double)yres / (double)viewport_height / 20.0;
-    scale = scaleX<scaleY ? scaleX : scaleY;*/
+        background_color.m_a));        
     
     // reset status variables
     m_drawing_mask = false;
@@ -704,56 +684,22 @@ public:
       }
   }
 
-  bool allow_glyph_textures() {
-    // We want to render all glyphs in place 
-    return false; 
-  }
-
   void  end_display()
   // Clean up after rendering a frame.  Client program is still
   // responsible for calling glSwapBuffers() or whatever.
   {
   
     if (m_drawing_mask) 
-      log_msg("warning: rendering ended while drawing a mask");
+      log_debug("warning: rendering ended while drawing a mask");
       
     while (! m_alpha_mask.empty()) {
-      log_msg("warning: rendering ended while masks were still active");
+      log_debug("warning: rendering ended while masks were still active");
       disable_mask();      
     }
   
     // nothing to do
   }
 
-  void  set_matrix(const gnash::matrix& m)
-  // Set the current transform for mesh & line-strip rendering.
-  {
-    // used only for drawing line strips...    
-    m_current_matrix = m;
-  }
-
-  void  set_cxform(const gnash::cxform& cx)
-  // Set the current color transform for mesh & line-strip rendering.
-  {
-    // used only for drawing line strips...    
-    m_current_cxform = cx;
-  }
-
-  static void apply_matrix(const gnash::matrix& /*m*/)
-  // add user space transformation
-  {
-    // TODO: what's the use for this, anyway?? 
-    log_msg("apply_matrix(); called - NOT IMPLEMENTED");
-  }
-
-  static void apply_color(const gnash::rgba& /*c*/)
-  // Set the given color.
-  {
-    // TODO: what's the use for this, anyway?? 
-    log_msg("apply_color(); called - NOT IMPLEMENTED");
-  }
-
-
   template <class ras_type>
   void apply_clip_box(ras_type& ras, 
     const geometry::Range2d<int>& bounds)
@@ -767,14 +713,16 @@ public:
   }
 
 
-  void  draw_line_strip(const void* coords, int vertex_count, const rgba& color)
+  
+  void  draw_line_strip(const void* coords, int vertex_count, const rgba& color,
+	                const matrix& line_mat)
   // Draw the line strip formed by the sequence of points.
   {
   
-    assert(m_pixf != NULL);
+    assert(m_pixf.get());
 
     matrix mat = stage_matrix;
-    mat.concatenate(m_current_matrix);    
+    mat.concatenate(line_mat);    
 
     if ( _clipbounds.size()==0 ) return;
 
@@ -784,62 +732,81 @@ public:
     
     typedef agg::rasterizer_scanline_aa<> ras_type;
 
-    ras_type ras;
-    agg::scanline_p8 sl;
+    ras_type ras;    
     agg::renderer_scanline_aa_solid<
       agg::renderer_base<PixelFormat> > ren_sl(rbase);
       
-    for (unsigned int cno=0; cno<_clipbounds.size(); cno++) {
+    // -- create path --
+    agg::path_storage path;
+    agg::conv_stroke<agg::path_storage> stroke(path);
+    stroke.width(1);
+    stroke.line_cap(agg::round_cap);
+    stroke.line_join(agg::round_join);
+    path.remove_all(); // Not obligatory in this case
+
+    const boost::int16_t *vertex = static_cast<const boost::int16_t*>(coords);
     
-      const geometry::Range2d<int>& bounds = _clipbounds[cno];
-            
-      apply_clip_box<ras_type> (ras, bounds);
-          
-      agg::path_storage path;
-      agg::conv_stroke<agg::path_storage> stroke(path);
-      stroke.width(1);
-      stroke.line_cap(agg::round_cap);
-      stroke.line_join(agg::round_join);
-      path.remove_all(); // Not obligatory in this case
-  
-      const int16_t *vertex = static_cast<const int16_t*>(coords);
-      
+    mat.transform(&pnt, point(vertex[0], vertex[1]));
+    path.move_to(pnt.x, pnt.y);
+
+    for (vertex += 2;  vertex_count > 1;  vertex_count--, vertex += 2) {
       mat.transform(&pnt, point(vertex[0], vertex[1]));
-      path.move_to(pnt.m_x, pnt.m_y);
-  
-      for (vertex += 2;  vertex_count > 1;  vertex_count--, vertex += 2) {
-        mat.transform(&pnt, point(vertex[0], vertex[1]));
-        path.line_to(pnt.m_x, pnt.m_y);
+      path.line_to(pnt.x, pnt.y);
+    }
+    
+    // -- render --
+    
+    if (m_alpha_mask.empty()) {
+    
+      // No mask active
+      
+      agg::scanline_p8 sl;      
+      
+      for (unsigned int cno=0; cno<_clipbounds.size(); cno++) {
+      
+        const geometry::Range2d<int>& bounds = _clipbounds[cno];
+              
+        apply_clip_box<ras_type> (ras, bounds);
+        
+        // The vectorial pipeline
+        ras.add_path(stroke);
+    
+        // Set the color and render the scanlines
+        ren_sl.color(agg::rgba8_pre(color.m_r, color.m_g, color.m_b, color.m_a));
+        
+        agg::render_scanlines(ras, sl, ren_sl);     
+        
       }
-      // The vectorial pipeline
-      ras.add_path(stroke);
-  
-      // Set the color and render the scanlines
-      ren_sl.color(agg::rgba8_pre(color.m_r, color.m_g, color.m_b, color.m_a));
-  
+      
+    } else {
+    
+      // Mask is active!
+
+      typedef agg::scanline_u8_am<agg::alpha_mask_gray8> sl_type;
+      
+      sl_type sl(m_alpha_mask.back()->get_amask());      
+      
+      for (unsigned int cno=0; cno<_clipbounds.size(); cno++) {
+      
+        const geometry::Range2d<int>& bounds = _clipbounds[cno];
+              
+        apply_clip_box<ras_type> (ras, bounds);
+        
+        // The vectorial pipeline
+        ras.add_path(stroke);
+    
+        // Set the color and render the scanlines
+        ren_sl.color(agg::rgba8_pre(color.m_r, color.m_g, color.m_b, color.m_a));
+        
+        agg::render_scanlines(ras, sl, ren_sl);     
+        
+      }
     
-      agg::render_scanlines(ras, sl, ren_sl);
     }
 
   } // draw_line_strip
 
 
-  void  draw_bitmap(
-  const gnash::matrix& /*m*/,
-  const gnash::bitmap_info* /*bi*/,
-  const gnash::rect& /*coords*/,
-  const gnash::rect& /*uv_coords*/,
-  const gnash::rgba& /*color*/)
-  // Draw a rectangle textured with the given bitmap, with the
-  // given color.  Apply given transform; ignore any currently
-  // set transforms.
-  //
-  // Intended for textured glyph rendering.
-  {
-    log_msg("  draw_bitmap NOT IMPLEMENTED\n");
-    // could be implemented, but is not used
-  }
-
   void begin_submit_mask()
   {
     // Set flag so that rendering of shapes is simplified (only solid fill) 
@@ -867,16 +834,9 @@ public:
   }
   
 
-
-
-
-
   void draw_glyph(shape_character_def *def,
       const matrix& mat, const rgba& color, float /*pixel_scale*/) {
-      
-    // NOTE: def->get_bound() is NULL for glyphs so we can't check the 
-    // clipping area (bounds_in_clipping_area):
-    // create a new path with the matrix applied   
+    
     std::vector<path> paths;    
     apply_matrix_to_path(def->get_paths(), paths, mat);
     
@@ -891,13 +851,14 @@ public:
     agg_style_handler sh;
     build_agg_styles(sh, m_single_fill_styles, mat, m_neutral_cxform);
     
-    // select all clipping ranges. 
-    // NOTE: Glyphs are loaded like normal shape definitons, but w/o style
-    // definitons, which unfortunately include shape bounds. So "def" has
-    // no bounds (isNull) and thus select_clipbounds() won't work.
-    // TODO: Find a different solution since it's suboptimal to render in
-    // all clipping bounds.
-    select_all_clipbounds();
+    // select relevant clipping bounds
+    if (def->get_bound().is_null())   // can happen (spaces? to be investigated..)
+      select_all_clipbounds();
+    else
+      select_clipbounds(def, mat);
+    
+    
+    if (_clipbounds_selected.empty()) return; // nothing to draw
       
     // draw the shape
     if (m_drawing_mask)
@@ -929,7 +890,7 @@ public:
     rect ch_bounds = def->get_bound();
 
     if (ch_bounds.is_null()) {
-      log_msg("warning: select_clipbounds encountered a character definition "
+      log_debug("warning: select_clipbounds encountered a character definition "
         "with null bounds");
       return;
     }   
@@ -1010,7 +971,7 @@ public:
     if (m_drawing_mask) {
       
       // Shape is drawn inside a mask, skip sub-shapes handling and outlines
-      draw_mask_shape(paths, true);      
+      draw_mask_shape(paths, false);   // never use even-odd for masks
     
     } else {
     
@@ -1018,7 +979,7 @@ public:
       select_clipbounds(def, mat);
       
       if (_clipbounds_selected.empty()) {
-        log_msg("warning: AGG renderer skipping a whole character");
+        log_debug("warning: AGG renderer skipping a whole character");
         return; // nothing to draw!?
       }
     
@@ -1039,13 +1000,13 @@ public:
       for (unsigned int subshape=0; subshape<subshape_count; subshape++)
       {
         if (have_shape)
-	{
+        {
           draw_shape(subshape, paths, agg_paths, sh, true);    
-	}
+        }
         if (have_outline)      
-	{
+        {
           draw_outlines(subshape, paths, agg_paths_rounded, line_styles, cx, mat);
-	}
+        }
       }
       
     } // if not drawing mask
@@ -1105,28 +1066,28 @@ public:
     for (pno=0; pno<pcount; pno++) {
     
       path &the_path = paths_out[pno];     
-      point oldpnt(the_path.m_ax, the_path.m_ay);
+      point oldpnt(the_path.ap.x, the_path.ap.y);
       point newpnt;
       mat.transform(&newpnt, oldpnt);
-      the_path.m_ax = newpnt.m_x;    
-      the_path.m_ay = newpnt.m_y;
+      the_path.ap.x = newpnt.x;    
+      the_path.ap.y = newpnt.y;
       
       ecount = the_path.m_edges.size();
       for (eno=0; eno<ecount; eno++) {
       
         edge &the_edge = the_path.m_edges[eno];
         
-        oldpnt.m_x = the_edge.m_ax;
-        oldpnt.m_y = the_edge.m_ay;
+        oldpnt.x = the_edge.ap.x;
+        oldpnt.y = the_edge.ap.y;
         mat.transform(&newpnt, oldpnt);
-        the_edge.m_ax = newpnt.m_x;
-        the_edge.m_ay = newpnt.m_y;
+        the_edge.ap.x = newpnt.x;
+        the_edge.ap.y = newpnt.y;
         
-        oldpnt.m_x = the_edge.m_cx;
-        oldpnt.m_y = the_edge.m_cy;
+        oldpnt.x = the_edge.cp.x;
+        oldpnt.y = the_edge.cp.y;
         mat.transform(&newpnt, oldpnt);
-        the_edge.m_cx = newpnt.m_x;
-        the_edge.m_cy = newpnt.m_y;
+        the_edge.cp.x = newpnt.x;
+        the_edge.cp.y = newpnt.y;
       
       }          
       
@@ -1168,15 +1129,15 @@ public:
     
     size_t pcount = paths.size();
 
-    dest.resize(pcount);    
+    dest.resize(pcount);
     
     for (size_t pno=0; pno<pcount; pno++) {
       
       const gnash::path& this_path = paths[pno];
       agg::path_storage& new_path = dest[pno];
       
-      new_path.move_to(this_path.m_ax + subpixel_offset, 
-        this_path.m_ay + subpixel_offset);
+      new_path.move_to(this_path.ap.x + subpixel_offset, 
+        this_path.ap.y + subpixel_offset);
       
       size_t ecount = this_path.m_edges.size();
       
@@ -1185,13 +1146,13 @@ public:
         const edge& this_edge = this_path.m_edges[eno];
         
         if (this_edge.is_straight())
-          new_path.line_to(this_edge.m_ax + subpixel_offset, 
-            this_edge.m_ay + subpixel_offset);
+          new_path.line_to(this_edge.ap.x + subpixel_offset, 
+            this_edge.ap.y + subpixel_offset);
         else
-          new_path.curve3(this_edge.m_cx + subpixel_offset, 
-            this_edge.m_cy + subpixel_offset,
-            this_edge.m_ax + subpixel_offset, 
-            this_edge.m_ay + subpixel_offset);       
+          new_path.curve3(this_edge.cp.x + subpixel_offset, 
+            this_edge.cp.y + subpixel_offset,
+            this_edge.ap.x + subpixel_offset, 
+            this_edge.ap.y + subpixel_offset);       
         
       }    
     }    
@@ -1227,8 +1188,8 @@ public:
       const gnash::path& this_path = paths[pno];
       agg::path_storage& new_path = dest[pno];
       
-      float prev_ax = this_path.m_ax;
-      float prev_ay = this_path.m_ay;  
+      float prev_ax = this_path.ap.x;
+      float prev_ay = this_path.ap.y;  
       bool prev_align_x = true;
       bool prev_align_y = true;
       
@@ -1238,8 +1199,8 @@ public:
         
         const edge& this_edge = this_path.m_edges[eno];
         
-        float this_ax = this_edge.m_ax;  
-        float this_ay = this_edge.m_ay;  
+        float this_ax = this_edge.ap.x;  
+        float this_ay = this_edge.ap.y;  
         
         if (this_edge.is_straight()) {
         
@@ -1308,8 +1269,8 @@ public:
             new_path.move_to(prev_ax, prev_ay);
         
           // never align curves!
-          new_path.curve3(this_edge.m_cx + subpixel_offset, 
-            this_edge.m_cy + subpixel_offset,
+          new_path.curve3(this_edge.cp.x + subpixel_offset, 
+            this_edge.cp.y + subpixel_offset,
             this_ax + subpixel_offset, 
             this_ay + subpixel_offset);
             
@@ -1337,7 +1298,7 @@ public:
     const std::vector<line_style> &line_styles,
     const matrix& linestyle_matrix) {
     
-    assert(0); // should not be used currently
+    abort(); // should not be used currently
     
     size_t pcount=paths.size(); 
     dest.resize(pcount);
@@ -1415,6 +1376,18 @@ public:
           break;
         } 
 
+        case SWF::FILL_FOCAL_GRADIENT:
+        {
+          matrix m = fill_styles[fno].get_gradient_matrix();
+          matrix cm;
+          cm.set_inverse(fillstyle_matrix);
+          m.concatenate(cm);
+          m.concatenate(inv_stage_matrix);
+          
+          sh.add_gradient_focal(fill_styles[fno], m, cx);
+          break;
+        }
+
         case SWF::FILL_TILED_BITMAP:
         case SWF::FILL_CLIPPED_BITMAP:
         smooth=true;  // continue with next case!
@@ -1427,7 +1400,7 @@ public:
           cm.set_inverse(fillstyle_matrix);
           m.concatenate(cm);
           m.concatenate(inv_stage_matrix);
-          
+
           sh.add_bitmap(dynamic_cast<agg_bitmap_info_base*> 
             (fill_styles[fno].get_bitmap_info()), m, cx, 
             (fill_type==SWF::FILL_TILED_BITMAP) || (fill_type==SWF::FILL_TILED_BITMAP_HARD),
@@ -1512,7 +1485,7 @@ public:
     Thank to Maxim Shemanarev for providing us such a great tool with AGG...
     */
     
-    assert(m_pixf != NULL);
+    assert(m_pixf.get());
     
     assert(!m_drawing_mask);
     
@@ -1576,7 +1549,7 @@ public:
         rasc.add_path(curve);
       
       }
-      //log_msg("%d edges\n", edge_count);
+      //log_debug("%d edges\n", edge_count);
       
               
       agg::render_scanlines_compound_layered(rasc, sl, rbase, alloc, sh);
@@ -1632,10 +1605,6 @@ public:
     typedef agg_mask_style_handler sh_type;
     sh_type sh;                   
        
-    // anti-aliased scanline rasterizer
-    typedef agg::rasterizer_scanline_aa<> ras_type;
-    ras_type ras;
-    
     // compound rasterizer used for flash shapes
     typedef agg::rasterizer_compound_aa<agg::rasterizer_sl_clip_dbl> rasc_type;  
     rasc_type rasc;
@@ -1673,7 +1642,7 @@ public:
                   this_path.m_fill1==0 ? -1 : 0);
                   
       // starting point of path
-      path.move_to(this_path.m_ax, this_path.m_ay);
+      path.move_to(this_path.ap.x, this_path.ap.y);
     
       unsigned int ecount = this_path.m_edges.size();
       for (unsigned int eno=0; eno<ecount; eno++) {
@@ -1681,10 +1650,10 @@ public:
         const edge &this_edge = this_path.m_edges[eno];
 
         if (this_edge.is_straight())
-          path.line_to(this_edge.m_ax, this_edge.m_ay);
+          path.line_to(this_edge.ap.x, this_edge.ap.y);
         else
-          path.curve3(this_edge.m_cx, this_edge.m_cy,
-                      this_edge.m_ax, this_edge.m_ay);
+          path.curve3(this_edge.cp.x, this_edge.cp.y,
+                      this_edge.ap.x, this_edge.ap.y);
         
       } // for edge
       
@@ -1743,7 +1712,7 @@ public:
     const std::vector<line_style> &line_styles, const cxform& cx, 
     const matrix& linestyle_matrix, scanline_type& sl) {
     
-    assert(m_pixf != NULL);
+    assert(m_pixf.get());
     
     if (m_drawing_mask)    // Flash ignores lines in mask /definitions/
       return;    
@@ -1830,56 +1799,57 @@ public:
 
   
   /// Draws the given polygon.
-  void  draw_poly(const point* corners, size_t corner_count, const rgba& fill, 
-    const rgba& outline) {
+  template <class scanline_type>
+  void draw_poly_impl(const point* corners, size_t corner_count, const rgba& fill, 
+    const rgba& outline, scanline_type& sl, const matrix& poly_mat) {
     
-    assert(m_pixf != NULL);
+    assert(m_pixf.get());
 
     if (corner_count<1) return;
     
     if ( _clipbounds.size()==0 ) return;
     
     matrix mat = stage_matrix;
-    mat.concatenate(m_current_matrix);
+    mat.concatenate(poly_mat);
     
-    // TODO: Use aliased scanline renderer instead of anti-aliased one since
-    // it is undesired anyway.
     typedef agg::rasterizer_scanline_aa<> ras_type;
     renderer_base rbase(*m_pixf);
-    agg::scanline_p8 sl;
     ras_type ras;
     agg::renderer_scanline_aa_solid<
       agg::renderer_base<PixelFormat> > ren_sl(rbase);
-
+      
+    // -- create path --
+    agg::path_storage path;
+    point pnt, origin;
+    
+    
+    // Note: The coordinates are rounded and 0.5 is added to snap them to the 
+    // center of the pixel. This avoids blurring caused by anti-aliasing.
+    
+    mat.transform(&origin, 
+      point(trunc(corners[0].x), trunc(corners[0].y)));
+    path.move_to(trunc(origin.x)+0.5, trunc(origin.y)+0.5);
+    
+    for (unsigned int i=1; i<corner_count; i++) {
+    
+      mat.transform(&pnt, point(corners[i].x, corners[i].y));
+        
+      path.line_to(trunc(pnt.x)+0.5, trunc(pnt.y)+0.5);
+    }
+    
+    // close polygon
+    path.line_to(trunc(origin.x)+0.5, trunc(origin.y)+0.5);
+    
+    
+    
+    // -- render --
+      
+    // iterate through clipping bounds
     for (unsigned int cno=0; cno<_clipbounds.size(); cno++) {
     
-      const geometry::Range2d<int>& bounds = _clipbounds[cno];
-         
-      apply_clip_box<ras_type> (ras, bounds); 
-
-          // TODO: what do do if _clipbox.isNull() or _clipbox.isWorld() ?
-    //       currently an assertion will fail when get{Min,Max}{X,Y}
-    //       are called below
-  
-      agg::path_storage path;
-      point pnt, origin;
-      
-      // Note: The coordinates are rounded and 0.5 is added to snap them to the 
-      // center of the pixel. This avoids blurring caused by anti-aliasing.
-      
-      mat.transform(&origin, 
-        point(trunc(corners[0].m_x), trunc(corners[0].m_y)));
-      path.move_to(trunc(origin.m_x)+0.5, trunc(origin.m_y)+0.5);
-      
-      for (unsigned int i=1; i<corner_count; i++) {
-      
-        mat.transform(&pnt, point(corners[i].m_x, corners[i].m_y));
-          
-        path.line_to(trunc(pnt.m_x)+0.5, trunc(pnt.m_y)+0.5);
-      }
-      
-      // close polygon
-      path.line_to(trunc(origin.m_x)+0.5, trunc(origin.m_y)+0.5);
+      const geometry::Range2d<int>& bounds = _clipbounds[cno];         
+      apply_clip_box<ras_type> (ras, bounds);     
+            
       
       // fill polygon
       if (fill.m_a>0) {
@@ -1903,6 +1873,34 @@ public:
       }
     }
     
+  } //draw_poly_impl
+  
+  
+  void draw_poly(const point* corners, size_t corner_count, const rgba& fill, 
+    const rgba& outline, const matrix& mat, bool masked) {
+    
+    if (masked && !m_alpha_mask.empty()) {
+    
+      // apply mask
+      
+      typedef agg::scanline_u8_am<agg::alpha_mask_gray8> sl_type; 
+      
+      sl_type sl(m_alpha_mask.back()->get_amask());
+         
+      draw_poly_impl<sl_type> (corners, corner_count, fill, outline, sl, mat);       
+    
+    } else {
+    
+      // no mask
+      
+      typedef agg::scanline_p8 sl_type; // packed scanline (faster for solid fills)
+      
+      sl_type sl;
+         
+      draw_poly_impl<sl_type> (corners, corner_count, fill, outline, sl, mat);
+    
+    }
+    
   }
 
 
@@ -1918,8 +1916,8 @@ public:
     // and Intersect() against valid range instead.
     point p(world_x, world_y);
     stage_matrix.transform(p);
-    x = (int)p.m_x;
-    y = (int)p.m_y;
+    x = (int)p.x;
+    y = (int)p.y;
   }
 
   geometry::Range2d<int> world_to_pixel(const rect& wb)
@@ -2002,7 +2000,7 @@ public:
       
       count++;
     }
-    //log_msg("%d inv. bounds in frame", count);
+    //log_debug("%d inv. bounds in frame", count);
     
   }
   
@@ -2046,8 +2044,8 @@ public:
   }
 
   virtual void get_scale(point& scale) {
-    scale.m_x = PIXELS_TO_TWIPS(stage_matrix.get_x_scale());
-    scale.m_y = PIXELS_TO_TWIPS(stage_matrix.get_y_scale());
+    scale.x = PIXELS_TO_TWIPS(stage_matrix.get_x_scale());
+    scale.y = PIXELS_TO_TWIPS(stage_matrix.get_y_scale());
   }
   
   virtual unsigned int getBytesPerPixel() const {
@@ -2072,8 +2070,7 @@ private:  // private variables
 
   agg::rendering_buffer m_rbuf;  
 
-  // TODO: use an auto_ptr, since we're deleting in the destructor...
-  PixelFormat *m_pixf;
+  std::auto_ptr<PixelFormat> m_pixf;
   
   /// clipping rectangle
   std::vector< geometry::Range2d<int> > _clipbounds;
@@ -2088,15 +2085,35 @@ private:  // private variables
 
 
 
+// detect the endianess of the host (would prefer to NOT have this function
+// here)
+bool is_little_endian_host() {
+
+  union {
+    boost::uint16_t word;
+    struct {
+      boost::uint8_t b1;
+      boost::uint8_t b2;
+    } s;
+  } u;
+    
+  u.s.b1 = 1;
+  u.s.b2 = 2;
+  
+  return u.word == 0x0201;
+
+}
 
-// TODO: Replace "pixelformat" with a enum!
 
 DSOEXPORT render_handler_agg_base*  create_render_handler_agg(const char *pixelformat)
 {
 
   if (!pixelformat) return NULL;
 
-  log_msg("framebuffer pixel format is %s", pixelformat);
+  if (is_little_endian_host())
+    log_debug("framebuffer pixel format is %s (little-endian host)", pixelformat);
+  else
+    log_debug("framebuffer pixel format is %s (big-endian host)", pixelformat);
   
 #ifdef PIXELFORMAT_RGB555  
   if (!strcmp(pixelformat, "RGB555"))
@@ -2127,23 +2144,44 @@ DSOEXPORT render_handler_agg_base*  create_render_handler_agg(const char *pixelf
 #ifdef PIXELFORMAT_BGRA32  
   if (!strcmp(pixelformat, "BGRA32"))
     return new render_handler_agg<agg::pixfmt_bgra32_pre> (32);
+#endif   
+#ifdef PIXELFORMAT_RGBA32 
+  if (!strcmp(pixelformat, "ARGB32"))
+    return new render_handler_agg<agg::pixfmt_argb32_pre> (32);
+  else 
+#endif   
+#ifdef PIXELFORMAT_BGRA32  
+  if (!strcmp(pixelformat, "ABGR32"))
+    return new render_handler_agg<agg::pixfmt_abgr32_pre> (32);
         
   else 
 #endif
   {
     log_error("Unknown pixelformat: %s\n", pixelformat);
     return NULL;
-    //assert(0);
+    //abort();
   }
   
   return NULL; // avoid compiler warning
 }
 
-DSOEXPORT char *agg_detect_pixel_format(unsigned int rofs, unsigned int rsize,
+
+DSOEXPORT const char *agg_detect_pixel_format(unsigned int rofs, unsigned int rsize,
   unsigned int gofs, unsigned int gsize,
   unsigned int bofs, unsigned int bsize,
   unsigned int bpp) {
   
+  if (!is_little_endian_host()) {
+  
+    // Swap bits for big endian hosts, because the following tests assume
+    // little endians. The pixel format string matches the bytes in memory.
+    
+    rofs = bpp - rofs - rsize;
+    gofs = bpp - gofs - gsize;
+    bofs = bpp - bofs - bsize; 
+  
+  }
+  
   // 15 bits RGB (hicolor)
   if ((rofs==10) && (rsize==5)
    && (gofs==5) && (gsize==5)
@@ -2182,7 +2220,23 @@ DSOEXPORT char *agg_detect_pixel_format(unsigned int rofs, unsigned int rsize,
     else
       return "RGBA32";
       
-  }  
+  } else
+  // special 32 bits (mostly on big endian hosts)
+  if ((rofs==8) && (rsize==8)
+   && (gofs==16) && (gsize==8)
+   && (bofs==24) && (bsize==8)) {
+   
+   return "ARGB32";
+   
+  } else
+  // special 32 bits (mostly on big endian hosts)
+  if ((rofs==24) && (rsize==8)
+   && (gofs==16) && (gsize==8)
+   && (bofs==8) && (bsize==8)) {
+   
+   return "ABGR32";
+   
+  }
   
   return NULL; // unknown format
   
