@@ -1,9 +1,9 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -15,9 +15,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
 #ifndef BACKEND_RENDER_HANDLER_AGG_STYLE_H
 #define BACKEND_RENDER_HANDLER_AGG_STYLE_H
 
@@ -34,8 +31,6 @@
 //#define DEBUG_LIMIT_COLOR_ALPHA 
 
 
-using namespace gnash;
-
 namespace gnash {
 
 /// Internal style class that represents a fill style. Roughly speaking, AGG 
@@ -46,7 +41,7 @@ class agg_style_base
 {
 public:
   // for solid styles:
-  bool m_is_solid;
+  bool m_is_solid;   
   agg::rgba8 m_color; // defined here for easy access
   
   // for non-solid styles:
@@ -77,7 +72,7 @@ public:
 
   void generate_span(agg::rgba8* /*span*/, int /*x*/, int /*y*/, unsigned /*len*/)
   {
-    assert(0); // never call generate_span for solid fill styles
+    abort(); // never call generate_span for solid fill styles
   }
 };
 
@@ -115,10 +110,13 @@ template <class PixelFormat, class span_allocator_type, class img_source_type,
 class agg_style_bitmap : public agg_style_base
 {
 public:
+
+  bool m_force_premultiply;
     
-  agg_style_bitmap(int width, int height, int rowlen, uint8_t* data, 
+  agg_style_bitmap(int width, int height, int rowlen, boost::uint8_t* data, 
     gnash::matrix mat, gnash::cxform cx) :
     
+    m_force_premultiply(false),
     m_rbuf(data, width, height, rowlen),  
     m_pixf(m_rbuf),
     m_img_src(m_pixf),
@@ -148,14 +146,20 @@ public:
   void generate_span(agg::rgba8* span, int x, int y, unsigned len)
   {
     m_sg.generate(span, x, y, len);
-
     // Apply color transform
     // TODO: Check if this can be optimized
-    if (!m_cx.is_identity())      
-    for (unsigned int i=0; i<len; i++) {
-      m_cx.transform(span->r, span->g, span->b, span->a);
-      span->premultiply();
-      ++span;
+    if (!m_cx.is_identity()) {      
+      for (unsigned int i=0; i<len; i++) {
+        m_cx.transform(span->r, span->g, span->b, span->a);
+        span->premultiply();
+        ++span;
+      }
+    } else 
+    if (m_force_premultiply) {
+      for (unsigned int i=0; i<len; i++) { 
+        span->premultiply();
+        span++;
+      }    
     }
   }
     
@@ -247,7 +251,15 @@ public:
     m_sg.generate(span, x, y, len);
   }
   
-private:
+  // Provide access to our gradient adaptor to allow re-initialization of
+  // focal gradients. I wanted to do this using partial template specialization
+  // but it became too complex for something that can be solved in a very easy
+  // (slightly unelegant) way. 
+  gradient_adaptor_type& get_gradient_adaptor() {
+    return m_gradient_adaptor;
+  }
+  
+protected:
 
   
   // Color transform
@@ -277,6 +289,7 @@ private:
 
 
 
+
 // --- AGG HELPER CLASSES ------------------------------------------------------
 
 /// Style handler for AGG's compound rasterizer. This is the class which is
@@ -311,11 +324,11 @@ public:
     /// Adds a new bitmap fill style
     void add_bitmap(agg_bitmap_info_base* bi, gnash::matrix mat, gnash::cxform cx, 
       bool repeat, bool smooth) {
-      
+
       if (bi==NULL) {
-	// See server/styles.h comments about when NULL return is possible.
-	// Don't warn here, we already warn at parse-time
-        //log_msg("WARNING: add_bitmap called with bi=NULL");
+        // See server/styles.h comments about when NULL return is possible.
+        // Don't warn here, we already warn at parse-time
+        //log_debug("WARNING: add_bitmap called with bi=NULL");
         add_color(agg::rgba8_pre(0,0,0,0));
         return;
       }
@@ -338,7 +351,7 @@ public:
           if (bi->get_bpp()==32)
             add_bitmap_repeat_aa_rgba32 (bi, mat, cx);      
           else
-            assert(0);
+            abort();
             
         } else {
         
@@ -348,7 +361,7 @@ public:
           if (bi->get_bpp()==32)
             add_bitmap_repeat_nn_rgba32 (bi, mat, cx);      
           else
-            assert(0);
+            abort();
             
         } // if smooth
       } else {
@@ -360,7 +373,7 @@ public:
           if (bi->get_bpp()==32)
             add_bitmap_clip_aa_rgba32 (bi, mat, cx);      
           else
-            assert(0);
+            abort();
             
         } else {
         
@@ -370,7 +383,7 @@ public:
           if (bi->get_bpp()==32)
             add_bitmap_clip_nn_rgba32 (bi, mat, cx);      
           else
-            assert(0);
+            abort();
             
         } // if smooth
       } // if repeat
@@ -382,7 +395,7 @@ public:
     
 
     void add_bitmap_repeat_nn_rgb24(agg_bitmap_info_base* bi, gnash::matrix mat, gnash::cxform cx) {
-    
+
       // tiled, nearest neighbor method (faster)   
 
       typedef agg::pixfmt_rgb24_pre PixelFormat;
@@ -484,8 +497,8 @@ public:
         interpolator_type, sg_type> st_type;
       
       st_type* st = new st_type(bi->get_width(), bi->get_height(),
-          bi->get_rowlen(), bi->get_data(), mat, cx);       
-        
+          bi->get_rowlen(), bi->get_data(), mat, cx);
+          
       m_styles.push_back(st);
     }
         
@@ -506,8 +519,8 @@ public:
         interpolator_type, sg_type> st_type;
       
       st_type* st = new st_type(bi->get_width(), bi->get_height(),
-          bi->get_rowlen(), bi->get_data(), mat, cx);       
-        
+          bi->get_rowlen(), bi->get_data(), mat, cx);
+          
       m_styles.push_back(st);
     }
     
@@ -608,11 +621,40 @@ public:
       st_type* st = new st_type(fs, transl, cx, 64/2);  // div 2 because we need radius, not diameter     
         
       // NOTE: The value 64 is based on the bitmap texture used by other
-      // Gnash renderers which is normally 64x64 pixels for linear gradients.       
+      // Gnash renderers which is normally 64x64 pixels for radial gradients.       
         
       m_styles.push_back(st);
     }
+
+    void add_gradient_focal(const gnash::fill_style& fs, gnash::matrix mat, gnash::cxform cx)
+    {
+      typedef agg::rgba8 color_type;
+      typedef agg::span_allocator<color_type> span_allocator_type;
+      typedef agg::span_interpolator_linear<agg::trans_affine> interpolator_type;
+      typedef agg::gradient_radial_focus gradient_func_type;
+      typedef gradient_func_type gradient_adaptor_type;
+      typedef agg::gradient_lut<agg::color_interpolator<color_type>, 256> color_func_type;
+      typedef agg::span_gradient<color_type, interpolator_type,
+        gradient_adaptor_type, color_func_type> sg_type;
     
+      typedef agg_style_gradient<color_type, span_allocator_type,
+        interpolator_type, gradient_func_type, gradient_adaptor_type,
+        color_func_type, sg_type> st_type;
+            
+      // move the center of the focal fill (not it's focal point) to where it 
+      // should be.
+      gnash::matrix transl;      
+      transl.concatenate_translation(-32.0f, -32.0f);
+      transl.concatenate(mat);
+      
+      st_type* st = new st_type(fs, transl, cx, 64/2); 
+      
+      // re-initialize focal gradient settings
+      gradient_adaptor_type& adaptor = st->get_gradient_adaptor();
+      adaptor.init(32.0, fs.get_focal_point()*32.0, 0.0);
+    
+      m_styles.push_back(st);
+    }
 
     /// Returns the color of a certain fill style (solid)
     const agg::rgba8& color(unsigned style) const 
@@ -658,7 +700,7 @@ public:
   
   void generate_span(agg::gray8* /*span*/, int /*x*/, int /*y*/, int /*len*/, unsigned /*style*/)
   {
-    assert(0); // never call generate_span for solid fill styles
+    abort(); // never call generate_span for solid fill styles
   }
 
 private:
