@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include <map>
@@ -45,7 +45,8 @@ using namespace std;
 namespace gnash
 {
 
-static LogFile& dbglogfile = LogFile::getDefaultInstance();
+#define dbglogfile cerr		// FIXME: use log_*()
+
 #ifdef USE_DEBUGGER
 static Debugger& debugger = Debugger::getDefaultInstance();
 #endif
@@ -80,7 +81,7 @@ void dump_callbacks(map<string, as_value> &calls)
 }
 
 static void
-generic_callback(GtkWidget *widget, gpointer data)
+generic_callback(GtkWidget * /*widget*/, gpointer data)
 {
 //    GNASH_REPORT_FUNCTION;
 //    g_print ("Hello World - %d\n", *(int *)data;
@@ -105,9 +106,9 @@ generic_callback(GtkWidget *widget, gpointer data)
     
     as_value	val;
     as_environment env;
-    env.push_val(handler.to_object());
+    env.push(handler);
     env.push(event);
-    env.push(handler.to_string());
+    env.push(handler);
     as_object obj = val.to_object();
 
     // Call the AS function defined in the source file using this extension
@@ -119,14 +120,14 @@ attachInterface(as_object *obj)
 {
 //    GNASH_REPORT_FUNCTION;
 
-    obj->set_member("window_new", &gtkext_window_new);
-    obj->set_member("signal_connect", &gtkext_signal_connect);
-    obj->set_member("container_set_border_width", &gtkext_container_set_border_width);
-    obj->set_member("button_new_with_label", &gtkext_button_new_with_label);
-    obj->set_member("signal_connect_swapped", &gtkext_signal_connect_swapped);
-    obj->set_member("container_add", &gtkext_container_add);
-    obj->set_member("widget_show", &gtkext_widget_show);
-    obj->set_member("main", &gtkext_main);
+    obj->init_member("window_new", new builtin_function(gtkext_window_new));
+    obj->init_member("signal_connect", new builtin_function(gtkext_signal_connect));
+    obj->init_member("container_set_border_width", new builtin_function(gtkext_container_set_border_width));
+    obj->init_member("button_new_with_label", new builtin_function(gtkext_button_new_with_label));
+    obj->init_member("signal_connect_swapped", new builtin_function(gtkext_signal_connect_swapped));
+    obj->init_member("container_add", new builtin_function(gtkext_container_add));
+    obj->init_member("widget_show", new builtin_function(gtkext_widget_show));
+    obj->init_member("main", new builtin_function(gtkext_main));
 }
 
 static as_object*
@@ -141,7 +142,7 @@ getInterface()
 }
 
 static as_value
-gtkext_ctor(const fn_call& fn)
+gtkext_ctor(const fn_call& /*fn*/)
 {
 //    GNASH_REPORT_FUNCTION;
     GtkExt *obj = new GtkExt();
@@ -227,7 +228,7 @@ as_value gtkext_signal_connect(const fn_call& fn)
 	GtkExt *window = dynamic_cast<GtkExt *>(fn.arg(0).to_object().get());
 	string name = fn.arg(1).to_string();
 	as_value func = fn.arg(2).to_as_function();
-	int data = fn.arg(3).to_number();
+	//int data = fn.arg(3).to_int();
 
 	dbglogfile << "Adding callback " << func.to_string()
 		   << " for event \"" << name << "\"" << endl;
@@ -248,7 +249,7 @@ as_value gtkext_container_set_border_width(const fn_call& fn)
     
     if (fn.nargs > 0) {
 	GtkExt *window = dynamic_cast<GtkExt *>(fn.arg(0).to_object().get());
-	int width = fn.arg(1).to_number();
+	int width = fn.arg(1).to_int();
 	window->container_set_border_width(width);
 	dbglogfile << "set container border width to " << width << " !" << endl;
     }
@@ -328,7 +329,7 @@ as_value gtkext_widget_show(const fn_call& fn)
     return as_value();
 }
 
-// gtk_main taks no arguments.
+// gtk_main takes no arguments.
 as_value gtkext_main(const fn_call& fn)
 {
 //    GNASH_REPORT_FUNCTION;
@@ -358,12 +359,7 @@ extern "C" {
  	    attachInterface(cl.get());
 	}
 	
-	VM& vm = VM::get(); // cache this ?
-	std::string name = "GtkExt";
-	if ( vm.getSWFVersion() < 7 ) {
-	    boost::to_lower(name, vm.getLocale());
-	}
-	obj.init_member(name, cl.get());
+	obj.init_member("GtkExt", cl.get());
     }
 } // end of extern C
 
