@@ -1,6 +1,6 @@
 // Player.cpp:  Top level flash player, for gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 //
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #ifndef DEFAULT_GUI
@@ -41,6 +41,7 @@
 #include "GnashException.h"
 #include "noseek_fd_adapter.h"
 #include "VM.h"
+#include "SystemClock.h"
 
 #include "log.h"
 #include <iostream>
@@ -55,17 +56,15 @@ gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
 
 /*static private*/
 void
-Player::setFlashVars(gnash::sprite_instance& m, const std::string& varstr)
+Player::setFlashVars(const std::string& varstr)
 {
-	gnash::sprite_instance* si = m.get_root_movie();
-	assert(si);
-
-	typedef sprite_instance::VariableMap maptype;
+	typedef Gui::VariableMap maptype;
 
 	maptype vars;
 	URL::parse_querystring(varstr, vars);
 
-	si->setVariables(vars);
+	_gui->addFlashVars(vars);
+	//si->setVariables(vars);
 }
 
 Player::Player()
@@ -88,6 +87,7 @@ Player::Player()
 #ifdef GNASH_FPS_DEBUG
 	,_fpsDebugTime(0.0)
 #endif
+	,_hostfd(-1)
 {
 	init();
 }
@@ -116,8 +116,6 @@ void
 Player::init_logfile()
 {
     dbglogfile.setWriteDisk(false);
-//    rcfile.loadFiles();
-//    rcfile.dump();
 
     RcInitFile& rcfile = RcInitFile::getDefaultInstance();
     if (rcfile.useWriteLog()) {
@@ -147,15 +145,9 @@ Player::init_logfile()
     //
     if (!delay && rcfile.getTimerDelay() > 0) {
         delay = rcfile.getTimerDelay();
-        log_msg (_("Timer delay set to %d milliseconds"), delay);
+        log_debug (_("Timer delay set to %d milliseconds"), delay);
     }    
 
-    // Remove the logfile that's created by default, since leaving a short
-    // file is confusing.
-    if (dbglogfile.getWriteDisk() == false) {
-        dbglogfile.removeLog();
-    }
-
 }
 
 void
@@ -163,9 +155,9 @@ Player::init_sound()
 {
     if (do_sound) {
 #ifdef SOUND_SDL
-        _sound_handler.reset( gnash::create_sound_handler_sdl() );
+        _sound_handler.reset( gnash::media::create_sound_handler_sdl() );
 #elif defined(SOUND_GST)
-        _sound_handler.reset( gnash::create_sound_handler_gst() );
+        _sound_handler.reset( gnash::media::create_sound_handler_gst() );
 #else
         log_error(_("Sound requested but no sound support compiled in"));
         return;
@@ -209,6 +201,18 @@ Player::load_movie()
 {
 	gnash::movie_definition* md=NULL;
 
+	RcInitFile& rcfile = RcInitFile::getDefaultInstance();
+	URL vurl(_url);
+	//cout << "URL is " << vurl.str() << " (" << _url << ")" << endl;
+	if ( vurl.protocol() == "file" )
+	{
+		const std::string& path = vurl.path();
+		size_t lastSlash = path.find_last_of('/');
+		std::string dir = path.substr(0, lastSlash+1);
+		rcfile.addLocalSandboxPath(dir);
+		log_debug(_("%s appended to local sandboxes"), dir.c_str());
+	}
+
     try {
 	if ( _infile == "-" )
 	{
@@ -217,8 +221,23 @@ Player::load_movie()
 	}
 	else
 	{
+		URL url(_infile);
+		if ( url.protocol() == "file" )
+		{
+			std::string path = url.path();
+			// We'll need to allow load of the file, no matter virtual url
+			// specified...
+			// This is kind of hackish, cleaner would be adding an argument
+			// to create_library_movie to skip the security checking phase.
+			// NOTE that if we fail to allow this load, the konqueror plugin
+			// would not be able to load anything
+			//
+			rcfile.addLocalSandboxPath(path);
+			log_debug(_("%s appended to local sandboxes"), path.c_str());
+		}
+
 		// _url should be always set at this point...
-		md = gnash::create_library_movie(URL(_infile), _url.c_str(), false);
+		md = gnash::create_library_movie(url, _url.c_str(), false);
 	}
     } catch (const GnashException& er) {
 	fprintf(stderr, "%s\n", er.what());
@@ -239,10 +258,6 @@ int
 Player::run(int argc, char* argv[], const char* infile, const char* url)
 {
     
-	bool background = true;
-
-	assert(tu_types_validate());
-
 	// Call this at run() time, so the caller has
 	// a cache of setting some parameter before calling us...
 	// (example: setDoSound(), setWindowId() etc.. ) 
@@ -279,6 +294,31 @@ Player::run(int argc, char* argv[], const char* infile, const char* url)
 		return EXIT_FAILURE;
 	}
 
+    	// Parse querystring (before FlashVars, see testsuite/misc-ming.all/FlashVarsTest*)
+	setFlashVars(URL(_url).querystring());
+
+	// Parse parameters
+	StringNoCaseEqual noCaseCompare;
+	for ( map<string,string>::const_iterator it=params.begin(),
+		itEnd=params.end(); it != itEnd; ++it)
+	{
+		if ( noCaseCompare(it->first, "flashvars") )
+		{
+			setFlashVars(it->second);
+			continue;
+		}
+
+	    	if ( noCaseCompare(it->first, "base") )
+		{
+			setBaseUrl(it->second);
+			continue;
+		}
+
+		// too much noise...
+		//log_debug(_("Unused parameter %s = %s"),
+		//	it->first.c_str(), it->second.c_str());
+	}
+
 	// Set base url for this movie (needed before parsing)
 	gnash::set_base_url(URL(_baseurl));
 
@@ -304,40 +344,29 @@ Player::run(int argc, char* argv[], const char* infile, const char* url)
 
     if ( ! width || ! height )
     {
-        log_error(_("Input movie has collapsed dimensions " SIZET_FMT "/" SIZET_FMT ". Giving up."), width, height);
-	return EXIT_FAILURE;
+        //log_error(_("Input movie has collapsed dimensions " SIZET_FMT "/" SIZET_FMT ". Giving up."), width, height);
+        log_debug(_("Input movie has collapsed dimensions " SIZET_FMT "/" SIZET_FMT ". Setting to 1/1 and going on."), width, height);
+	if ( ! width ) width = 1;
+	if ( ! height ) height = 1;
+	//return EXIT_FAILURE;
     }
 
     // Now that we know about movie size, create gui window.
     _gui->createWindow(_url.c_str(), width, height);
 
-    movie_root& root = VM::init(*_movie_def).getRoot();
-    sprite_instance* m = root.get_root_movie();
+    SystemClock clock; // use system clock here...
+    movie_root& root = VM::init(*_movie_def, clock).getRoot();
+
+    // Set host requests fd (if any)
+    if ( _hostfd != -1 ) root.setHostFD(_hostfd);
+
+
+    _gui->setStage(&root);
 
     // Start loader thread
     _movie_def->completeLoad();
 
-    // Parse parameters
-    for ( map<string,string>::const_iterator it=params.begin(),
-		itEnd=params.end(); it != itEnd; ++it)
-    {
-	// todo: use a case-insensitive string type
-    	if ( it->first == "flashvars" || it->first == "FlashVars" )
-	{
-		setFlashVars(*m, it->second);
-		continue;
-	}
-
-	// too much noise...
-        //log_debug(_("Unused parameter %s = %s"),
-	//	it->first.c_str(), it->second.c_str());
-    }
-
-    // Parse querystring
-    setFlashVars(*m, URL(_url).querystring());
-
-    root.set_display_viewport(0, 0, width, height);
-    root.set_background_alpha(background ? 1.0f : 0.05f);
+    _gui->setMovieDefinition(_movie_def);
 
     if (!delay) {
       delay = (unsigned int) (1000 / movie_fps) ; // milliseconds per frame
@@ -363,7 +392,7 @@ void
 Player::fs_callback(gnash::sprite_instance* movie, const char* command, const char* args)
 // For handling notification callbacks from ActionScript.
 {
-    log_msg(_("fs_callback(%p): %s %s"), (void*)movie, command, args);
+    log_debug(_("fs_callback(%p): %s %s"), (void*)movie, command, args);
 }
 
 /* private */
