@@ -1,5 +1,5 @@
 //
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,14 +15,11 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-//
-//
-
 #ifndef __FBSUP_H__
 #define __FBSUP_H__
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include <vector>
@@ -47,9 +44,12 @@
 // emulated by the Kernel) 
 //#define USE_MOUSE_PS2
 
+// Define this to read from /dev/input/event0 (new generic input subsystem)
+#define USE_INPUT_EVENTS
+
 // Define this to support eTurboTouch / eGalax touchscreens. When reading from
 // a serial device, it must be initialized (stty) externally. 
-#define USE_MOUSE_ETT
+//#define USE_MOUSE_ETT
 
 #ifdef USE_MOUSE_PS2
 #define MOUSE_DEVICE "/dev/input/mice"
@@ -60,6 +60,11 @@
 #endif
 
 
+// Define this to request a new virtual terminal at startup. This doesn't always
+// work and probably is not necessary anyway
+//#define REQUEST_NEW_VT
+
+
 namespace gnash
 {
 
@@ -108,6 +113,9 @@ class FBGui : public Gui
 {
 	private:
 		int fd;
+		int original_vt;       // virtual terminal that was active at startup
+		int original_kd;       // keyboard mode at startup
+		int own_vt;            // virtual terminal we are running in   
 		unsigned char *fbmem;  // framebuffer memory
 #ifdef DOUBLE_BUFFER
 		unsigned char *buffer; // offscreen buffer
@@ -117,11 +125,16 @@ class FBGui : public Gui
 
     int m_stage_width;
     int m_stage_height;
+    unsigned m_rowsize;
 
   	int input_fd; /// file descriptor for /dev/input/mice
+  	int keyb_fd; /// file descriptor for /dev/input/event* (keyboard)
   	int mouse_x, mouse_y, mouse_btn;
   	unsigned char mouse_buf[256];
   	int mouse_buf_size;
+  	
+  	// Keyboard SHIFT/CTRL/ALT states (left + right)
+  	bool keyb_lshift, keyb_rshift, keyb_lctrl, keyb_rctrl, keyb_lalt, keyb_ralt;
 
     struct fb_var_screeninfo var_screeninfo;
   	struct fb_fix_screeninfo fix_screeninfo;
@@ -134,11 +147,15 @@ class FBGui : public Gui
   	
   	bool initialize_renderer();
   	
+  	/// Tries to find a accessible tty
+  	char* find_accessible_tty(int no);
+  	char* find_accessible_tty(const char* format, int no);
+  	
   	/// switches from text mode to graphics mode (disables the text terminal)
-  	void disable_terminal();
+  	bool disable_terminal();
   	
   	/// reverts disable_terminal() changes
-  	void enable_terminal();
+  	bool enable_terminal();
 
 #ifdef USE_MOUSE_PS2  	
   	/// Sends a command to the mouse and waits for the response
@@ -154,6 +171,20 @@ class FBGui : public Gui
   	/// Checks for any mouse activity
   	void check_mouse();
   	
+  	/// Initializes keyboard routines 
+  	bool init_keyboard();
+  	  	
+  	/// Translates a scancode from the Linux Input Subsystem to a Gnash key code 
+    gnash::key::code scancode_to_gnash_key(int code, bool shift);
+  	
+  	/// Checks for any keyboard activity
+  	void check_keyboard();
+  	
+#ifdef USE_INPUT_EVENTS  	
+    /// Applies builtin touchscreen calibration
+  	void apply_ts_calibration(float* cx, float* cy, int rawx, int rawy);
+#endif
+  	
   	int valid_x(int x);
   	int valid_y(int y);
   	  	
