@@ -1,6 +1,6 @@
 // gtk_glue_gtkglext.cpp:  Gnome ToolKit glue of some sort, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: gtk_glue_gtkglext.cpp,v 1.12 2007/07/01 10:54:02 bjacques Exp $ */
+/* $Id: gtk_glue_gtkglext.cpp,v 1.16 2008/02/19 19:20:49 bwy Exp $ */
 
 #include "gtk_glue_gtkglext.h"
 #include "log.h"
@@ -65,12 +65,13 @@ GtkGlExtGlue::init(int argc, char** argv[])
 
     gint major, minor;
     gdk_gl_query_version (&major, &minor);
-    log_msg (_("OpenGL extension version - %d.%d"),
+    log_debug (_("OpenGL extension version - %d.%d"),
               (int)major, (int)minor);
 
     GdkGLConfigMode glcmode = (GdkGLConfigMode)(GDK_GL_MODE_RGB |
-                                                GDK_GL_MODE_DEPTH |
-                                                GDK_GL_MODE_DOUBLE);
+                                                GDK_GL_MODE_STENCIL |
+                                                GDK_GL_MODE_DOUBLE |
+                                                GDK_GL_MODE_ACCUM);
     _glconfig = gdk_gl_config_new_by_mode (glcmode);
 
     if (!_glconfig) {
@@ -83,10 +84,10 @@ GtkGlExtGlue::init(int argc, char** argv[])
         log_error (_("No appropriate OpenGL-capable visual found."));
         gtk_main_quit(); // XXX
       } else {
-        log_msg (_("Got single-buffered visual."));
+        log_debug (_("Got single-buffered visual."));
       }
     } else {
-      log_msg (_("Got double-buffered visual."));
+      log_debug (_("Got double-buffered visual."));
     }
     
     return true;
