@@ -1,9 +1,9 @@
 //
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
@@ -15,10 +15,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-//
-//
-
-/* $Id: riscos_glue_agg.cpp,v 1.4 2007/05/28 15:41:00 ann Exp $ */
+/* $Id: riscos_glue_agg.cpp,v 1.8 2008/02/19 19:20:49 bwy Exp $ */
 
 #include <cstdio>
 #include <cerrno>
@@ -84,7 +81,7 @@ RiscosAggGlue::setRenderHandlerSize(int width, int height)
   if (width == _width && height == _height)
     return;
 
-  int new_bufsize = width*height*(_bpp/8);
+  int new_bufsize = width*height*((_bpp+7)/8);
 
   // TODO: At the moment we only increase the buffer and never decrease it.
   // Should be changed sometime.
@@ -94,12 +91,12 @@ RiscosAggGlue::setRenderHandlerSize(int width, int height)
     _offscreenbuf = static_cast<unsigned char *>( realloc(_offscreenbuf, new_bufsize) );
 
     if (!_offscreenbuf) {
-      log_msg("Could not allocate %i bytes for offscreen buffer: %s\n",
+      log_debug("Could not allocate %i bytes for offscreen buffer: %s\n",
                new_bufsize, strerror(errno) );
       return;
     }
 
-    log_msg("RISC OS-AGG: %i bytes offscreen buffer allocated\n", new_bufsize);
+    log_debug("RISC OS-AGG: %i bytes offscreen buffer allocated\n", new_bufsize);
 
     _offscreenbuf_size = new_bufsize;
     memset(_offscreenbuf, 0, new_bufsize);
@@ -115,7 +112,8 @@ RiscosAggGlue::setRenderHandlerSize(int width, int height)
          _offscreenbuf,
          _offscreenbuf_size,
          _width,
-         _height);
+         _height,
+         _width*((_bpp+7)/8));
 }
 
 void
