@@ -1,6 +1,6 @@
 // sdl_agg_glue.cpp:  Glue between SDL and Anti-Grain Geometry, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -32,8 +32,8 @@ namespace gnash
 SdlAggGlue::SdlAggGlue()
 	:
 _sdl_surface(NULL),
-_screen(NULL),
 _offscreenbuf(NULL),
+_screen(NULL),
 _agg_renderer(NULL)
 {
 //    GNASH_REPORT_FUNCTION;
@@ -48,7 +48,7 @@ SdlAggGlue::~SdlAggGlue()
 }
 
 bool
-SdlAggGlue::init(int argc, char** argv[])
+SdlAggGlue::init(int /*argc*/, char*** /*argv*/)
 {
 //    GNASH_REPORT_FUNCTION;
 
@@ -75,16 +75,16 @@ SdlAggGlue::createRenderHandler(int bpp)
         break;
       default:
         log_error (_("AGG's bit depth must be 16, 24 or 32 bits, not %d."), _bpp);
-        assert(0);
+        abort();
     }
     return _agg_renderer;
 }
 
 
 bool
-SdlAggGlue::prepDrawingArea(int width, int height, uint32_t sdl_flags)
+SdlAggGlue::prepDrawingArea(int width, int height, boost::uint32_t sdl_flags)
 {
-    int depth_bytes = _bpp / 8;
+    int depth_bytes = _bpp / 8;  // TODO: <Udo> is this correct? Gives 1 for 15 bit modes!
 
     assert(_bpp % 8 == 0);
 
@@ -97,7 +97,7 @@ SdlAggGlue::prepDrawingArea(int width, int height, uint32_t sdl_flags)
 
     int stride = width * depth_bytes;
 
-    uint32_t rmask, gmask, bmask, amask;
+    boost::uint32_t rmask, gmask, bmask, amask;
 
     switch(_bpp) {
       case 32: // RGBA32
@@ -119,7 +119,7 @@ SdlAggGlue::prepDrawingArea(int width, int height, uint32_t sdl_flags)
         amask = 0;
         break;
       default:
-        assert(0);
+        abort();
     }
 
 #define CHUNK_SIZE (100 * 100 * depth_bytes)
@@ -128,7 +128,7 @@ SdlAggGlue::prepDrawingArea(int width, int height, uint32_t sdl_flags)
 
     _offscreenbuf = new unsigned char[bufsize];
 
-    log_msg (_("SDL-AGG: %i byte offscreen buffer allocated"), bufsize);
+    log_debug (_("SDL-AGG: %i byte offscreen buffer allocated"), bufsize);
 
 
     // Only the AGG renderer has the function init_buffer, which is *not* part of
@@ -136,7 +136,8 @@ SdlAggGlue::prepDrawingArea(int width, int height, uint32_t sdl_flags)
     // address) during run-time.
     render_handler_agg_base * renderer =
       static_cast<render_handler_agg_base *>(_agg_renderer);
-    renderer->init_buffer(_offscreenbuf, bufsize, width, height);
+    renderer->init_buffer(_offscreenbuf, bufsize, width, height,
+      width*((_bpp+7)/8));
 
 
     _sdl_surface = SDL_CreateRGBSurfaceFrom((void *) _offscreenbuf, width, height,
