@@ -1,6 +1,6 @@
 // log.cpp:  Message logging functions, for gnash.
 //
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,10 +17,10 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: log.cpp,v 1.54.2.2 2007/08/20 16:39:33 strk Exp $ */
+/* $Id: log.cpp,v 1.63.2.1 2008/02/22 11:25:07 strk Exp $ */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include <cstdio>
@@ -29,6 +29,7 @@
 #include <fstream>
 #include <string>
 #include <cstring>
+//#include <boost/format.hpp>
 
 #if defined(_WIN32) && defined(WIN32)
 // Required for SYSTEMTIME definitions
@@ -39,24 +40,17 @@
 # include <unistd.h>
 #endif
 
-#include <libxml/xmlmemory.h>
-#include <libxml/parser.h>
-#include <libxml/xmlreader.h>
 #include <ctime>
-/* #define BOOST_HAS_THREADS 1
-   This should be set by boost itself.
-*/
-
 
 #include "log.h"
 
-using namespace std;
-
+using std::cout;
+using std::endl;
 
 namespace gnash {
 
 // static data to be shared amongst all classes.
-ofstream LogFile::_console;
+//ofstream LogFile::_console;
 int LogFile::_verbose = 0;
 bool LogFile::_actiondump = false;
 bool LogFile::_parserdump = false;
@@ -74,20 +68,56 @@ hexify(unsigned char *p, const unsigned char *s, int length, bool ascii) {
     // convert some characters so it'll look right in the log
     for (int i=0 ; i<length; i++) {
         // use the hex value
-	if (isprint(s[i]) && ascii) {
-	    if (i>1) {
-		if (!isprint(s[i-1])) {
-		    *p++ = ' ';
-		}
+
+	if (ascii) {
+	    if (isprint(s[i])) {
+		*p++ = s[i];
+		continue;
+	    } else {
+ 		if ((s[i] == 0xd) || (s[i] == 0xa)) {
+		    *p++ = s[i];
+ 		    continue;
+ 		}
+		*p++ = '^';
 	    }
-	    *p++ = s[i];
-	    if (!isprint(s[i+1])) {
-		*p++ = ' ';
-	    }
-	} else {
+	} else {		// if not ascii outout requested
 	    *p++ = hexchars[s[i] >> 4];
 	    *p++ = hexchars[s[i] & 0xf];
+	    *p++ = ' ';		// add a space between bytes
 	}
+	    
+// 	if (isascii(s[i]) && ascii) {
+// 	    *p++ = hexchars[s[i] >> 4];
+// 	    *p++ = hexchars[s[i] & 0xf];
+// //		    *p++ = ' ';
+// 	    *p++ = '%';
+// 	    continue;
+// 	}
+// 	*p++ = s[i];
+// // 	    if (!isprint(s[i+1])) {
+// // 		*p++ = hexchars[s[i] >> 4];
+// // 		*p++ = hexchars[s[i] & 0xf];
+// // //		*p++ = ' ';
+// // 		*p++ = '$';
+// // 	    }
+
+//     } else {
+// 	    if (ascii) {
+// 		if (s[i] == 0xd) {
+// //		    *p++ = '\r';
+// 		    *p++ = '@';
+// 		    continue;
+// 		}
+// 		if (s[i] == 0xa) {		
+// //		    *p++ = '\n';
+// 		    *p++ = '#';
+// 		    continue;
+// 		}
+// 	    } else {
+// 		*p++ = hexchars[s[i] >> 4];
+// 		*p++ = hexchars[s[i] & 0xf];
+// 	    }
+// 	}
     }
 
     *p = '\0';
@@ -96,8 +126,8 @@ hexify(unsigned char *p, const unsigned char *s, int length, bool ascii) {
 }
 
 // FIXME: localize these, so they print local regional timestamps.
-ostream&
-timestamp(ostream& x) {
+std::ostream&
+timestamp(std::ostream& x) {
     time_t t;
     char buf[10];
 
@@ -109,7 +139,7 @@ timestamp(ostream& x) {
 }
 
 
-string
+std::string
 timestamp() {
 
     time_t t;
@@ -122,11 +152,9 @@ timestamp() {
     std::stringstream ss;
     ss << getpid() << "] " << buf;
     return ss.str();
-    //string sbuf = buf;
-    //return sbuf;
 }
 
-ostream& datetimestamp(ostream& x) {
+std::ostream& datetimestamp(std::ostream& x) {
     time_t t;
     char buf[20];
 
@@ -154,22 +182,6 @@ namespace {
     LogFile& dbglogfile = LogFile::getDefaultInstance();
 }
 
-// Printf-style log for debugging info.
-void
-log_msg(const char* fmt, ...)
-{
-    va_list ap;
-    char tmp[BUFFER_SIZE];
-
-    va_start (ap, fmt);
-    vsnprintf (tmp, BUFFER_SIZE, fmt, ap);
-    tmp[BUFFER_SIZE-1] = '\0';
-
-    dbglogfile.log(tmp);
-
-    va_end (ap);
-}
-
 void
 log_trace(const char* fmt, ...)
 {
@@ -186,10 +198,18 @@ log_trace(const char* fmt, ...)
     va_end (ap);
 }
 
+//void
+//logTrace(const boost::format& fmt)
+//{
+//    dbglogfile.log(N_("TRACE"), fmt.str());
+//}
+
 void
 log_debug(const char* fmt, ...)
 {
 
+    if (dbglogfile.getVerbosity() < DEBUGLEVEL) return;
+
     va_list ap;
     char tmp[BUFFER_SIZE];
 
@@ -207,6 +227,13 @@ log_debug(const char* fmt, ...)
     va_end (ap);
 }
 
+//void
+//logDebug(const boost::format& fmt)
+//{
+//    if (dbglogfile.getVerbosity() < DEBUGLEVEL) return;
+//    dbglogfile.log(N_("DEBUG"), fmt.str());
+//}
+
 void
 log_action(const char* fmt, ...)
 {
@@ -223,6 +250,15 @@ log_action(const char* fmt, ...)
     dbglogfile.setStamp(stamp);
 }
 
+//void
+//logAction(const boost::format& fmt)
+//{
+//    bool stamp = dbglogfile.getStamp();
+//    dbglogfile.setStamp(false);
+//    dbglogfile.log(fmt.str());
+//    dbglogfile.setStamp(stamp);
+//}
+
 void
 log_parse(const char* fmt, ...)
 {
@@ -239,6 +275,12 @@ log_parse(const char* fmt, ...)
     va_end (ap);
 }
 
+//void
+//logParse(const boost::format& fmt)
+//{
+//    dbglogfile.log(fmt.str());
+//}
+
 // Printf-style error log.
 void
 log_error(const char* fmt, ...)
@@ -255,6 +297,12 @@ log_error(const char* fmt, ...)
     va_end (ap);
 }
 
+//void
+//logError(const boost::format& fmt)
+//{
+//    dbglogfile.log(N_("ERROR"), fmt.str());
+//}
+
 void
 log_unimpl(const char* fmt, ...)
 {
@@ -265,11 +313,17 @@ log_unimpl(const char* fmt, ...)
     vsnprintf (tmp, BUFFER_SIZE-1, fmt, ap);
     tmp[BUFFER_SIZE-1] = '\0';
 
-    dbglogfile.log(_("ERROR: Unimplemented"), tmp);
+    dbglogfile.log(_("UNIMPLEMENTED"), tmp);
 
     va_end (ap);
 }
 
+//void
+//logUnimpl(const boost::format& fmt)
+//{
+//    dbglogfile.log(N_("UNIMPLEMENTED"), fmt.str());
+//}
+
 void
 log_security(const char* fmt, ...)
 {
@@ -285,6 +339,12 @@ log_security(const char* fmt, ...)
     va_end (ap);
 }
 
+//void
+//logSecurity(const boost::format& fmt)
+//{
+//    dbglogfile.log(N_("SECURITY"), fmt.str());
+//}
+
 void
 log_swferror(const char* fmt, ...)
 {
@@ -300,6 +360,12 @@ log_swferror(const char* fmt, ...)
     va_end (ap);
 }
 
+//void
+//logSWFError(const boost::format& fmt)
+//{
+//    dbglogfile.log(N_("MALFORMED SWF"), fmt.str());
+//}
+
 void
 log_aserror(const char* fmt, ...)
 {
@@ -315,14 +381,14 @@ log_aserror(const char* fmt, ...)
     va_end (ap);
 }
 
-const char *
-LogFile::getEntry(void)
-{
-    return _logentry.c_str();
-}
+//void
+//logASError(const boost::format& fmt)
+//{
+//    dbglogfile.log(N_("ACTIONSCRIPT ERROR"), fmt.str());
+//}
 
 void
-LogFile::log(const char* msg)
+LogFile::log(const std::string& msg)
 {
     boost::mutex::scoped_lock lock(_ioMutex);
 
@@ -331,7 +397,7 @@ LogFile::log(const char* msg)
 }
 
 void
-LogFile::log(const char* label, const char* msg)
+LogFile::log(const std::string& label, const std::string& msg)
 {
     boost::mutex::scoped_lock lock(_ioMutex);
 
@@ -340,67 +406,58 @@ LogFile::log(const char* label, const char* msg)
 }
 
 // Default constructor
-LogFile::LogFile (void)
+LogFile::LogFile ()
 	:
 	_state(CLOSED),
 	_stamp(true),
-	_write(true),
-	_trace(false)
+	_write(false)
 {
-    string loadfile;
-
-// Flip this ifdef to have the default files be stored in /tmp instead
-// of in the users home directory.
-#if 0
-    char *home = getenv("HOME");
-    if (home) {
-	loadfile = home;
-	loadfile += DEFAULT_LOGFILE;
-	return parse_file(loadfile);
-    }
-#else
-  // Flip this ifdef to have the default files go to /tmp, instead of
-  // the current directory. For the plugin, this creates the debug
-  // file in the users home directory when running from a desktop menu.
-# if 0
-    loadfile = "/tmp/";
-    loadfile += DEFAULT_LOGFILE;
-# else
-
     RcInitFile& rcfile = RcInitFile::getDefaultInstance();
-    loadfile = rcfile.getDebugLog();
-    if ( loadfile.empty() ) loadfile = DEFAULT_LOGFILE;
+    _write = rcfile.useWriteLog();
 
-# endif
-#endif
-
-    // TODO: expand ~ to getenv("HOME") !!
-
-    openLog(loadfile);
 }
 
-LogFile::LogFile (const char *filespec)
-	:
-	_state(CLOSED),
-	_stamp(true),
-	_write(true)
+LogFile::~LogFile()
 {
-    openLog(filespec);
+	if (_state == OPEN) closeLog();
 }
 
 bool
-LogFile::openLog (const char *filespec)
+LogFile::openLogIfNeeded ()
 {
-    boost::mutex::scoped_lock lock(_ioMutex);
-    if (_state == OPEN) {
-	_outstream.close ();
-	_state = CLOSED;
+    if (_state != CLOSED) return true;
+    if (!_write) return false;
+
+    RcInitFile& rcfile = RcInitFile::getDefaultInstance();
+
+    std::string loadfile = rcfile.getDebugLog();
+    if ( loadfile.empty() ) loadfile = DEFAULT_LOGFILE;
+
+    // TODO: expand ~ to getenv("HOME") !!
+
+    return openLog(loadfile);
+}
+
+bool
+LogFile::openLog (const std::string& filespec)
+{
+    // NOTE:
+    // don't need to lock the mutex here, as this method
+    // is intended to be called only by openLogIfNeeded,
+    // which in turn is called by operator<< which is called
+    // by the public log_xxx functions that log themselves
+
+    if (_state != CLOSED) {
+	cout << "Closing previously opened stream" << endl;
+        _outstream.close ();
+        _state = CLOSED;
     }
 
-    _outstream.open (filespec, ios::out);
-    if( ! _outstream ) {
+    // Append, don't truncate, the log file
+    _outstream.open (filespec.c_str(), std::ios::app|std::ios::out); // ios::out
+    if( _outstream.fail() ) {
 	// Can't use log_error here...
-        std::cerr << "ERROR: can't open debug log file " << filespec << " for writing." << std::endl;
+        cout << "ERROR: can't open debug log file " << filespec << " for appending." << endl;
         return false;
     }       
 
@@ -417,7 +474,7 @@ LogFile::closeLog (void)
 {
     boost::mutex::scoped_lock lock(_ioMutex);
     if (_state == OPEN) {
-	_outstream.flush();
+           _outstream.flush();
         _outstream.close();
     }
     _state = CLOSED;
@@ -429,253 +486,43 @@ bool
 LogFile::removeLog (void)
 {
     if (_state == OPEN) {
-	_outstream.close ();
+        _outstream.close ();
     }
 
     // Ignore the error, we don't care
     unlink(_filespec.c_str());
     _filespec.clear();
-    _logentry.clear();
 
     return true;
 }
 
-/// \brief print a char
-LogFile&
-LogFile::operator << (char x)
-{
-    if (_verbose) {
-	cout << x;
-    }
+/// \brief print a string
+///
 
-    if (_write) {
-	_outstream << x;
-    }
 
-    _state = INPROGRESS;
-
-  return *this;
-}
-
-/// \brief print a long
-LogFile&
-LogFile::operator << (long x)
-{
-    if (_verbose) {
-	cout << x;
-    }
-
-    if (_write) {
-	_outstream << x;
-    }
-
-    _state = INPROGRESS;
-
-  return *this;
-}
-
-/// \brief print an unsigned integer
-LogFile&
-LogFile::operator << (unsigned int x)
-{
-    if (_verbose) {
-	cout << x;
-    }
-
-    if (_write) {
-	_outstream << x;
-    }
-
-    _state = INPROGRESS;
-
-    return *this;
-}
-
-/// \brief print an unsigned long
-LogFile&
-LogFile::operator << (unsigned long x)
-{
-    if (_verbose) {
-	cout << x;
-    }
-
-    if (_write) {
-	_outstream << x;
-    }
-
-    _state = INPROGRESS;
-
-    return *this;
-}
-
-/// \brief print a float
-LogFile&
-LogFile::operator << (float x)
-{
-    if (_verbose > 0) {
-	cout << x;
-    }
-
-    if (_write) {
-	_outstream << x;
-    }
-
-    _state = INPROGRESS;
-
-  return *this;
-}
-
-/// \brief print a double
-LogFile&
-LogFile::operator << (double &x)
-{
-    if (_verbose) {
-	cout << x;
-    }
-
-    if (_write) {
-	_outstream << x;
-    }
-    _state = INPROGRESS;
-
-    return *this;
-}
-
-/// \brief print an integer
-LogFile&
-LogFile::operator << (int x)
-{
-
-    if (_verbose) {
-	cout << x;
-    }
-
-    if (_write) {
-	_outstream << x;
-    }
-
-    _state = INPROGRESS;
-
-    return *this;
-}
-
-/// \brief print a pointer
-LogFile&
-LogFile::operator << (void *ptr)
-{
-    if (_verbose) {
-	cout << ptr;
-    }
-
-    if (_write) {
-	_outstream << ptr;
-    }
-
-    _state = INPROGRESS;
-
-    return *this;
-}
-
-/// \brief print an STL string
 LogFile&
 LogFile::operator << (const std::string &s)
 {
-    if (_verbose) {
-	cout << s;
-    }
+    // NOTE: _state will be == INPROGRESS right
 
-    if (_write) {
-	_outstream << s;
-    }
+    if (_stamp == true && (_state != INPROGRESS) )
+    {
+	std::string ts = timestamp();
 
-    _state = INPROGRESS;
-
-    return *this;
-}
-
-/// \brief print a const char * string
-///
-/// This function is more complex than the other logging functions
-/// because it has hooks to look for the TRACE: keyword for logging
-/// function calls. We always want these to be logged to the disk
-/// file, but optionally printed to the terminal window.
-///
-/// Since the tracing functions use varargs, they always become a
-/// const char * string after processing the arguments. That means we
-/// can look for the keyword to determine what to do.
-LogFile&
-LogFile::operator << (const char *str)
-{
-    string c(str);
-
-    _logentry = timestamp();
-    _logentry += ": ";
-
-    // See if we have the TRACE keyword
-    if (strstr(str, N_("DEBUG: "))) {
-	_trace = true;
-    }
-
-    if (_stamp == true && (_state == IDLE || _state == OPEN)) {
-	_state = INPROGRESS;
-	if (_trace) {
-	    if (_verbose >= TRACELEVEL) {
-		cout << _logentry  << c;
-	    }
-	} else {
-	    if (_verbose) {
-		cout << _logentry  << c;
-	    }
+        if (_verbose) cout << ts << ": " << s;
+        if (openLogIfNeeded())
+        {
+		_outstream << ts << ": " << s;
 	}
-	if (_write) {
-	    _outstream << _logentry << c;
-	}
-    } else {
-	if (_trace) {
-	    if (_verbose >= TRACELEVEL) {
-		cout << c;
-	    }
-	} else {
-	    if (_verbose) {
-		cout << c;
-	    }
-	}
-	if (_write) {
-	    _outstream << c;
+        _state = INPROGRESS;
+    }
+    else
+    {
+        if (_verbose) cout << s;
+        if (openLogIfNeeded()) {
+		_outstream << s;   
 	}
     }
-    _logentry += c;
-
-    return *this;
-}
-
-LogFile&
-LogFile::operator << (unsigned char const *c)
-{
-    _logentry = timestamp();
-    _logentry += ": ";
-
-    if (c == -0) {
-      return *this;
-    }
-
-    if (_stamp == true && (_state == IDLE || _state == OPEN)) {
-	_state = INPROGRESS;
-	if (_verbose) {
-	    cout << _logentry  << c;
-	}
-	if (_write) {
-	    _outstream << _logentry << c;
-	}
-    } else {
-	if (_verbose) {
-	    cout << c;
-	}
-	if (_write) {
-	    _outstream << c;
-	}
-    }
-    _logentry += (const char*)c;
 
     return *this;
 }
@@ -684,23 +531,16 @@ LogFile::operator << (unsigned char const *c)
 LogFile&
 LogFile::operator << (std::ostream & (&)(std::ostream &))
 {
-    if (_trace) {
-	if (_verbose) {
-	    cout << endl;
-	}
-    } else {
-	if (_verbose) {
-	    cout << endl;
-	}
-    }
 
-    if (_write) {
-	_outstream << endl;;
-	_outstream.flush();
+    if (_verbose) cout << endl;
+
+    if (openLogIfNeeded())
+    {
+        _outstream << endl;;
+        _outstream.flush();
     }
 
     _state = IDLE;
-    _trace = false;
 
     return *this;
 }
