@@ -1,6 +1,6 @@
 // network.cpp:  TCP/IP support, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 //
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "utility.h"
@@ -30,7 +30,7 @@
 #include <cstdio>
 #include <cerrno>
 #include <fcntl.h>
-#ifdef HAVE_WINSOCK_H
+#if defined(HAVE_WINSOCK_H) && !defined(__OS2__)
 # include <winsock2.h>
 # include <windows.h>
 # include <sys/stat.h>
@@ -43,6 +43,7 @@
 # include <netinet/in.h>
 # include <arpa/inet.h>
 # include <sys/socket.h>
+# include <sys/un.h>
 # include <netdb.h>
 # include <sys/param.h>
 # include <sys/select.h>
@@ -74,7 +75,7 @@ Network::Network()
 	_timeout(5)
 {
     //GNASH_REPORT_FUNCTION;
-#ifdef HAVE_WINSOCK_H
+#if defined(HAVE_WINSOCK_H) && !defined(__OS2__)
     WORD wVersionRequested;
     WSADATA wsaData;
     wVersionRequested = MAKEWORD(1, 1);		// Windows Sockets 1.1
@@ -89,7 +90,7 @@ Network::Network()
 Network::~Network()
 {
     //GNASH_REPORT_FUNCTION;
-#ifdef HAVE_WINSOCK_H
+#if defined(HAVE_WINSOCK_H) && !defined(__OS2__)
     WSACleanup();
 #else
     closeNet();
@@ -187,7 +188,7 @@ Network::createServer(short port)
 //		char  ascip[INET_ADDRSTRLEN];
 //		inet_ntop(sock_in.sin_family, &_ipaddr, ascip, INET_ADDRSTRLEN);
 		char *ascip = ::inet_ntoa(sock_in.sin_addr);
-		log_msg(_("Server bound to service on %s, port %hd, using fd %d"),
+		log_debug(_("Server bound to service on %s, port %hd, using fd %d"),
 		    ascip, ntohs(sock_in.sin_port),
 		    _listenfd);
 	}
@@ -229,9 +230,9 @@ Network::newConnection(bool block)
 
     alen = sizeof(struct sockaddr_in);
 
-#ifdef NET_DEBUG
-    log_msg(_("Trying to accept net traffic on fd %d"), _sockfd);
-#endif
+    if (_debug) {
+	log_debug(_("Trying to accept net traffic on fd %d"), _sockfd);
+    }
 
     if (_listenfd <= 2) {
         return false;
@@ -259,24 +260,26 @@ Network::newConnection(bool block)
         }
 
         if (FD_ISSET(0, &fdset)) {
-            log_msg(_("There is data at the console for stdin"));
+	    if (_debug) {
+		log_debug(_("There is data at the console for stdin"));
+	    }
             return true;
         }
 
         // If interupted by a system call, try again
         if (ret == -1 && errno == EINTR) {
-            log_msg(_("The accept() socket for fd %d was interupted by a system call"), _listenfd);
+            log_debug(_("The accept() socket for fd %d was interupted by a system call"), _listenfd);
         }
 
         if (ret == -1) {
-            log_msg(_("The accept() socket for fd %d never was available for writing"),
+            log_debug(_("The accept() socket for fd %d never was available for writing"),
                     _listenfd);
             return false;
         }
 
         if (ret == 0) {
             if (_debug) {
-                log_msg(_("The accept() socket for fd %d timed out waiting to write"),
+                log_debug(_("The accept() socket for fd %d timed out waiting to write"),
                         _listenfd);
             }
         }
@@ -293,24 +296,116 @@ Network::newConnection(bool block)
     }
 
     if (_debug) {
-	log_msg(_("Accepting tcp/ip connection on fd %d"), _sockfd);
+	log_debug(_("Accepting tcp/ip connection on fd %d"), _sockfd);
     }
 
     return true;
 }
 
+// Connect to a named pipe
+bool
+Network::connectSocket(const char *sockname)
+{
+//    GNASH_REPORT_FUNCTION;
+
+    struct sockaddr_un  addr;
+    fd_set              fdset;
+    struct timeval      tval;
+    int                 ret;
+    int                 retries;
+
+    addr.sun_family = AF_UNIX;
+    // socket names must be 108 bytes or less as specifiec in sys/un.h.
+    strncpy(addr.sun_path, sockname, 100);
+
+    _sockfd = ::socket(AF_UNIX, SOCK_STREAM, 0);
+    if (_sockfd < 0)
+        {
+            log_error(_("unable to create socket: %s"), strerror(errno));
+            _sockfd = -1;
+            return false;
+        }
+
+    retries = 2;
+    while (retries-- > 0) {
+        // We use select to wait for the read file descriptor to be
+        // active, which means there is a client waiting to connect.
+        FD_ZERO(&fdset);
+        FD_SET(_sockfd, &fdset);
+
+        // Reset the timeout value, since select modifies it on return. To
+        // block, set the timeout to zero.
+        tval.tv_sec = 5;
+        tval.tv_usec = 0;
+
+        ret = ::select(_sockfd+1, &fdset, NULL, NULL, &tval);
+
+        // If interupted by a system call, try again
+        if (ret == -1 && errno == EINTR)
+            {
+                log_debug(_("The connect() socket for fd %d was interupted by a system call"),
+                        _sockfd);
+                continue;
+            }
+
+        if (ret == -1)
+            {
+                log_debug(_("The connect() socket for fd %d never was available for writing"),
+                        _sockfd);
+#ifdef HAVE_WINSOCK_H
+                ::shutdown(_sockfd, 0); // FIXME: was SHUT_BOTH
+#else
+                ::shutdown(_sockfd, SHUT_RDWR);
+#endif
+                _sockfd = -1;
+                return false;
+            }
+        if (ret == 0) {
+            log_error(_("The connect() socket for fd %d timed out waiting to write"),
+                      _sockfd);
+            continue;
+        }
+
+        if (ret > 0) {
+            ret = ::connect(_sockfd, reinterpret_cast<struct sockaddr *>(&addr), sizeof(addr));
+            if (ret == 0) {
+                log_debug(_("\tsocket name %s for fd %d"), sockname, _sockfd);
+                _connected = true;
+                assert(_sockfd > 0);
+                return true;
+            }
+            if (ret == -1) {
+                log_error(_("The connect() socket for fd %d never was available for writing"),
+                        _sockfd);
+                _sockfd = -1;
+                assert(!_connected);
+                return false;
+            }
+        }
+    }
+    
+
+#ifndef HAVE_WINSOCK_H
+    fcntl(_sockfd, F_SETFL, O_NONBLOCK);
+#endif
+
+    _connected = true;
+    assert(_sockfd > 0);
+    return true;    
+}
+
 // Create a client connection to a tcp/ip based service
 bool
 Network::createClient(void)
 {
-    GNASH_REPORT_FUNCTION;
+//    GNASH_REPORT_FUNCTION;
 
     return createClient("localhost", RTMP);
 }
 bool
 Network::createClient(short /* port */)
 {
-    GNASH_REPORT_FUNCTION;
+//    GNASH_REPORT_FUNCTION;
 
     return false;
 }
@@ -318,7 +413,7 @@ Network::createClient(short /* port */)
 bool
 Network::createClient(const char *hostname)
 {
-    GNASH_REPORT_FUNCTION;
+//    GNASH_REPORT_FUNCTION;
 
     return createClient(hostname, RTMP);
 }
@@ -326,7 +421,7 @@ Network::createClient(const char *hostname)
 bool
 Network::createClient(const char *hostname, short port)
 {
-    GNASH_REPORT_FUNCTION;
+//    GNASH_REPORT_FUNCTION;
 
     struct sockaddr_in  sock_in;
     fd_set              fdset;
@@ -344,15 +439,15 @@ Network::createClient(const char *hostname, short port)
         return false;
     }
 
-    log_msg(_("%s: to host %s at port %d"), __FUNCTION__, hostname, port);
+    log_debug(_("%s: to host %s at port %d"), __FUNCTION__, hostname, port);
 
     memset(&sock_in, 0, sizeof(struct sockaddr_in));
     memset(&thishostname, 0, MAXHOSTNAMELEN);
     if (strlen(hostname) == 0) {
         if (gethostname(thishostname, MAXHOSTNAMELEN) == 0) {
-            log_msg(_("The hostname for this machine is %s"), thishostname);
+            log_debug(_("The hostname for this machine is %s"), thishostname);
         } else {
-            log_msg(_("Couldn't get the hostname for this machine"));
+            log_debug(_("Couldn't get the hostname for this machine"));
             return false;
         }
     }
@@ -366,7 +461,7 @@ Network::createClient(const char *hostname, short port)
 #if 0
     char ascip[INET_ADDRSTRLEN];
     inet_ntop(sock_in.sin_family, &sock_in.sin_addr.s_addr, ascip, INET_ADDRSTRLEN);
-    log_msg(_("The IP address for this client socket is %s"), ascip);
+    log_debug(_("The IP address for this client socket is %s"), ascip);
 #endif
 
     proto = ::getprotobyname("TCP");
@@ -396,14 +491,14 @@ Network::createClient(const char *hostname, short port)
         // If interupted by a system call, try again
         if (ret == -1 && errno == EINTR)
             {
-                log_msg(_("The connect() socket for fd %d was interupted by a system call"),
+                log_debug(_("The connect() socket for fd %d was interupted by a system call"),
                         _sockfd);
                 continue;
             }
 
         if (ret == -1)
             {
-                log_msg(_("The connect() socket for fd %d never was available for writing"),
+                log_debug(_("The connect() socket for fd %d never was available for writing"),
                         _sockfd);
 #ifdef HAVE_WINSOCK_H
                 ::shutdown(_sockfd, 0); // FIXME: was SHUT_BOTH
@@ -425,7 +520,7 @@ Network::createClient(const char *hostname, short port)
 		char *ascip = ::inet_ntoa(sock_in.sin_addr);
 // 		char ascip[INET_ADDRSTRLEN];
 // 		inet_ntop(sock_in.sin_family, &sock_in.sin_addr.s_addr, ascip, INET_ADDRSTRLEN);
-                log_msg(_("\tport %d at IP %s for fd %d"), port,
+                log_debug(_("\tport %d at IP %s for fd %d"), port,
                         ascip, _sockfd);
                 _connected = true;
                 assert(_sockfd > 0);
@@ -500,7 +595,7 @@ Network::closeNet(int sockfd)
                 }
             }
 #endif
-            if (close(sockfd) < 0) {
+            if (::close(sockfd) < 0) {
                 log_error(_("Unable to close the socket for fd %d: %s"),
                         sockfd, strerror(errno));
 #ifndef HAVE_WINSOCK_H
@@ -508,7 +603,7 @@ Network::closeNet(int sockfd)
 #endif
                 retries++;
             } else {
-                log_msg(_("Closed the socket on fd %d"), sockfd);
+		log_debug(_("Closed the socket on fd %d"), sockfd);
                 return true;
             }
         }
@@ -519,7 +614,7 @@ Network::closeNet(int sockfd)
 bool
 Network::closeConnection(void)
 {
-    GNASH_REPORT_FUNCTION;
+//    GNASH_REPORT_FUNCTION;
 
     closeConnection(_sockfd);
     _sockfd = 0;
@@ -532,10 +627,11 @@ Network::closeConnection(void)
 bool
 Network::closeConnection(int fd)
 {
-    GNASH_REPORT_FUNCTION;
+//    GNASH_REPORT_FUNCTION;
 
     if (fd > 0) {
         ::close(fd);
+	log_debug("%s: Closed fd %d", __FUNCTION__, fd);
 //        closeNet(fd);
     }
 
@@ -576,7 +672,7 @@ Network::readNet(int fd, char *buffer, int nbytes, int timeout)
             + static_cast<double>(tp.tv_usec*1e-6);
     }
 #endif
-    if (fd) {
+    if (fd > 0) {
         FD_ZERO(&fdset);
         FD_SET(fd, &fdset);
 
@@ -605,10 +701,21 @@ Network::readNet(int fd, char *buffer, int nbytes, int timeout)
 
         ret = read(fd, buffer, nbytes);
 	if (_debug) {
-	    log_msg (_("read %d bytes from fd %d"), ret, fd);
+	    log_debug (_("read %d bytes from fd %d"), ret, fd);
 	}
     }
 
+#if 0
+    unsigned char *hexint;
+    hexint = new unsigned char[(nbytes + 3) *3];    
+    
+    hexify(hexint, (unsigned char *)buffer, ret, true);
+    log_debug (_("%s: Read packet data from fd %d: \n%s"),
+	     __FUNCTION__, fd, (char *)hexint);
+//     hexify(hexint,  (unsigned char *)buffer, ret, false);
+//     log_debug (_("%s: The packet data is: 0x%s"), __FUNCTION__, (char *)hexint);
+    delete hexint;
+#endif    
     return ret;
 
 }
@@ -707,16 +814,15 @@ Network::writeNet(int fd, char const *buffer, int nbytes, int timeout)
             bufptr += ret;
             if (ret != nbytes) {
 		if (_debug) {
-		    log_msg (_("wrote %d bytes to fd %d, expected %d"),
+		    log_debug (_("wrote %d bytes to fd %d, expected %d"),
 			       ret, fd, nbytes);
 		}
-//                retries++;
             } else {
 		if (_debug) {
-		    log_msg (_("wrote %d bytes to fd %d"),
+		    log_debug (_("wrote %d bytes to fd %d"),
 			       ret, fd);
 		}
-                return ret;
+//                return ret;
             }
         }
     }
@@ -729,12 +835,25 @@ Network::writeNet(int fd, char const *buffer, int nbytes, int timeout)
         if ((endtime.tv_sec - starttime.tv_sec) &&
             endtime.tv_usec - starttime.tv_usec)
         {
-            log_msg (_("took %d usec to write (%d bytes)"),
+            log_debug (_("took %d usec to write (%d bytes)"),
 		endtime.tv_usec - starttime.tv_usec, bytes_written);
         }
     }
 #endif
 
+#if 0
+    unsigned char *hexint;
+    hexint = new unsigned char[(nbytes + 3) *3];
+    
+    hexify(hexint, (unsigned char *)buffer, nbytes, true);
+    log_debug (_("%s: Wrote packet data to fd %d: \n%s"),
+	     __FUNCTION__, fd, (char *)hexint);
+//     hexify(hexint,  (unsigned char *)buffer, ret, false);
+//     log_debug (_("%s: Read packet data from fd %d: 0x%s"),
+// 	      __FUNCTION__, fd, (char *)hexint);
+     delete hexint;
+#endif    
+
     return ret;
 }
 
