@@ -12,22 +12,6 @@
 #include <cmath>
 #include <cfloat>
 
-// OSX doesn't have single precision math functions defined in math.h
-#ifdef __MACH__
-    #define sinf sin
-    #define cosf cos
-    #define tanf tan
-    #define asinf asin
-    #define acosf acos
-    #define atanf atan
-    #define atan2f atan2
-    #define sqrtf sqrt
-    #define logf log
-    #define expf exp
-    #define fabsf fabs
-    #define powf pow
-#endif
-
 // isfinite() comes with C99; fake version here in case compiler lacks it.
 #ifndef isfinite
 #define isfinite(x) (sizeof(x) == sizeof(float) ? isfinitef(x) : isfinited(x))
