@@ -1,9 +1,9 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -20,13 +20,13 @@
 //
 
 
-/* $Id: Range2d.h,v 1.16 2007/05/28 15:41:02 ann Exp $ */
+/* $Id: Range2d.h,v 1.20 2008/01/21 20:56:11 rsavoye Exp $ */
 
 #ifndef GNASH_RANGE2D_H
 #define GNASH_RANGE2D_H
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include <ostream>
@@ -36,8 +36,6 @@
 #include <iostream> // temporary include for debugging
 #include <cmath> // for floor / ceil
 
-using namespace std;
-
 namespace gnash {
 
 namespace geometry {
@@ -652,6 +650,18 @@ public:
 		assert ( isFinite() );
 		return _ymax;
 	}
+	
+	
+	/// Get area (width*height)
+  ///  
+	T getArea() const
+  {
+    assert ( !isWorld() );
+    if ( isNull() ) return 0;
+    return (_xmax - _xmin) * (_ymax - _ymin);
+    // this implementation is for float types, see specialization below
+    // for ints... 
+  } 
 
 	/// Expand this range to include the given Range2d
 	//
@@ -814,6 +824,29 @@ Range2d<unsigned int>::roundMax(float max)
 	return static_cast<unsigned int>(ceil((float)max));
 }
 
+/// Specialization of area value for int type.
+//
+/// Add one.
+///
+template <> inline int
+Range2d<int>::getArea() const
+{
+  assert ( !isWorld() );
+  if ( isNull() ) return 0;
+  return (_xmax - _xmin + 1) * (_ymax - _ymin + 1);
+}
+
+/// Specialization of area value for unsigned int type.
+//
+/// Add one.
+///
+template <> inline unsigned int
+Range2d<unsigned int>::getArea() const
+{
+  assert ( isFinite() );
+  return (_xmax - _xmin + 1) * (_ymax - _ymin + 1);
+}
+
 
 } // namespace gnash::geometry
 } // namespace gnash
