@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,16 +15,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
-/* $Id: BitmapMovieInstance.h,v 1.5 2007/07/01 10:54:18 bjacques Exp $ */
+/* $Id: BitmapMovieInstance.h,v 1.7 2008/01/21 20:55:46 rsavoye Exp $ */
 
 #ifndef GNASH_BITMAPMOVIEINSTANCE_H
 #define GNASH_BITMAPMOVIEINSTANCE_H
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "movie_instance.h" // for inheritance
@@ -45,7 +42,7 @@ class BitmapMovieInstance : public movie_instance
 
 public:
 
-	BitmapMovieInstance(BitmapMovieDefinition* def); 
+	BitmapMovieInstance(BitmapMovieDefinition* def, character* parent=0); 
 
 	virtual ~BitmapMovieInstance() {}
 
