@@ -1,6 +1,6 @@
 // array.cpp:  ActionScript array class, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 //
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "as_value.h"
@@ -29,7 +29,8 @@
 #include "fn_call.h"
 #include "GnashException.h"
 #include "action.h" // for call_method
-#include "VM.h" // for PROPNAME macro to work
+#include "VM.h" // for PROPNAME, registerNative
+#include "Object.h" // for getObjectInterface()
 
 #include <string>
 #include <algorithm>
@@ -46,6 +47,7 @@ typedef boost::function2<bool, const as_value&, const as_value&> as_cmp_fn;
 static as_object* getArrayInterface();
 static void attachArrayProperties(as_object& proto);
 static void attachArrayInterface(as_object& proto);
+static void attachArrayStatics(as_object& proto);
 
 inline static bool int_lt_or_eq (int a)
 {
@@ -73,16 +75,16 @@ struct as_value_lt
 
 	inline int str_cmp(const as_value& a, const as_value& b)
 	{
-		std::string s = a.to_string_versioned(_sv, &_env);
-		return s.compare(b.to_string_versioned(_sv,& _env));
+		std::string s = a.to_string_versioned(_sv);
+		return s.compare(b.to_string_versioned(_sv));
 	}
 
 	inline int str_nocase_cmp(const as_value& a, const as_value& b)
 	{
 		using namespace boost::algorithm;
 
-		std::string c = to_upper_copy(a.to_string_versioned(_sv, &_env));
-		std::string d = to_upper_copy(b.to_string_versioned(_sv, &_env));
+		std::string c = to_upper_copy(a.to_string_versioned(_sv));
+		std::string d = to_upper_copy(b.to_string_versioned(_sv));
 		return c.compare(d);
 	}
 
@@ -92,8 +94,8 @@ struct as_value_lt
 		if (b.is_undefined()) return true;
 		if (a.is_null()) return false;
 		if (b.is_null()) return true;
-		double aval = a.to_number(&_env);
-		double bval = b.to_number(&_env);
+		double aval = a.to_number();
+		double bval = b.to_number();
 		if (isnan(aval)) return false;
 		if (isnan(bval)) return true;
 		return aval < bval;
@@ -105,8 +107,8 @@ struct as_value_lt
 		if (a.is_undefined()) return true;
 		if (b.is_null()) return false;
 		if (a.is_null()) return true;
-		double aval = a.to_number(&_env);
-		double bval = b.to_number(&_env);
+		double aval = a.to_number();
+		double bval = b.to_number();
 		if (isnan(bval)) return false;
 		if (isnan(aval)) return true;
 		return aval > bval;
@@ -116,8 +118,8 @@ struct as_value_lt
 	{
 		if (a.is_undefined() && b.is_undefined()) return true;
 		if (a.is_null() && b.is_null()) return true;
-		double aval = a.to_number(&_env);
-		double bval = b.to_number(&_env);
+		double aval = a.to_number();
+		double bval = b.to_number();
 		if (isnan(aval) && isnan(bval)) return true;
 		return aval == bval;
 	}
@@ -254,7 +256,7 @@ struct as_value_num_nocase_eq : public as_value_lt
 // Note:
 // fUniqueSort and fReturnIndexedArray must first be stripped from the flag
 as_cmp_fn
-get_basic_cmp(uint8_t flags, as_environment& env)
+get_basic_cmp(boost::uint8_t flags, as_environment& env)
 {
 	as_cmp_fn f;
 
@@ -307,7 +309,7 @@ get_basic_cmp(uint8_t flags, as_environment& env)
 // Note:
 // fUniqueSort and fReturnIndexedArray must first be stripped from the flag
 as_cmp_fn
-get_basic_eq(uint8_t flags, as_environment& env)
+get_basic_eq(boost::uint8_t flags, as_environment& env)
 {
 	as_cmp_fn f;
 	flags &= ~(as_array_object::fDescending);
@@ -344,12 +346,14 @@ public:
 	as_function& _comp;
 	as_object* _object;
 	bool (*_zeroCmp)(const int);
+	as_environment& _env;
 
 	as_value_custom(as_function& comparator, bool (*zc)(const int), 
-		boost::intrusive_ptr<as_object> this_ptr)
+		boost::intrusive_ptr<as_object> this_ptr, as_environment& env)
 		:
 		_comp(comparator),
-		_zeroCmp(zc)
+		_zeroCmp(zc),
+		_env(env)
 	{
 		_object = this_ptr.get();
 	}
@@ -357,12 +361,21 @@ public:
 	bool operator() (const as_value& a, const as_value& b)
 	{
 		as_value cmp_method(&_comp);
-		as_environment env;
 		as_value ret(0);
 
-		env.push(a);
-		env.push(b);
-		ret = call_method(cmp_method, &env, _object, 2, 1);
+#ifndef NDEBUG
+		size_t prevStackSize = _env.stack_size();
+#endif
+
+		_env.push(a);
+		_env.push(b);
+		ret = call_method(cmp_method, &_env, _object, 2, _env.stack_size()-1);
+		_env.drop(2);
+
+#ifndef NDEBUG
+		assert(prevStackSize == _env.stack_size());
+#endif
+
 		return (*_zeroCmp)((int)ret.to_number());
 	}
 };
@@ -372,10 +385,10 @@ class as_value_prop
 {
 public:
 	as_cmp_fn _comp;
-	const std::string& _prop;
+	string_table::key _prop;
 	
 	// Note: cmpfn must implement a strict weak ordering
-	as_value_prop(const std::string& name, 
+	as_value_prop(string_table::key name, 
 		as_cmp_fn cmpfn)
 		:
 		_comp(cmpfn),
@@ -386,6 +399,8 @@ public:
 	bool operator() (const as_value& a, const as_value& b)
 	{
 		as_value av, bv;
+
+		// why do we cast ao/bo to objects here ?
 		boost::intrusive_ptr<as_object> ao = a.to_object();
 		boost::intrusive_ptr<as_object> bo = b.to_object();
 		
@@ -399,27 +414,34 @@ public:
 class as_value_multiprop
 {
 public:
-	const as_cmp_fn* _cmps;
-	std::deque<std::string>& _prps;
+	typedef std::deque<as_cmp_fn> Comps;
+	Comps& _cmps;
+
+	typedef std::deque<string_table::key> Props;
+	Props& _prps;
 
 	// Note: all as_cmp_fns in *cmps must implement strict weak ordering
-	as_value_multiprop(std::deque<std::string>& prps, as_cmp_fn* cmps)
+	as_value_multiprop(std::deque<string_table::key>& prps, 
+		std::deque<as_cmp_fn>& cmps)
 		:
+		_cmps(cmps),
 		_prps(prps)
 	{
-		_cmps = cmps;
 	}
 
 	bool operator() (const as_value& a, const as_value& b)
 	{
-		const as_cmp_fn* cmp = _cmps;
-		std::deque<std::string>::iterator pit;
+		std::deque<as_cmp_fn>::iterator cmp = _cmps.begin();
+		Props::iterator pit;
+
+		// why do we cast ao/bo to objects here ?
+		boost::intrusive_ptr<as_object> ao = a.to_object();
+		boost::intrusive_ptr<as_object> bo = b.to_object();
 		
 		for (pit = _prps.begin(); pit != _prps.end(); ++pit, ++cmp)
 		{
 			as_value av, bv;
-			boost::intrusive_ptr<as_object> ao = a.to_object();
-			boost::intrusive_ptr<as_object> bo = b.to_object();
+
 			ao->get_member(*pit, &av);
 			bo->get_member(*pit, &bv);
 
@@ -437,21 +459,24 @@ public:
 class as_value_multiprop_eq : public as_value_multiprop
 {
 public:
-	as_value_multiprop_eq(std::deque<std::string>& prps, as_cmp_fn* eq)
-		: as_value_multiprop(prps, eq)
+	as_value_multiprop_eq(std::deque<string_table::key>& prps, 
+		std::deque<as_cmp_fn>& cmps)
+		: as_value_multiprop(prps, cmps)
 	{
 	}
 
 	bool operator() (const as_value& a, const as_value& b)
 	{
-		const as_cmp_fn* cmp = _cmps;
-		std::deque<std::string>::iterator pit;
-		
+		Comps::iterator cmp = _cmps.begin();
+		Props::iterator pit;
+
+		// why do we cast ao/bo to objects here ?
+		boost::intrusive_ptr<as_object> ao = a.to_object();
+		boost::intrusive_ptr<as_object> bo = b.to_object();
+
 		for (pit = _prps.begin(); pit != _prps.end(); ++pit, ++cmp)
 		{
 			as_value av, bv;
-			boost::intrusive_ptr<as_object> ao = a.to_object();
-			boost::intrusive_ptr<as_object> bo = b.to_object();
 			ao->get_member(*pit, &av);
 			bo->get_member(*pit, &bv);
 
@@ -464,8 +489,8 @@ public:
 
 // Convenience function to strip fUniqueSort and fReturnIndexedArray from sort
 // flag. Presence of flags recorded in douniq and doindex.
-static inline uint8_t
-flag_preprocess(uint8_t flgs, bool* douniq, bool* doindex)
+static inline boost::uint8_t
+flag_preprocess(boost::uint8_t flgs, bool* douniq, bool* doindex)
 {
 	*douniq = (flgs & as_array_object::fUniqueSort);
 	*doindex = (flgs & as_array_object::fReturnIndexedArray);
@@ -476,31 +501,31 @@ flag_preprocess(uint8_t flgs, bool* douniq, bool* doindex)
 
 // Convenience function to process and extract flags from an as_value array
 // of flags (as passed to sortOn when sorting on multiple properties)
-static void
+std::deque<boost::uint8_t> 
 get_multi_flags(std::deque<as_value>::const_iterator itBegin, 
-	std::deque<as_value>::const_iterator itEnd, 
-	uint8_t* flgs, bool* uniq, bool* index)
+	std::deque<as_value>::const_iterator itEnd, bool* uniq, bool* index)
 {
 	std::deque<as_value>::const_iterator it = itBegin;
-	int i = 0;
+	std::deque<boost::uint8_t> flgs;
 
 	// extract fUniqueSort and fReturnIndexedArray from first flag
 	if (it != itEnd)
 	{
-		uint8_t flag = static_cast<uint8_t>((*it).to_number());
+		boost::uint8_t flag = static_cast<boost::uint8_t>((*it).to_number());
 		flag = flag_preprocess(flag, uniq, index);
-		flgs[i++] = flag;
+		flgs.push_back(flag);
 		++it;
 	}
 
 	while (it != itEnd)
 	{
-		uint8_t flag = static_cast<uint8_t>((*it).to_number());
+		boost::uint8_t flag = static_cast<boost::uint8_t>((*it).to_number());
 		flag &= ~(as_array_object::fReturnIndexedArray);
 		flag &= ~(as_array_object::fUniqueSort);
-		flgs[i++] = flag;
+		flgs.push_back(flag);
 		++it;
 	}
+	return flgs;
 }
 
 as_array_object::as_array_object()
@@ -555,10 +580,12 @@ as_array_object::end()
 }
 
 int
-as_array_object::index_requested(const std::string& name)
+as_array_object::index_requested(string_table::key name)
 {
+	std::string name_str = VM::get().getStringTable().value(name);
+
 	as_value temp;
-	temp.set_string(name);
+	temp.set_string(name_str);
 	double value = temp.to_number();
 
 	// if we were sent a string that can't convert like "asdf", it returns as NaN. -1 means invalid index
@@ -590,7 +617,9 @@ as_array_object::pop()
 	// If the array is empty, report an error and return undefined!
 	if ( elements.empty() ) 
 	{
-	    log_error(_("tried to pop element from back of empty array, returning undef"));
+		IF_VERBOSE_ASCODING_ERRORS(
+		log_aserror(_("tried to pop element from back of empty array, returning undef"));
+		);
 		return as_value(); // undefined
 	}
 
@@ -606,7 +635,9 @@ as_array_object::shift()
 	// If the array is empty, report an error and return undefined!
 	if ( elements.empty() ) 
 	{
-		log_error(_("tried to shift element from front of empty array, returning undef"));
+		IF_VERBOSE_ASCODING_ERRORS(
+		log_aserror(_("tried to shift element from front of empty array, returning undef"));
+		);
 		return as_value(); // undefined
 	}
 
@@ -624,7 +655,7 @@ as_array_object::reverse()
 }
 
 std::string
-as_array_object::join(const std::string& separator, as_environment* env) const
+as_array_object::join(const std::string& separator, as_environment*) const
 {
 	// TODO - confirm this is the right format!
 	// Reportedly, flash version 7 on linux, and Flash 8 on IE look like
@@ -645,12 +676,12 @@ as_array_object::join(const std::string& separator, as_environment* env) const
 			itEnd=elements.end();
 
 		// print first element w/out separator prefix
-		temp += (*it++).to_string_versioned(swfversion, env);
+		temp += (*it++).to_string_versioned(swfversion);
 
 		// print subsequent elements with separator prefix
 		while ( it != itEnd )
 		{
-			temp += separator + (*it++).to_string_versioned(swfversion, env);
+			temp += separator + (*it++).to_string_versioned(swfversion);
 		}
 	}
 
@@ -702,7 +733,7 @@ as_array_object::slice(unsigned int start, unsigned int one_past_end)
 	std::auto_ptr<as_array_object> newarray(new as_array_object);
 
 #ifdef GNASH_DEBUG
-	log_msg(_("Array.slice(%u, %u) called"), start, one_past_end);
+	log_debug(_("Array.slice(%u, %u) called"), start, one_past_end);
 #endif
 
 	size_t newsize = one_past_end - start;
@@ -731,8 +762,8 @@ as_array_object::splice(unsigned start, unsigned len,
 	std::ostream_iterator<as_value> ostrIter(ss, "," ) ;
 	std::copy(replace.begin(), replace.end(), ostrIter);
         ss << ") called";
-	log_msg("%s", ss.str().c_str());
-	log_msg(_("Current array is %s"), toString().c_str());
+	log_debug("%s", ss.str().c_str());
+	log_debug(_("Current array is %s"), toString().c_str());
 #endif
 
 	container::iterator itStart = elements.begin()+start;
@@ -760,9 +791,24 @@ as_array_object::splice(unsigned start, unsigned len,
 	return ret;
 }
 
+bool
+as_array_object::removeFirst(const as_value& v)
+{
+	for (iterator it = elements.begin(); it != elements.end(); ++it)
+	{
+		if ( v.equals(*it) )
+		{
+			elements.erase(it);
+			return true;
+		}
+	}
+	return false;
+}
+
 /* virtual public, overriding as_object::get_member */
 bool
-as_array_object::get_member(const std::string& name, as_value *val)
+as_array_object::get_member(string_table::key name, as_value *val,
+	string_table::key nsname)
 {
 	// an index has been requested
 	int index = index_requested(name);
@@ -772,7 +818,7 @@ as_array_object::get_member(const std::string& name, as_value *val)
 		return true;
 	}
 
-	return get_member_default(name, val);
+	return get_member_default(name, val, nsname);
 }
 
 void
@@ -781,10 +827,24 @@ as_array_object::resize(unsigned int newsize)
 	elements.resize(newsize);
 }
 
+void
+as_array_object::set_indexed(unsigned int index,
+	const as_value& val)
+{
+	if (index >= elements.size())
+	{
+		// make sure the vector is large enough.
+		elements.resize(index + 1);
+	}
+
+	elements[index] = val;
+	return;
+}
+
 /* virtual public, overriding as_object::set_member */
 void
-as_array_object::set_member(const std::string& name,
-		const as_value& val )
+as_array_object::set_member(string_table::key name,
+		const as_value& val, string_table::key nsname)
 {
 	int index = index_requested(name);
 
@@ -803,7 +863,7 @@ as_array_object::set_member(const std::string& name,
 		return;
 	}
 
-	as_object::set_member_default(name,val);
+	as_object::set_member_default(name,val, nsname);
 }
 
 as_array_object*
@@ -827,7 +887,7 @@ array_splice(const fn_call& fn)
 #ifdef GNASH_DEBUG
 	std::stringstream ss;
 	fn.dump_args(ss);
-	log_msg(_("Array(%s).splice(%s) called"), array->toString().c_str(), ss.str().c_str());
+	log_debug(_("Array(%s).splice(%s) called"), array->toString().c_str(), ss.str().c_str());
 #endif
 
 	if (fn.nargs < 1)
@@ -844,12 +904,12 @@ array_splice(const fn_call& fn)
 	// Get start offset
 	//----------------
 	unsigned startoffset;
-	int start = fn.arg(0).to_number<int>(&(fn.env()));
+	int start = fn.arg(0).to_number<int>();
 	if ( start < 0 ) start = array->size()+start; // start is negative, so + means -abs()
 	startoffset = iclamp(start, 0, origlen);
 #ifdef GNASH_DEBUG
 	if ( startoffset != start )
-		log_msg(_("Array.splice: start:%d became %u"), start, startoffset);
+		log_debug(_("Array.splice: start:%d became %u"), start, startoffset);
 #endif
 
 	//----------------
@@ -858,7 +918,7 @@ array_splice(const fn_call& fn)
 	unsigned len = origlen - start;
 	if (fn.nargs > 1)
 	{
-		int lenval = fn.arg(1).to_number<int>(&(fn.env()));
+		int lenval = fn.arg(1).to_number<int>();
 		if ( lenval < 0 )
 		{
 			IF_VERBOSE_ASCODING_ERRORS(
@@ -891,8 +951,9 @@ array_sort(const fn_call& fn)
 {
 	boost::intrusive_ptr<as_array_object> array = 
 		ensureType<as_array_object>(fn.this_ptr);
+
 	as_environment& env = fn.env();
-	uint8_t flags = 0;
+	boost::uint8_t flags = 0;
 
 	if ( fn.nargs == 0 )
 	{
@@ -901,7 +962,7 @@ array_sort(const fn_call& fn)
 	}
 	else if ( fn.nargs == 1 && fn.arg(0).is_number() )
 	{
-		flags=static_cast<uint8_t>(fn.arg(0).to_number());
+		flags=static_cast<boost::uint8_t>(fn.arg(0).to_number());
 	}
 	else if ( fn.arg(0).is_as_function() )
 	{
@@ -910,13 +971,13 @@ array_sort(const fn_call& fn)
 		bool (*icmp)(int);
 	
 		if ( fn.nargs == 2 && fn.arg(1).is_number() )
-			flags=static_cast<uint8_t>(fn.arg(1).to_number());
+			flags=static_cast<boost::uint8_t>(fn.arg(1).to_number());
 		
 		if (flags & as_array_object::fDescending) icmp = &int_lt_or_eq;
 		else icmp = &int_gt;
 
 		as_value_custom avc = 
-			as_value_custom(*as_func, icmp, fn.this_ptr);
+			as_value_custom(*as_func, icmp, fn.this_ptr, env);
 
 		if ( (flags & as_array_object::fReturnIndexedArray) )
 			return as_value(array->sort_indexed(avc));
@@ -930,7 +991,8 @@ array_sort(const fn_call& fn)
 		IF_VERBOSE_ASCODING_ERRORS(
 		log_aserror(_("Sort called with invalid arguments."));
 		)
-		return as_value();
+		if ( fn.arg(0).is_undefined() ) return as_value();
+		return as_value((boost::intrusive_ptr<as_object>)array);
 	}
 	bool do_unique, do_index;
 	flags = flag_preprocess(flags, &do_unique, &do_index);
@@ -955,18 +1017,20 @@ array_sortOn(const fn_call& fn)
 		ensureType<as_array_object>(fn.this_ptr);
 	as_environment& env = fn.env();
 	bool do_unique = false, do_index = false;
-	uint8_t flags = 0;
-	int sv = VM::get().getSWFVersion();
+	boost::uint8_t flags = 0;
+
+	VM& vm = VM::get();
+	int sv = vm.getSWFVersion();
+	string_table& st = vm.getStringTable();
 
 	// cases: sortOn("prop) and sortOn("prop", Array.FLAG)
 	if ( fn.nargs > 0 && fn.arg(0).is_string() )
 	{
-		std::string propField = 
-			PROPNAME(fn.arg(0).to_string_versioned(sv, &env));
+		string_table::key propField = st.find(PROPNAME(fn.arg(0).to_string_versioned(sv)));
 
 		if ( fn.nargs > 1 && fn.arg(1).is_number() )
 		{
-			flags = static_cast<uint8_t>(fn.arg(1).to_number());
+			flags = static_cast<boost::uint8_t>(fn.arg(1).to_number());
 			flags = flag_preprocess(flags, &do_unique, &do_index);
 		}
 		as_value_prop avc = as_value_prop(propField, 
@@ -990,16 +1054,15 @@ array_sortOn(const fn_call& fn)
 	{
 		boost::intrusive_ptr<as_array_object> props = 
 			ensureType<as_array_object>(fn.arg(0).to_object());
-		std::deque<std::string> prp;
+		std::deque<string_table::key> prp;
 		unsigned int optnum = props->size();
-		as_cmp_fn cmp[optnum];
-		as_cmp_fn eq[optnum];
+		std::deque<as_cmp_fn> cmp;
+		std::deque<as_cmp_fn> eq;
 
 		for (std::deque<as_value>::const_iterator it = props->begin();
 			it != props->end(); ++it)
 		{
-			std::string s = 
-				PROPNAME((*it).to_string_versioned(sv, &env));
+			string_table::key s = st.find(PROPNAME((*it).to_string_versioned(sv)));
 			prp.push_back(s);
 		}
 		
@@ -1008,8 +1071,7 @@ array_sortOn(const fn_call& fn)
 		{
 			// assign each cmp function to the standard cmp fn
 			as_cmp_fn c = get_basic_cmp(0, env);
-			for (unsigned int i = 0; i < optnum; i++)
-				cmp[i] = c;
+			cmp.assign(optnum, c);
 		}
 		// case: sortOn(["prop1", "prop2"], [Array.FLAG1, Array.FLAG2])
 		else if ( fn.arg(1).is_object() )
@@ -1018,46 +1080,47 @@ array_sortOn(const fn_call& fn)
 				ensureType<as_array_object>(fn.arg(1).to_object());
 			if (farray->size() == optnum)
 			{
-				uint8_t flgs[optnum];
 				std::deque<as_value>::const_iterator 
 					fBegin = farray->begin(),
 					fEnd = farray->end();
 
-				get_multi_flags(fBegin, fEnd, flgs, 
-					&do_unique, &do_index);
+				std::deque<boost::uint8_t> flgs = 
+					get_multi_flags(fBegin, fEnd, 
+						&do_unique, &do_index);
 
-				for (unsigned int i = 0; i < optnum; i++)
-					cmp[i] = get_basic_cmp(flgs[i], env);
+				std::deque<boost::uint8_t>::const_iterator it = 
+					flgs.begin();
+
+				while (it != flgs.end())
+					cmp.push_back(get_basic_cmp(*it++, env));
 
 				if (do_unique)
 				{
-					for (unsigned int i = 0; i < optnum; i++)
-						eq[i] = get_basic_eq(flgs[i], env);
+					it = flgs.begin();
+					while (it != flgs.end())
+						eq.push_back(get_basic_eq(*it++, env));
 				}
 			}
 			else
 			{
 				as_cmp_fn c = get_basic_cmp(0, env);
-				for (unsigned int i = 0; i < optnum; i++)
-					cmp[i] = c;
+				cmp.assign(optnum, c);
 			}
 		}
 		// case: sortOn(["prop1", "prop2"], Array.FLAG)
 		else if ( fn.arg(1).is_number() )
 		{
-			uint8_t flags = 
-				static_cast<uint8_t>(fn.arg(1).to_number());
+			boost::uint8_t flags = 
+				static_cast<boost::uint8_t>(fn.arg(1).to_number());
 			flag_preprocess(flags, &do_unique, &do_index);
 			as_cmp_fn c = get_basic_cmp(flags, env);
 
-			for (unsigned int i = 0; i < optnum; i++)
-				cmp[i] = c;
+			cmp.assign(optnum, c);
 			
 			if (do_unique)
 			{
 				as_cmp_fn e = get_basic_eq(flags, env);
-				for (unsigned int i = 0; i < optnum; i++)
-					eq[i] = e;
+				eq.assign(optnum, e);
 			}
 		}
 
@@ -1179,7 +1242,7 @@ array_join(const fn_call& fn)
 	as_environment* env = &(fn.env());
 
 	if (fn.nargs > 0)
-		separator = fn.arg(0).to_string_versioned(swfversion, env);
+		separator = fn.arg(0).to_string_versioned(swfversion);
 
 	std::string ret = array->join(separator, env);
 
@@ -1201,7 +1264,7 @@ array_to_string(const fn_call& fn)
 {
 	boost::intrusive_ptr<as_array_object> array = ensureType<as_array_object>(fn.this_ptr);
 
-	std::string ret = array->toString(&(fn.env()));
+	std::string ret = array->toString();
 
 		IF_VERBOSE_ACTION
 		(
@@ -1314,7 +1377,16 @@ array_length(const fn_call& fn)
 
 	if ( fn.nargs ) // setter
 	{
-		array->resize(unsigned(fn.arg(0).to_number(&(fn.env()))));
+		int length = fn.arg(0).to_int();
+		if ( length < 0 ) // TODO: set a max limit too ?
+		{
+			IF_VERBOSE_ASCODING_ERRORS(
+			log_aserror("Attempt to set Array.length to a negative value %d", length);
+			)
+			length = 0;
+		}
+
+		array->resize(length);
 		return as_value();
 	}
 	else // getter
@@ -1327,7 +1399,7 @@ as_value
 array_new(const fn_call& fn)
 {
 	IF_VERBOSE_ACTION (
-		log_action(_("array_new called, nargs = %d"), fn.nargs);
+	log_action(_("array_new called, nargs = %d"), fn.nargs);
 	);
 
 	boost::intrusive_ptr<as_array_object>	ao = new as_array_object;
@@ -1338,18 +1410,10 @@ array_new(const fn_call& fn)
 	}
 	else if (fn.nargs == 1 && fn.arg(0).is_number() )
 	{
-		// Create an empty array with the given number of undefined elements.
-		//
-		as_value index_number, undef_value;
-		int sv = VM::get().getSWFVersion();
-		as_environment* env = &(fn.env());
-
-		undef_value.set_undefined();
-		for (int i = 0; i < int(fn.arg(0).to_number()); i++)
-		{
-			index_number.set_int(i);
-			ao->set_member(index_number.to_string_versioned(sv, env).c_str(), undef_value);
-		}
+		// TODO: limit max size !!
+		int newSize = fn.arg(0).to_int();
+		if ( newSize < 0 ) newSize = 0;
+		else ao->resize(newSize);
 	}
 	else
 	{
@@ -1362,7 +1426,7 @@ array_new(const fn_call& fn)
 	}
 
 	IF_VERBOSE_ACTION (
-		log_action(_("array_new setting object %p in result"), (void*)ao.get());
+	log_action(_("array_new setting object %p in result"), (void*)ao.get());
 	);
 
 	return as_value(ao.get());
@@ -1376,26 +1440,13 @@ attachArrayProperties(as_object& proto)
 
 	gettersetter = new builtin_function(&array_length, NULL);
 	proto.init_property("length", *gettersetter, *gettersetter);
+
+	proto.init_member("size", new builtin_function(array_size));
 }
 
 static void
-attachArrayInterface(as_object& proto)
+attachArrayStatics(as_object& proto)
 {
-
-	proto.init_member("join", new builtin_function(array_join));
-	proto.init_member("concat", new builtin_function(array_concat));
-	proto.init_member("slice", new builtin_function(array_slice));
-	proto.init_member("push", new builtin_function(array_push));
-	proto.init_member("unshift", new builtin_function(array_unshift));
-	proto.init_member("pop", new builtin_function(array_pop));
-	proto.init_member("shift", new builtin_function(array_shift));
-	proto.init_member("splice", new builtin_function(array_splice));
-	proto.init_member("sort", new builtin_function(array_sort));
-	proto.init_member("size", new builtin_function(array_size));
-	proto.init_member("sortOn", new builtin_function(array_sortOn));
-	proto.init_member("reverse", new builtin_function(array_reverse));
-	proto.init_member("toString", new builtin_function(array_to_string));
-
 	proto.init_member("CASEINSENSITIVE", as_array_object::fCaseInsensitive);
 	proto.init_member("DESCENDING", as_array_object::fDescending);
 	proto.init_member("UNIQUESORT", as_array_object::fUniqueSort);
@@ -1403,13 +1454,68 @@ attachArrayInterface(as_object& proto)
 	proto.init_member("NUMERIC", as_array_object::fNumeric);
 }
 
+static void
+attachArrayInterface(as_object& proto)
+{
+	VM& vm = proto.getVM();
+
+	// Array.push
+	vm.registerNative(array_push, 252, 1);
+	proto.init_member("push", vm.getNative(252, 1));
+
+	// Array.pop
+	vm.registerNative(array_pop, 252, 2);
+	proto.init_member("pop", vm.getNative(252, 2));
+
+	// Array.concat
+	vm.registerNative(array_concat, 252, 3);
+	proto.init_member("concat", vm.getNative(252, 3));
+
+	// Array.shift
+	vm.registerNative(array_shift, 252, 4);
+	proto.init_member("shift", vm.getNative(252, 4));
+
+	// Array.unshift
+	vm.registerNative(array_unshift, 252, 5);
+	proto.init_member("unshift", vm.getNative(252, 5));
+
+	// Array.slice
+	vm.registerNative(array_slice, 252, 6);
+	proto.init_member("slice", vm.getNative(252, 6));
+
+	// Array.join
+	vm.registerNative(array_join, 252, 7);
+	proto.init_member("join", vm.getNative(252, 7));
+
+	// Array.splice
+	vm.registerNative(array_splice, 252, 8);
+	proto.init_member("splice", vm.getNative(252, 8));
+
+	// Array.toString
+	vm.registerNative(array_to_string, 252, 9);
+	proto.init_member("toString", vm.getNative(252, 9));
+
+	// Array.sort
+	vm.registerNative(array_sort, 252, 10);
+	proto.init_member("sort", vm.getNative(252, 10));
+
+	// Array.reverse
+	vm.registerNative(array_reverse, 252, 11);
+	proto.init_member("reverse", vm.getNative(252, 11));
+
+	// Array.sortOn
+	vm.registerNative(array_sortOn, 252, 12);
+	proto.init_member("sortOn", vm.getNative(252, 12));
+
+}
+
 static as_object*
 getArrayInterface()
 {
 	static boost::intrusive_ptr<as_object> proto = NULL;
 	if ( proto == NULL )
 	{
-		proto = new as_object();
+		proto = new as_object(getObjectInterface());
 		VM::get().addStatic(proto.get());
 
 		attachArrayInterface(*proto);
@@ -1430,17 +1536,32 @@ array_class_init(as_object& glob)
 
 	if ( ar == NULL )
 	{
+		VM& vm = glob.getVM();
+		vm.registerNative(array_new, 252, 0);
 		ar = new builtin_function(&array_new, getArrayInterface());
-		VM::get().addStatic(ar.get());
+		vm.addStatic(ar.get());
 
-		// We replicate interface to the Array class itself
-		attachArrayInterface(*ar);
+		// Attach static members
+		attachArrayStatics(*ar);
 	}
 
 	// Register _global.Array
 	glob.init_member("Array", ar.get());
 }
 
+void
+as_array_object::enumerateNonProperties(as_environment& env) const
+{
+	// TODO: only actually defined elements should be pushed on the env
+	//       but we currently have no way to distinguish between defined
+	//       and non-defined elements
+	for (unsigned int i=0; i<size(); ++i)
+	{
+		// here should be something like, if ( isDefined(i) ) ...
+		env.push(as_value(i));
+	}
+}
+
 #ifdef GNASH_USE_GC
 void
 as_array_object::markReachableResources() const
