@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 #define GNASH_AS_ENVIRONMENT_H
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "as_value.h" // for composition (vector + frame_slot)
@@ -51,29 +51,33 @@ public:
 
 	as_environment()
 		:
-		m_target(0)
+		m_stack(),
+		m_target(0),
+		_original_target(0)
 	{
 	}
 
 	character* get_target() { return m_target; }
 	void set_target(character* target);
 
+	character* get_original_target() { return _original_target; }
+
 	// Reset target to its original value
 	void reset_target() { m_target = _original_target; }
 
 	/// @{ Stack access/manipulation
 
-	// @@ TODO do more checking on these
-	template<class T>
-	// stack access/manipulation
-	void	push(T val) { push_val(as_value(val)); }
-	void	push_val(const as_value& val) { m_stack.push_back(val); }
+	/// Push a value on the stack
+	void	push(const as_value& val)
+	{
+		m_stack.push_back(val);
+	}
 
 
 	/// Pops an as_value off the stack top and return it.
 	as_value pop()
 	{
-		assert( m_stack.size() > 0 );
+		assert( ! m_stack.empty() );
 		as_value result = m_stack.back();
 		m_stack.pop_back();
 		return result;
@@ -85,8 +89,9 @@ public:
 	///
 	as_value& top(size_t dist)
 	{
-		assert ( m_stack.size() > dist );
-		return m_stack[m_stack.size() - 1 - dist];
+		size_t ssize = m_stack.size();
+		assert ( ssize > dist );
+		return m_stack[ssize - 1 - dist];
 	}
 
 	/// Get stack value at the given distance from bottom.
@@ -102,8 +107,9 @@ public:
 	/// Drop 'count' values off the top of the stack.
 	void drop(size_t count)
 	{
-		assert ( m_stack.size() >= count );
-		m_stack.resize(m_stack.size() - count);
+		size_t ssize = m_stack.size();
+		assert ( ssize >= count );
+		m_stack.resize(ssize - count);
 	}
 
 	/// Insert 'count' undefined values before 'offset'.
@@ -257,28 +263,13 @@ public:
 	/// Create the specified local var if it doesn't exist already.
 	void	declare_local(const std::string& varname);
 
-	/// Retrieve the named member (variable).
-	//
-	/// If no member is found under the given name
-	/// 'val' is untouched and 'false' is returned.
-	/// 
-	/// TODO: rename to get_variable
-	///
-	bool	get_member(const std::string& varname, as_value* val) const;
-
-	/// Set the named variable 
-	//
-	/// TODO: rename to set_variable, take a std::string
-	///
-	void	set_member(const std::string& varname, const as_value& val);
-
 	/// Add 'count' local registers (add space to end)
 	//
 	/// Local registers are only meaningful within a function2 context.
 	///
 	void add_local_registers(unsigned int register_count)
 	{
-		assert(_localFrames.size());
+		assert(!_localFrames.empty());
 		return _localFrames.back().registers.resize(register_count);
 	}
 
@@ -288,7 +279,7 @@ public:
 	///
 	size_t num_local_registers() const
 	{
-		assert(_localFrames.size());
+		assert(!_localFrames.empty());
 		return _localFrames.back().registers.size();
 	}
 
@@ -296,16 +287,16 @@ public:
 	//
 	/// Local registers are only meaningful within a function2 context.
 	///
-	as_value& local_register(uint8_t n)
+	as_value& local_register(boost::uint8_t n)
 	{
-		assert(_localFrames.size());
+		assert(!_localFrames.empty());
 		return _localFrames.back().registers[n];
 	}
 
         /// Set the Nth local register to something
-        void set_local_register(uint8_t n, as_value &val)
+        void set_local_register(boost::uint8_t n, as_value &val)
 	{
-		if (_localFrames.size() )
+		if ( ! _localFrames.empty() )
 		{
 			Registers& registers = _localFrames.back().registers;
 			if ( n < registers.size() )
@@ -323,7 +314,7 @@ public:
 	}
 
         /// Set the Nth local register to something
-        void set_global_register(uint8_t n, as_value &val) {
+        void set_global_register(boost::uint8_t n, as_value &val) {
 	    if (n <= 4) {
 		m_global_register[n] = val;
 	    }
@@ -342,18 +333,16 @@ public:
 	/// Find the sprite/movie referenced by the given path.
 	//
 	/// Supports both /slash/syntax and dot.syntax
+	/// Case insensitive for SWF up to 6, sensitive from 7 up
 	///
 	character* find_target(const std::string& path) const;
 
-	/// \brief
-	/// Find the sprite/movie represented by the given value.
+	/// Find the object referenced by the given path.
 	//
-	/// The value might be a reference to the object itself, or a
-	/// string giving a relative path name to the object.
+	/// Supports both /slash/syntax and dot.syntax
+	/// Case insensitive for SWF up to 6, sensitive from 7 up
 	///
-	/// @@ make private ? --strk;
-	///
-	character* find_target(const as_value& val) const;
+	as_object* find_object(const std::string& path, const ScopeStack* scopeStack=NULL) const;
 
 	/// Dump content of the stack to a std::ostream
 	void dump_stack(std::ostream& out=std::cerr)
@@ -388,30 +377,26 @@ public:
 	///
 	int get_version() const;
 
-	// See if the given variable name is actually a sprite path
-	// followed by a variable name.  These come in the format:
-	//
-	// 	/path/to/some/sprite/:varname
-	//
-	// (or same thing, without the last '/')
-	//
-	// or
-	//	path.to.some.var
-	//
-	// If that's the format, puts the path part (no colon or
-	// trailing slash) in *path, and the varname part (no colon, no dot)
-	// in *var and returns true.
-	//
-	// If no colon or dot, returns false and leaves *path & *var alone.
-	//
-	/// @param is_slash_based
-	///	If not null gets set to true if path is slash-based
-	///	(path/to/:variable), and to false if path is dot-based
-	///	(path.to.variable).
+	/// See if the given variable name is actually a sprite path
+	/// followed by a variable name.  These come in the format:
+	///
+	///	/path/to/some/sprite/:varname
+	///
+	/// (or same thing, without the last '/')
+	///
+	/// or
+	///	path.to.some.var
+	///
+	/// If that's the format, puts the path part (no colon or
+	/// trailing slash) in *path, and the varname part (no colon, no dot)
+	/// in *var and returns true.
+	///
+	/// If no colon or dot, returns false and leaves *path & *var alone.
 	///
 	/// TODO: return an integer: 0 not a path, 1 a slash-based path, 2 a dot-based path
+	///
 	static bool parse_path(const std::string& var_path, std::string& path,
-		std::string& var, bool* is_slash_based=NULL);
+			std::string& var);
 
 	/// \brief
 	/// Try to parse a string as a variable path
@@ -420,12 +405,14 @@ public:
 	///
 	/// 	/path/to/some/sprite/:varname
 	///
-	/// (or same thing, without the last '/')
+	/// or
+	///
+	///     /path/to/some/sprite 
 	///
 	/// or
 	///	path.to.some.var
 	///
-	/// If there's no dot nor comma, or if the 'path' part
+	/// If there's no dot nor colon, or if the 'path' part
 	/// does not resolve to an object, this function returns false.
 	/// Otherwise, true is returned and 'target' and 'val'
 	/// parameters are appropriaterly set.
@@ -449,6 +436,10 @@ public:
 	{
 		CallFrame(as_function* funcPtr);
 
+		CallFrame(const CallFrame& other) : locals(other.locals),
+			registers(other.registers), func(other.func)
+		{/**/}
+
 		/// function use this 
 		LocalVars locals;
 
@@ -466,6 +457,47 @@ public:
 #endif // GNASH_USE_GC
 	};
 
+	/// A class to wrap frame access.  Stack allocating a frame guard
+	/// will ensure that all CallFrame pushes have a corresponding
+	/// CallFrame pop, even in the presence of extraordinary returns.
+	class FrameGuard
+	{
+	public:
+		FrameGuard(as_function* func)
+		{ as_environment::pushCallFrame(func); }
+		~FrameGuard() { as_environment::popCallFrame(); }
+	};
+
+	/// Get top element of the call stack
+	//
+	CallFrame& topCallFrame()
+	{
+		assert(!_localFrames.empty());
+		return _localFrames.back();
+	}
+
+	/// Return the depth of call stack
+	size_t callStackDepth()
+	{
+		return _localFrames.size();
+	}
+
+private:
+
+	static const short unsigned int numGlobalRegisters = 4;
+
+	typedef std::vector<CallFrame> CallStack;
+		
+	static CallStack _localFrames;
+
+	as_value m_global_register[numGlobalRegisters];
+
+	/// Movie target. 
+	character* m_target;
+
+	/// Movie target. 
+	character* _original_target;
+
 	/// Push a frame on the calls stack.
 	//
 	/// This should happen right before calling an ActionScript
@@ -477,55 +509,14 @@ public:
 	/// @param func
 	///	The function being called
 	///
-	void pushCallFrame(as_function* func);
+	static void pushCallFrame(as_function* func);
 
 	/// Remove current call frame from the stack
 	//
 	/// This should happen when an ActionScript function returns.
 	///
-	void popCallFrame()
-	{
-		assert(_localFrames.size());
-		_localFrames.pop_back();
-	}
-
-	/// Get top element of the call stack
-	//
-	CallFrame& topCallFrame()
-	{
-		assert(_localFrames.size());
-		return _localFrames.back();
-	}
-
-	/// Return the depth of call stack
-	size_t callStackDepth()
-	{
-		return _localFrames.size();
-	}
-
-	/// Clear the call stack
-	void clearCallFrames()
-	{
-		_localFrames.clear();
-	}
-
-private:
-
-	typedef std::vector<CallFrame> CallStack;
-		
-	CallStack _localFrames;
-
-	/// Variables available in this environment
-	Variables _variables;
-
-	as_value m_global_register[4];
-
-	/// Movie target. 
-	character* m_target;
-
-	/// Movie target. 
-	character* _original_target;
-
+	static void popCallFrame();
+	
 	/// Return the (possibly UNDEFINED) value of the named variable.
 	//
 	/// @param varname 
@@ -627,22 +618,6 @@ private:
 	///
 	static bool setLocal(LocalVars& locals, const std::string& varname, const as_value& val);
 
-	/// Find an object referenced by the given path (slash syntax).
-	//
-	/// Supports /slash/syntax 
-	///
-	/// Return NULL if path doesn't point to an object.
-	///
-	as_object* find_object_slashsyntax(const std::string& path) const;
-
-	/// Find an object referenced by the given path (dot syntax).
-	//
-	/// Supports dot.syntax
-	///
-	/// Return NULL if path doesn't point to an object.
-	///
-	as_object* find_object_dotsyntax(const std::string& path) const;
-
 };
 
 
