@@ -1,6 +1,6 @@
 // xml.cpp:  XML markup language support, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,10 +17,10 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: xml.cpp,v 1.44 2007/07/01 10:54:32 bjacques Exp $ */
+/* $Id: xml.cpp,v 1.75 2008/02/19 19:20:55 bwy Exp $ */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "log.h"
@@ -39,35 +39,33 @@
 #include "tu_file.h"
 #include "URL.h"
 #include "VM.h"
+#include "namedStrings.h"
+#include "timers.h" // for setting up timers to check loads
 
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 #include <libxml/xmlreader.h>
-#include <unistd.h>
-#include <sys/types.h>
-#include <sys/stat.h>
+//#include <unistd.h>
+//#include <sys/types.h>
+//#include <sys/stat.h>
 #include <string>
 #include <sstream>
 #include <vector>
-//#include <boost/lexical_cast.hpp>
 #include <boost/algorithm/string/case_conv.hpp>
 #include <memory>
 
-using namespace std;
-
 namespace gnash {
   
 //#define DEBUG_MEMORY_ALLOCATION 1
 
+// Define this to enable verbosity of XML loads
+//#define DEBUG_XML_LOADS 1
+
 static as_object* getXMLInterface();
 static void attachXMLInterface(as_object& o);
 static void attachXMLProperties(as_object& o);
 
-// Callback functions for xmlReadIO
-static int closeTuFile (void * context);
-static int readFromTuFile (void * context, char * buffer, int len);
-
 DSOEXPORT as_value xml_new(const fn_call& fn);
 static as_value xml_load(const fn_call& fn);
 static as_value xml_addrequestheader(const fn_call& fn);
@@ -78,8 +76,8 @@ static as_value xml_getbytestotal(const fn_call& fn);
 static as_value xml_parsexml(const fn_call& fn);
 static as_value xml_send(const fn_call& fn);
 static as_value xml_sendandload(const fn_call& fn);
+static as_value xml_ondata(const fn_call& fn);
 
-static LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
 #ifdef USE_DEBUGGER
 static Debugger& debugger = Debugger::getDefaultInstance();
 #endif
@@ -87,16 +85,20 @@ static Debugger& debugger = Debugger::getDefaultInstance();
 XML::XML() 
     :
     XMLNode(getXMLInterface()),
+    _doc(0),
+    _firstChild(0),
     _loaded(-1), 
-    _bytes_loaded(0),
-    _bytes_total(0),
-    _status(sOK)
+    _status(sOK),
+    _loadThreads(),
+    _loadCheckerTimer(0),
+    _bytesTotal(-1),
+    _bytesLoaded(-1)
 {
     //GNASH_REPORT_FUNCTION;
 #ifdef DEBUG_MEMORY_ALLOCATION
-    log_msg(_("Creating XML data at %p"), this);
+    log_debug(_("Creating XML data at %p"), this);
 #endif
-    //log_msg("%s: %p", __FUNCTION__, this);
+    //log_debug("%s: %p", __FUNCTION__, this);
     attachXMLProperties(*this);
 }
 
@@ -105,131 +107,127 @@ XML::XML()
 XML::XML(const std::string& xml_in)
     :
     XMLNode(getXMLInterface()),
+    _doc(0),
+    _firstChild(0),
     _loaded(-1), 
-    _bytes_loaded(0),
-    _bytes_total(0),
-    _status(sOK)
+    _status(sOK),
+    _loadThreads(),
+    _loadCheckerTimer(0),
+    _bytesTotal(-1),
+    _bytesLoaded(-1)
 {
     //GNASH_REPORT_FUNCTION;
 #ifdef DEBUG_MEMORY_ALLOCATION
-    log_msg(_("Creating XML data at %p"), this);
+    log_debug(_("Creating XML data at %p"), this);
 #endif
     parseXML(xml_in);
 }
 
-XML::XML(struct node * /* childNode */)
-    :
-    XMLNode(getXMLInterface()),
-    _loaded(-1), 
-    _bytes_loaded(0),
-    _bytes_total(0),
-    _status(sOK)
-{
-    GNASH_REPORT_FUNCTION;
-#ifdef DEBUG_MEMORY_ALLOCATION
-    log_msg(_("\tCreating XML data at %p"), this);
-#endif
-    //log_msg("%s: %p", __FUNCTION__, this);
-}
-
 bool
-XML::get_member(const std::string& name, as_value *val)
+XML::get_member(string_table::key name, as_value *val, string_table::key nsname)
 {
-        if ( name == "status" ) 
+        if (name == NSV::PROP_STATUS) 
         {
                 val->set_int(_status);
                 return true;
         }
-        else if ( name == "loaded" )
+        else if (name == NSV::PROP_LOADED)
         {
                 if ( _loaded < 0 ) val->set_undefined();
                 else val->set_bool(_loaded);
                 return true;
         }
 
-        return get_member_default(name, val);
+        return get_member_default(name, val, nsname);
 }
 
 void
-XML::set_member(const std::string& name, const as_value& val)
+XML::set_member(string_table::key name, const as_value& val, 
+	string_table::key nsname)
 {
-        if ( name == "status" )
+        if (name == NSV::PROP_STATUS)
 	{
-		_status = XML::Status(val.to_number());
+		if ( ! val.is_number() )
+		{
+			_status = static_cast<XML::Status>(std::numeric_limits<boost::int32_t>::min());
+		}
+		else
+		{
+			unsigned int statusNumber = static_cast<int>(val.to_number());
+			_status = XML::Status( static_cast<XML::Status>(statusNumber) );
+		}
 		return;
 	}
-        else if ( name == "loaded" )
+        else if (name == NSV::PROP_LOADED)
         {
                 bool b = val.to_bool();
-		log_msg(_("set_member 'loaded' (%s) became boolean %d"), val.to_debug_string().c_str(), b);
+		//log_debug(_("set_member 'loaded' (%s) became boolean %d"), val.to_debug_string().c_str(), b);
                 if ( b ) _loaded = 1;
                 else _loaded = 0;
                 return;
         }
 
-        set_member_default(name, val);
+        set_member_default(name, val, nsname);
 }
 
 XML::~XML()
 {
-    GNASH_REPORT_FUNCTION;
+    //GNASH_REPORT_FUNCTION;
+
+    for (LoadThreadList::iterator it=_loadThreads.begin(); it != _loadThreads.end(); ++it)
+    {
+        delete *it; // supposedly joins the thread
+    }
+
+    if ( _loadCheckerTimer )
+    {
+        VM& vm = getVM();
+        vm.getRoot().clear_interval_timer(_loadCheckerTimer);
+    }
     
 #ifdef DEBUG_MEMORY_ALLOCATION
-    log_msg(_("\tDeleting XML top level node at %p"), this);
+    log_debug(_("\tDeleting XML top level node at %p"), this);
 #endif
   
 }
 
 void
-XML::onLoadEvent(bool success)
+XML::onLoadEvent(bool success, as_environment& env)
 {
     // Do the events that (appear to) happen as the movie
     // loads.  frame1 tags and actions are executed (even
     // before advance() is called).  Then the onLoad event
     // is triggered.
 
-    // In ActionScript 2.0, event method names are CASE SENSITIVE.
-    // In ActionScript 1.0, event method names are CASE INSENSITIVE.
-    // TODO: move to get_function_name directly ?
-    std::string method_name = "onLoad";
-    if ( _vm.getSWFVersion() < 7 )
-        boost::to_lower(method_name, _vm.getLocale());
-
-    if ( method_name.empty() ) return;
-
     as_value	method;
-    if ( ! get_member(method_name, &method) ) return;
+    if (!get_member(NSV::PROP_ON_LOAD, &method) ) return;
     if ( method.is_undefined() ) return;
     if ( ! method.is_function() ) return;
 
-    as_environment env; // how to set target here ??
+#ifndef NDEBUG
+    size_t prevStackSize = env.stack_size();
+#endif
     env.push(as_value(success));
     call_method(method, &env, this, 1, env.stack_size()-1);
+    env.drop(1);
+#ifndef NDEBUG
+    assert( prevStackSize == env.stack_size());
+#endif
 }
 
 void
-XML::onCloseEvent()
+XML::onCloseEvent(as_environment& env)
 {
     // Do the events that (appear to) happen as the movie
     // loads.  frame1 tags and actions are executed (even
     // before advance() is called).  Then the onLoad event
     // is triggered.
 
-    // In ActionScript 2.0, event method names are CASE SENSITIVE.
-    // In ActionScript 1.0, event method names are CASE INSENSITIVE.
-    // TODO: move to get_function_name directly ?
-    std::string method_name = "onClose";
-    if ( _vm.getSWFVersion() < 7 )
-        boost::to_lower(method_name, _vm.getLocale());
-
-    if ( method_name.empty() ) return;
-
     as_value	method;
-    if ( ! get_member(method_name, &method) ) return;
+    if (! get_member(NSV::PROP_ON_CLOSE, &method) ) return;
     if ( method.is_undefined() ) return;
     if ( ! method.is_function() ) return;
 
-    as_environment env; // how to set target here ??
     call_method(method, &env, this, 0, 0);
 }
 
@@ -240,31 +238,43 @@ XML::extractNode(XMLNode& element, xmlNodePtr node, bool mem)
     xmlChar *ptr = NULL;
     boost::intrusive_ptr<XMLNode> child;
 
-//    log_msg(_("Created new element for %s at %p"), node->name, element);
+//    log_debug(_("Created new element for %s at %p"), node->name, element);
 
-//    log_msg(_("%s: extracting node %s"), __FUNCTION__, node->name);
+//    log_debug(_("%s: extracting node %s"), __FUNCTION__, node->name);
 
     // See if we have any Attributes (properties)
     attr = node->properties;
     while (attr != NULL)
     {
-        //log_msg(_("extractNode %s has property %s, value is %s"),
+        //log_debug(_("extractNode %s has property %s, value is %s"),
         //          node->name, attr->name, attr->children->content);
-        XMLAttr attrib(reinterpret_cast<const char*>(attr->name),
-			reinterpret_cast<const char*>(attr->children->content));
+        
+        std::ostringstream name, content;
 
-        //log_msg(_("\tPushing attribute %s for element %s has value %s"),
+        name << attr->name;
+        content << attr->children->content;
+        
+        XMLAttr attrib(name.str(), content.str());
+
+        //log_debug(_("\tPushing attribute %s for element %s has value %s"),
         //        attr->name, node->name, attr->children->content);
         element._attributes.push_back(attrib);
         attr = attr->next;
     }
-
-    if (node->type == XML_ELEMENT_NODE)
+    if (node->type == XML_COMMENT_NODE)
+    {
+    	// Comments apparently not handled until AS3
+    	// Comments in a text node are a *sibling* of the text node
+    	// for libxml2.
+    	return false;
+    }
+    else if (node->type == XML_ELEMENT_NODE)
     {
             element.nodeTypeSet(tElement);
 
-            std::string name(reinterpret_cast<const char*>(node->name));
-            element.nodeNameSet(name);
+            std::ostringstream name;
+            name << node->name;
+            element.nodeNameSet(name.str());
     }
     else if ( node->type == XML_TEXT_NODE )
     {
@@ -274,19 +284,20 @@ XML::extractNode(XMLNode& element, xmlNodePtr node, bool mem)
             if (ptr == NULL) return false;
 	    if (node->content)
 	    {
-		const char* in = reinterpret_cast<const char*>(ptr);
+		std::ostringstream in;
+		in << ptr;
 		// XML_PARSE_NOBLANKS seems not to be working, so here's
 		// a custom implementation of it.
 		if ( ignoreWhite() )
 		{
-			if ( strspn(in, " \n\t\r") == strlen(in) )
+			if ( in.str().find_first_not_of(" \n\t\r") == std::string::npos )
 			{
-				log_msg("Text node value consists in blanks only, discarding");
+				log_debug("Text node value consists in blanks only, discarding");
+				xmlFree(ptr);
 				return false;
 			}
 		}
-		std::string val(in);
-		element.nodeValueSet(val);
+		element.nodeValueSet(in.str());
 	    }
             xmlFree(ptr);
     }
@@ -298,8 +309,7 @@ XML::extractNode(XMLNode& element, xmlNodePtr node, bool mem)
     {
         child = new XMLNode();
         child->setParent(&element);
-        if ( ! extractNode(*child, childnode, mem) ) break;
-        element._children.push_back(child);
+        if ( extractNode(*child, childnode, mem) ) element._children.push_back(child);
         childnode = childnode->next;
     }
 
@@ -341,7 +351,7 @@ XML::parseXML(const std::string& xml_in)
 {
     //GNASH_REPORT_FUNCTION;
 
-    //log_msg(_("Parse XML from memory: %s"), xml_in.c_str());
+    //log_debug(_("Parse XML from memory: %s"), xml_in.c_str());
 
     if (xml_in.empty()) {
         log_error(_("XML data is empty"));
@@ -378,6 +388,147 @@ XML::parseXML(const std::string& xml_in)
   
 }
 
+void
+XML::queueLoad(std::auto_ptr<tu_file> str)
+{
+    //GNASH_REPORT_FUNCTION;
+
+    // Set the "loaded" parameter to false
+    VM& vm = getVM();
+    string_table& st = vm.getStringTable();
+    string_table::key loadedKey = st.find("loaded");
+    set_member(loadedKey, as_value(false));
+
+    bool startTimer = _loadThreads.empty();
+
+    std::auto_ptr<LoadThread> lt ( new LoadThread );
+    lt->setStream(str);
+
+    // we push on the front to avoid invalidating
+    // iterators when queueLoad is called as effect
+    // of onData invocation.
+    // Doing so also avoids processing queued load
+    // request immediately
+    // 
+    _loadThreads.push_front(lt.get());
+#ifdef DEBUG_XML_LOADS
+    log_debug("Pushed thread %p to _loadThreads, number of XML load threads now: " SIZET_FMT, (void*)lt.get(),  _loadThreads.size());
+#endif
+    lt.release();
+
+
+    if ( startTimer )
+    {
+        boost::intrusive_ptr<builtin_function> loadsChecker = \
+            new builtin_function(&XML::checkLoads_wrapper);
+        std::auto_ptr<Timer> timer(new Timer);
+        timer->setInterval(*loadsChecker, 50, this);
+        _loadCheckerTimer = getVM().getRoot().add_interval_timer(timer, true);
+#ifdef DEBUG_XML_LOADS
+        log_debug("Registered XML loads interval %d", _loadCheckerTimer);
+#endif
+    }
+
+    _bytesLoaded = 0;
+    _bytesTotal = -1;
+
+}
+
+long int
+XML::getBytesLoaded() const
+{
+    return _bytesLoaded;
+}
+
+long int
+XML::getBytesTotal() const
+{
+    return _bytesTotal;
+}
+
+/* private */
+void
+XML::checkLoads()
+{
+#ifdef DEBUG_XML_LOADS
+    static int call=0;
+    log_debug("XML %p checkLoads call %d, _loadThreads: %d", (void *)this, _loadThreads.size(), ++call);
+#endif
+
+    if ( _loadThreads.empty() ) return; // nothing to do
+
+    string_table::key onDataKey = NSV::PROP_ON_DATA;
+
+    for (LoadThreadList::iterator it=_loadThreads.begin();
+            it != _loadThreads.end(); )
+    {
+        LoadThread* lt = *it;
+
+        // TODO: notify progress 
+
+	_bytesLoaded = lt->getBytesLoaded();
+        _bytesTotal = lt->getBytesTotal();
+
+#ifdef DEBUG_XML_LOADS
+        log_debug("XML loads thread %p got %ld/%ld bytes", (void*)lt, lt->getBytesLoaded(), lt->getBytesTotal() );
+#endif
+        if ( lt->completed() )
+        {
+            size_t xmlsize = lt->getBytesLoaded();
+            boost::scoped_array<char> buf(new char[xmlsize+1]);
+            size_t actuallyRead = lt->read(buf.get(), xmlsize);
+            if ( actuallyRead != xmlsize )
+			{
+				// This would be either a bug of LoadThread or an expected
+				// possibility which lacks documentation (thus a bug in documentation)
+				//
+#ifdef DEBUG_XML_LOADS
+				log_debug("LoadThread::getBytesLoaded() returned %d but ::read(%d) returned %d",
+					xmlsize, xmlsize, actuallyRead);
+#endif
+			}
+            buf[actuallyRead] = '\0';
+            as_value dataVal(buf.get()); // memory copy here (optimize?)
+
+            it = _loadThreads.erase(it);
+            delete lt; // supposedly joins the thread...
+
+            // might push_front on the list..
+            callMethod(onDataKey, dataVal);
+
+#ifdef DEBUG_XML_LOADS
+            log_debug("Completed load, _loadThreads have now " SIZET_FMT " elements", _loadThreads.size());
+#endif
+        }
+        else
+        {
+            ++it;
+        }
+    }
+
+    if ( _loadThreads.empty() ) 
+    {
+#ifdef DEBUG_XML_LOADS
+        log_debug("Clearing XML load checker interval timer");
+#endif
+    	VM& vm = getVM();
+        vm.getRoot().clear_interval_timer(_loadCheckerTimer);
+        _loadCheckerTimer=0;
+    }
+}
+
+/* private static */
+as_value
+XML::checkLoads_wrapper(const fn_call& fn)
+{
+#ifdef DEBUG_XML_LOADS
+    log_debug("checkLoads_wrapper called");
+#endif
+
+	boost::intrusive_ptr<XML> ptr = ensureType<XML>(fn.this_ptr);
+	ptr->checkLoads();
+	return as_value();
+}
 
 // This reads in an XML file from disk and parses into into a memory resident
 // tree which can be walked through later.
@@ -386,66 +537,29 @@ XML::load(const URL& url)
 {
     GNASH_REPORT_FUNCTION;
   
-    //log_msg(_("%s: mem is %d"), __FUNCTION__, mem);
-
-    // Clear current data
-    clear(); 
+    //log_debug(_("%s: mem is %d"), __FUNCTION__, mem);
 
     std::auto_ptr<tu_file> str ( StreamProvider::getDefaultInstance().getStream(url) );
     if ( ! str.get() ) 
     {
         log_error(_("Can't load XML file: %s (security?)"), url.str().c_str());
-        onLoadEvent(false);
         return false;
+        // TODO: this is still not correct.. we should still send onData later...
+        //as_value nullValue; nullValue.set_null();
+        //callMethod(NSV::PROP_ON_DATA, nullValue);
     }
 
-    log_msg(_("Loading XML file from url: '%s'"), url.str().c_str());
+    log_security(_("Loading XML file from url: '%s'"), url.str().c_str());
+    queueLoad(str);
 
-    initParser();
-
-    int options = getXMLOptions();
-
-    _doc = xmlReadIO(readFromTuFile, closeTuFile, str.get(), url.str().c_str(), NULL, options);
-    if ( str->get_error() )
-    {
-	xmlFreeDoc(_doc);
-        _doc = 0;
-        log_error(_("Can't read XML file %s (stream error %d)"), url.str().c_str(), str->get_error());
-        _loaded = 0;
-        onLoadEvent(false);
-        return false;
-    }
-
-    _bytes_total = str->get_size();
-
-    if (_doc == 0)
-    {
-        xmlErrorPtr err = xmlGetLastError();
-        log_error(_("Can't read XML file %s (%s)"), url.str().c_str(), err->message);
-        _loaded = 0;
-        onLoadEvent(false);
-        return false;
-    }
-
-    _bytes_loaded = _bytes_total;
-
-    bool ret = parseDoc(_doc, false);
-
-    xmlCleanupParser();
-    xmlFreeDoc(_doc);
-    xmlMemoryDump();
-    _loaded = ret ? 1 : 0;
-
-    onLoadEvent(ret);
-
-    return ret;
+    return true;
 }
 
 
 bool
 XML::onLoad()
 {
-    log_msg(_("%s: FIXME: onLoad Default event handler"), __FUNCTION__);
+    log_debug(_("%s: FIXME: onLoad Default event handler"), __FUNCTION__);
 
     return(_loaded);
 }
@@ -473,28 +587,46 @@ XML::addRequestHeader(const char * /* name */, const char * /* value */)
 }
 
 
-void
-XML::load()
-{
-    log_unimpl (__FUNCTION__);
-}
-
-void
-XML::parseXML()
-{
-    log_unimpl (__FUNCTION__);
-}
-
 void
 XML::send()
 {
     log_unimpl (__FUNCTION__);
 }
 
-void
-XML::sendAndLoad()
+bool
+XML::sendAndLoad(const URL& url, XML& target)
 {
-    log_unimpl (__FUNCTION__);
+    //GNASH_REPORT_FUNCTION;
+
+    std::stringstream ss;
+    toString(ss);
+    const std::string& data = ss.str();
+
+    VM& vm = getVM();
+    string_table& st = vm.getStringTable();
+    string_table::key ctypeKey = st.find("contentType");
+    as_value ctypeVal;
+    if ( get_member(ctypeKey, &ctypeVal) )
+    {
+       log_unimpl ("Custom ContentType (%s) in XML.sendAndLoad", ctypeVal.to_debug_string().c_str());
+    }
+  
+    //log_debug(_("%s: mem is %d"), __FUNCTION__, mem);
+
+    std::auto_ptr<tu_file> str ( StreamProvider::getDefaultInstance().getStream(url, data) );
+    if ( ! str.get() ) 
+    {
+        log_error(_("Can't load XML file: %s (security?)"), url.str().c_str());
+        return false;
+        // TODO: this is still not correct.. we should still send onData later...
+        //as_value nullValue; nullValue.set_null();
+        //callMethod(NSV::PROP_ON_DATA, nullValue);
+    }
+
+    log_security(_("Loading XML file from url: '%s'"), url.str().c_str());
+    target.queueLoad(str);
+
+    return true;
 }
 
 
@@ -514,7 +646,15 @@ xml_load(const fn_call& fn)
   
     boost::intrusive_ptr<XML> xml_obj = ensureType<XML>(fn.this_ptr);
   
-    const std::string& filespec = fn.arg(0).to_string(&(fn.env()));
+    if ( ! fn.nargs )
+    {
+        IF_VERBOSE_ASCODING_ERRORS(
+        log_aserror(_("XML.load(): missing argument"));
+        );
+        return rv;
+    }
+
+    const std::string& filespec = fn.arg(0).to_string();
 
     URL url(filespec, get_base_url());
 
@@ -551,6 +691,7 @@ attachXMLInterface(as_object& o)
     o.init_member("parseXML", new builtin_function(xml_parsexml));
     o.init_member("send", new builtin_function(xml_send));
     o.init_member("sendAndLoad", new builtin_function(xml_sendandload));
+    o.init_member("onData", new builtin_function(xml_ondata));
 
 }
 
@@ -573,22 +714,22 @@ xml_new(const fn_call& fn)
     boost::intrusive_ptr<XML> xml_obj;
     //const char    *data;
   
-    // log_msg(_("%s: nargs=%d"), __FUNCTION__, fn.nargs);
+    // log_debug(_("%s: nargs=%d"), __FUNCTION__, fn.nargs);
   
     if ( fn.nargs > 0 )
     {
         if ( fn.arg(0).is_object() )
         {
-            boost::intrusive_ptr<as_object> obj = fn.env().top(0).to_object();
+            boost::intrusive_ptr<as_object> obj = fn.arg(0).to_object();
             xml_obj = boost::dynamic_pointer_cast<XML>(obj);
             if ( xml_obj )
             {
-                log_msg(_("\tCloned the XML object at %p"), (void *)xml_obj.get());
+                log_debug(_("\tCloned the XML object at %p"), (void *)xml_obj.get());
                 return as_value(xml_obj->cloneNode(true).get());
             }
         }
 
-        const std::string& xml_in = fn.arg(0).to_string(&(fn.env()));
+        const std::string& xml_in = fn.arg(0).to_string();
         if ( xml_in.empty() )
         {
             IF_VERBOSE_ASCODING_ERRORS(
@@ -604,7 +745,7 @@ xml_new(const fn_call& fn)
     }
 
     xml_obj = new XML;
-    //log_msg(_("\tCreated New XML object at %p"), xml_obj);
+    //log_debug(_("\tCreated New XML object at %p"), xml_obj);
 
     return as_value(xml_obj.get());
 }
@@ -617,7 +758,7 @@ xml_new(const fn_call& fn)
 as_value xml_addrequestheader(const fn_call& fn)
 {
     GNASH_REPORT_FUNCTION;
-    log_msg(_("%s: %d args"), __PRETTY_FUNCTION__, fn.nargs);
+    log_debug(_("%s: %d args"), __PRETTY_FUNCTION__, fn.nargs);
     
 //    return as_value(ptr->getAllocated());
 //    ptr->addRequestHeader();
@@ -640,12 +781,10 @@ xml_createelement(const fn_call& fn)
 //    GNASH_REPORT_FUNCTION;
     
     if (fn.nargs > 0) {
-        const std::string& text = fn.arg(0).to_string(&(fn.env()));
+        const std::string& text = fn.arg(0).to_string();
 	XMLNode *xml_obj = new XMLNode();
-//	cerr << "create new child XMLNode is at " << (void *)xml_obj << endl;
 	xml_obj->nodeNameSet(text);
 	xml_obj->nodeTypeSet(XMLNode::tText);
-//	ptr->set_member(text, xml_obj); // FIXME: use a getter/setter !
 	// no return code from this method
 	return as_value(xml_obj);
 	
@@ -672,12 +811,12 @@ xml_createtextnode(const fn_call& fn)
     XMLNode *xml_obj;
 
     if (fn.nargs > 0) {
-	const std::string& text = fn.arg(0).to_string(&(fn.env()));
+	const std::string& text = fn.arg(0).to_string();
 	xml_obj = new XMLNode;
 	xml_obj->nodeValueSet(text);
 	xml_obj->nodeTypeSet(XMLNode::tText);
 	return as_value(xml_obj);
-//	log_msg(_("%s: xml obj is %p"), __PRETTY_FUNCTION__, xml_obj);
+//	log_debug(_("%s: xml obj is %p"), __PRETTY_FUNCTION__, xml_obj);
     } else {
 	log_error(_("no text for text node creation"));
     }
@@ -686,28 +825,18 @@ xml_createtextnode(const fn_call& fn)
 
 as_value xml_getbytesloaded(const fn_call& fn)
 {
-    boost::intrusive_ptr<XML> ptr = ensureType<XML>(fn.this_ptr);
-    if ( ptr->loaded() )
-	{
-		return as_value(ptr->getBytesLoaded());
-    }
-	else
-	{
-		return as_value();
-    }
+	boost::intrusive_ptr<XML> ptr = ensureType<XML>(fn.this_ptr);
+	long int ret = ptr->getBytesLoaded();
+	if ( ret < 0 ) return as_value();
+	else return as_value(ret);
 }
 
 as_value xml_getbytestotal(const fn_call& fn)
 {
-    boost::intrusive_ptr<XML> ptr = ensureType<XML>(fn.this_ptr);
-    if ( ptr->loaded() )
-	{
-		return as_value(ptr->getBytesTotal());
-	}
-	else
-	{
-		return as_value();
-	}
+	boost::intrusive_ptr<XML> ptr = ensureType<XML>(fn.this_ptr);
+	long int ret = ptr->getBytesTotal();
+	if ( ret < 0 ) return as_value();
+	else return as_value(ret);
 }
 
 as_value xml_parsexml(const fn_call& fn)
@@ -725,7 +854,7 @@ as_value xml_parsexml(const fn_call& fn)
         return as_value();
     }
 
-    const std::string& text = fn.arg(0).to_string(&(fn.env()));
+    const std::string& text = fn.arg(0).to_string();
     ptr->parseXML(text);
     
     return as_value();
@@ -750,8 +879,81 @@ xml_sendandload(const fn_call& fn)
     GNASH_REPORT_FUNCTION;
     boost::intrusive_ptr<XML> ptr = ensureType<XML>(fn.this_ptr);
     
+    if ( fn.nargs < 2 )
+    {
+        IF_VERBOSE_ASCODING_ERRORS(
+        std::stringstream ss;
+        fn.dump_args(ss);
+        log_aserror(_("XML.sendAndLoad(%s): missing arguments"),
+		ss.str().c_str());
+        );
+        return as_value(false);
+    }
+
+    const std::string& filespec = fn.arg(0).to_string();
+
+    boost::intrusive_ptr<as_object> targetObj = fn.arg(1).to_object();
+    if ( ! targetObj )
+    {
+        IF_VERBOSE_ASCODING_ERRORS(
+        std::stringstream ss;
+        fn.dump_args(ss);
+        log_aserror(_("XML.sendAndLoad(%s): second argument doesn't cast to an object"),
+		ss.str().c_str());
+        );
+        return as_value(false);
+    }
+    XML* target = dynamic_cast<XML*>(targetObj.get());
+    if ( ! target )
+    {
+        IF_VERBOSE_ASCODING_ERRORS(
+        std::stringstream ss;
+        fn.dump_args(ss);
+        log_aserror(_("XML.sendAndLoad(%s): second argument is not an XML object"),
+		ss.str().c_str());
+        );
+        return as_value(false);
+    }
+
+    URL url(filespec, get_base_url());
+
 //    return as_value(ptr->getAllocated());
-    ptr->sendAndLoad();
+    bool ret = ptr->sendAndLoad(url, *target);
+
+    return ret; // TODO: check expected return values
+}
+
+static as_value
+xml_ondata(const fn_call& fn)
+{
+    GNASH_REPORT_FUNCTION;
+
+    string_table::key onLoadKey = NSV::PROP_ON_LOAD;
+    string_table::key loadedKey = NSV::PROP_LOADED; 
+
+    as_object* thisPtr = fn.this_ptr.get();
+    assert(thisPtr);
+
+    // See http://gitweb.freedesktop.org/?p=swfdec/swfdec.git;a=blob;f=libswfdec/swfdec_initialize.as
+
+    as_value src; src.set_null();
+    if ( fn.nargs ) src = fn.arg(0);
+
+    if ( ! src.is_null() )
+    {
+        string_table::key parseXMLKey = NSV::PROP_PARSE_XML;
+        as_value tmp(true);
+        thisPtr->set_member(loadedKey, tmp);
+        thisPtr->callMethod(parseXMLKey, src);
+        thisPtr->callMethod(onLoadKey, tmp);
+    }
+    else
+    {
+        as_value tmp(true);
+        thisPtr->set_member(loadedKey, tmp);
+        thisPtr->callMethod(onLoadKey, tmp);
+    }
+
     return as_value();
 }
 
@@ -778,26 +980,6 @@ void xml_class_init(as_object& global)
 
 }
 
-// Callback function for xmlReadIO
-static int
-readFromTuFile (void * context, char * buffer, int len)
-{
-        tu_file* str = static_cast<tu_file*>(context);
-        size_t read = str->read_bytes(buffer, len);
-        if ( str->get_error() ) return -1;
-        else return read;
-}
-
-// Callback function for xmlReadIO
-static int
-closeTuFile (void * /*context*/)
-{
-        // nothing to do, the tu_file destructor will close
-        //tu_file* str = static_cast<tu_file*>(context);
-        //str->close();
-        return 0; // no error
-}
-
 #if 0 // not time for this (yet)
 static
 void _xmlErrorHandler(void* ctx, const char* fmt, ...)
@@ -847,8 +1029,9 @@ XML::ignoreWhite() const
     if ( VM::get().getSWFVersion() < 7 ) propname = "ignorewhite";
     else propname = "ignoreWhite";
 
+	string_table::key propnamekey = VM::get().getStringTable().find(propname);
     as_value val;
-    if ( ! const_cast<XML*>(this)->get_member(propname, &val) ) return false;
+    if (!const_cast<XML*>(this)->get_member(propnamekey, &val) ) return false;
     return val.to_bool();
 }
 
@@ -856,13 +1039,16 @@ XML::ignoreWhite() const
 int
 XML::getXMLOptions() const
 {
-    int options = XML_PARSE_RECOVER | XML_PARSE_NOWARNING | XML_PARSE_NOERROR;
-
+    int options = XML_PARSE_RECOVER | XML_PARSE_NOWARNING
+    		| XML_PARSE_NOERROR | XML_PARSE_NOCDATA;
+    // Using libxml2 to convert CDATA nodes to text seems to be what is
+    // required.
+    
     if ( ignoreWhite() )
     {
 	    // This doesn't seem to work, so the blanks skipping
 	    // is actually implemented in XML::extractNode instead.
-            //log_msg("Adding XML_PARSE_NOBLANKS to options");
+            //log_debug("Adding XML_PARSE_NOBLANKS to options");
             options |= XML_PARSE_NOBLANKS;
     }
 
