@@ -1,6 +1,6 @@
 // xmlsocket.cpp:  Network socket for XML-encoded information, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 //
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "utility.h"
@@ -31,6 +31,7 @@
 #include "VM.h"
 #include "builtin_function.h" // for setting timer, should likely avoid that..
 #include "URLAccessManager.h"
+#include "Object.h" // for getObjectInterface
 
 #include "log.h"
 
@@ -105,7 +106,7 @@ public:
 		/// The method will take care of polling from the
 		/// socket and invoking any onData handler.
 		///
-		void checkForIncomingData(as_environment& env);
+		void checkForIncomingData();
 
         XMLSocket obj;
 
@@ -137,7 +138,7 @@ XMLSocket::connect(const char *host, short port)
 {
     GNASH_REPORT_FUNCTION;
 
-    if ( ! URLAccessManager::allowHost(host, port) )
+    if ( ! URLAccessManager::allowXMLSocket(host, port) )
     {
 	    return false;
     }
@@ -177,14 +178,14 @@ XMLSocket::anydata(MessageList& msgs)
 bool XMLSocket::processingData()
 {
     //GNASH_REPORT_FUNCTION;
-    //log_msg(_("%s: processing flag is %d"), __FUNCTION__, _processing);
+    //log_debug(_("%s: processing flag is %d"), __FUNCTION__, _processing);
     return _processing;
 }
 
 void XMLSocket::processing(bool x)
 {
     GNASH_REPORT_FUNCTION;
-    //log_msg(_("%s: set processing flag to %d"), __FUNCTION__, x);
+    //log_debug(_("%s: set processing flag to %d"), __FUNCTION__, x);
     _processing = x;
 }
 
@@ -221,7 +222,7 @@ XMLSocket::anydata(int fd, MessageList& msgs)
         
         // If interupted by a system call, try again
         if (ret == -1 && errno == EINTR) {
-            log_msg(_("The socket for fd #%d was interupted by a system call"),
+            log_debug(_("The socket for fd #%d was interupted by a system call"),
                     fd);
             continue;
         }
@@ -231,26 +232,26 @@ XMLSocket::anydata(int fd, MessageList& msgs)
             return false;
         }
         if (ret == 0) {
-            //log_msg(_("%s: There is no data in the socket for fd #%d"),
+            //log_debug(_("%s: There is no data in the socket for fd #%d"),
             //   __FUNCTION__, fd);
             return false;
         }
         if (ret > 0) {
-            //log_msg(_("%s: There is data in the socket for fd #%d"),
+            //log_debug(_("%s: There is data in the socket for fd #%d"),
             //    __FUNCTION__, fd);
         }
         memset(buf, 0, INBUF);
         ret = ::read(_sockfd, buf, INBUF-2);
         cr = strlen(buf);
-        log_msg(_("%s: read %d bytes, first msg terminates at %d"), __FUNCTION__, ret, cr);
-        //log_msg(_("%s: read (%d,%d) %s"), __FUNCTION__, buf[0], buf[1], buf);
+        log_debug(_("%s: read %d bytes, first msg terminates at %d"), __FUNCTION__, ret, cr);
+        //log_debug(_("%s: read (%d,%d) %s"), __FUNCTION__, buf[0], buf[1], buf);
         ptr = buf;
         // If we get a single XML message, do less work
         if (ret == cr + 1)
 		{
             adjusted_size = memadjust(ret + 1);
             packet = new char[adjusted_size];
-            log_msg(_("Packet size is %d at %p"), ret + 1, packet);
+            log_debug(_("Packet size is %d at %p"), ret + 1, packet);
             memset(packet, 0, adjusted_size);
             strcpy(packet, ptr);
             eom = strrchr(packet, '\n'); // drop the CR off the end if there is one
@@ -258,7 +259,7 @@ XMLSocket::anydata(int fd, MessageList& msgs)
                 *eom = 0;
             }
             msgs.push_back( packet );
-            log_msg(_("%d: Pushing Packet of size " SIZET_FMT " at %p"), __LINE__, strlen(packet), packet);
+            log_debug(_("%d: Pushing Packet of size " SIZET_FMT " at %p"), __LINE__, strlen(packet), packet);
             processing(false);
             return true;
         }
@@ -268,8 +269,8 @@ XMLSocket::anydata(int fd, MessageList& msgs)
         while (strchr(ptr, '\n') > 0) {
             if (leftover) {
                 processing(false);
-                //log_msg(_("%s: The remainder is: \"%s\""), __FUNCTION__, leftover.get());
-                //log_msg(_("%s: The rest of the message is: \"%s\""), __FUNCTION__, ptr);
+                //log_debug(_("%s: The remainder is: \"%s\""), __FUNCTION__, leftover.get());
+                //log_debug(_("%s: The rest of the message is: \"%s\""), __FUNCTION__, ptr);
                 adjusted_size = memadjust(cr + strlen(leftover.get()) + 1);
                 packet = new char[adjusted_size];
                 memset(packet, 0, adjusted_size);
@@ -279,7 +280,7 @@ XMLSocket::anydata(int fd, MessageList& msgs)
                 if (eom) {
                     *eom = 0;
                 }
-                //log_msg(_("%s: The whole message is: \"%s\""), __FUNCTION__, packet);
+                //log_debug(_("%s: The whole message is: \"%s\""), __FUNCTION__, packet);
                 ptr = strchr(ptr, '\n') + 2; // messages are delimited by a "\n\0"
                 leftover.reset();
             } else {
@@ -290,20 +291,20 @@ XMLSocket::anydata(int fd, MessageList& msgs)
                 ptr += cr + 1;
             } // end of if remainder
             if (*packet == '<') {
-                //log_msg(_("%d: Pushing Packet #%d of size %d at %p: %s"), __LINE__,
+                //log_debug(_("%d: Pushing Packet #%d of size %d at %p: %s"), __LINE__,
                 //       data.size(), strlen(packet), packet, packet);
                 eom = strrchr(packet, '\n'); // drop the CR off the end there is one
                 if (eom) {
                     *eom = 0;
                 }
-                //log_msg(_("Allocating new packet at %p"), packet);
+                //log_debug(_("Allocating new packet at %p"), packet);
                 //data.push_back(packet);
                 msgs.push_back( std::string(packet) );
             } else {
                 log_error(_("Throwing out partial packet %s"), packet);
             }
             
-            //log_msg(_("%d messages in array now"), data.size());
+            //log_debug(_("%d messages in array now"), data.size());
             cr = strlen(ptr);
         } // end of while (cr)
         
@@ -311,11 +312,11 @@ XMLSocket::anydata(int fd, MessageList& msgs)
             leftover.reset( new char[strlen(ptr) + 1] );
             strcpy(leftover.get(), ptr);
             processing(true);
-            //log_msg(_("%s: Adding remainder: \"%s\""), __FUNCTION__, leftover.get());
+            //log_debug(_("%s: Adding remainder: \"%s\""), __FUNCTION__, leftover.get());
         }
         
         processing(false);
-        log_msg(_("%s: Returning %d messages"), __FUNCTION__, index);
+        log_debug(_("%s: Returning %d messages"), __FUNCTION__, index);
         return true;
         
     } // end of while (retires)
@@ -331,13 +332,13 @@ XMLSocket::send(std::string str)
     if ( ! connected() )
     {
         log_error(_("%s: socket not initialized"), __FUNCTION__);
-	assert(!_sockfd);
+	assert(_sockfd <= 0);
 	return false;
     }
     
     int ret = write(_sockfd, str.c_str(), str.size());
     
-    log_msg(_("%s: sent %d bytes, data was %s"), __FUNCTION__, ret, str.c_str());
+    log_debug(_("%s: sent %d bytes, data was %s"), __FUNCTION__, ret, str.c_str());
     if (ret == static_cast<signed int>(str.size())) {
         return true;
     } else {
@@ -397,7 +398,7 @@ XMLSocket::checkSockets(int fd)
     
     // If interupted by a system call, try again
     if (ret == -1 && errno == EINTR) {
-        log_msg(_("%s: The socket for fd #%d was interupted by a system call in this thread"),
+        log_debug(_("%s: The socket for fd #%d was interupted by a system call in this thread"),
                 __FUNCTION__, fd);
     }
     if (ret == -1) {
@@ -405,11 +406,11 @@ XMLSocket::checkSockets(int fd)
             __FUNCTION__, fd);
     }
     if (ret == 0) {
-        log_msg(_("%s: There is no data in the socket for fd #%d"),
+        log_debug(_("%s: There is no data in the socket for fd #%d"),
             __FUNCTION__, fd);
     }
     if (ret > 0) {
-        log_msg(_("%s: There is data in the socket for fd #%d"),
+        log_debug(_("%s: There is data in the socket for fd #%d"),
             __FUNCTION__, fd);
     }
     
@@ -427,7 +428,7 @@ xmlsocket_connect(const fn_call& fn)
 #ifdef GNASH_XMLSOCKET_DEBUG
     std::stringstream ss;
     fn.dump_args(ss);
-    log_msg(_("XMLSocket.connect(%s) called"), ss.str().c_str());
+    log_debug(_("XMLSocket.connect(%s) called"), ss.str().c_str());
 #endif
 
     boost::intrusive_ptr<xmlsocket_as_object> ptr = ensureType<xmlsocket_as_object>(fn.this_ptr);
@@ -438,10 +439,13 @@ xmlsocket_connect(const fn_call& fn)
     }
     
     as_value hostval = fn.arg(0);
-    const std::string& host = hostval.to_string(&fn.env());
-    int port = int(fn.arg(1).to_number(&fn.env()));
+    const std::string& host = hostval.to_string();
+    int port = int(fn.arg(1).to_number());
     
     bool success = ptr->obj.connect(host.c_str(), port);
+
+    VM& vm = VM::get();
+    string_table& st = vm.getStringTable();
     
     // Actually, if first-stage connection was successful, we
     // should NOT invoke onConnect(true) here, but postpone
@@ -450,26 +454,20 @@ xmlsocket_connect(const fn_call& fn)
     // confirm this is that onConnect is invoked *after* 
     // XMLSocket.connect() returned in these cases.
     //
-	boost::intrusive_ptr<as_function> handler = ptr->getEventHandler("onConnect");
-    if ( handler )
-    {
-        log_msg(_("XMLSocket.connect(): calling onConnect"));
-        as_environment env;
-        env.push(success);
-        val = call_method(handler.get(), &env, ptr.get(), 1, env.stack_size()-1); 
-    }
+    log_debug(_("XMLSocket.connect(): tring to call onConnect"));
+    ptr->callMethod(st.find(PROPNAME("onConnect")), success);
 	    
     if ( success )
     {
-        log_msg(_("Setting up timer for calling XMLSocket.onData()"));
+        log_debug(_("Setting up timer for calling XMLSocket.onData()"));
 
 	std::auto_ptr<Timer> timer(new Timer);
         boost::intrusive_ptr<builtin_function> ondata_handler = new builtin_function(&xmlsocket_inputChecker, NULL);
         unsigned interval = 50; // just make sure it's expired at every frame iteration (20 FPS used here)
         timer->setInterval(*ondata_handler, interval, boost::dynamic_pointer_cast<as_object>(ptr));
-        VM::get().getRoot().add_interval_timer(timer, true);
+        vm.getRoot().add_interval_timer(timer, true);
 
-        log_msg(_("Timer set"));
+        log_debug(_("Timer set"));
     }
 
     return as_value(success);
@@ -482,8 +480,8 @@ xmlsocket_send(const fn_call& fn)
     GNASH_REPORT_FUNCTION;
     
     boost::intrusive_ptr<xmlsocket_as_object> ptr = ensureType<xmlsocket_as_object>(fn.this_ptr);
-    const std::string& object = fn.arg(0).to_string(&fn.env());
-    //  log_msg(_("%s: host=%s, port=%g"), __FUNCTION__, host, port);
+    const std::string& object = fn.arg(0).to_string();
+    //  log_debug(_("%s: host=%s, port=%g"), __FUNCTION__, host, port);
     return as_value(ptr->obj.send(object));
 }
 
@@ -503,14 +501,14 @@ as_value
 xmlsocket_new(const fn_call& fn)
 {
     //GNASH_REPORT_FUNCTION;
-    //log_msg(_("%s: nargs=%d"), __FUNCTION__, nargs);
+    //log_debug(_("%s: nargs=%d"), __FUNCTION__, nargs);
     
     boost::intrusive_ptr<as_object> xmlsock_obj = new xmlsocket_as_object;
 
 #ifdef GNASH_XMLSOCKET_DEBUG
     std::stringstream ss;
     fn.dump_args(ss);
-    log_msg(_("new XMLSocket(%s) called - created object at %p"), ss.str().c_str(), (void*)xmlsock_obj.get());
+    log_debug(_("new XMLSocket(%s) called - created object at %p"), ss.str().c_str(), (void*)xmlsock_obj.get());
 #else
     UNUSED(fn);
 #endif
@@ -534,7 +532,7 @@ xmlsocket_inputChecker(const fn_call& fn)
         return as_value();
     }
 
-    ptr->checkForIncomingData(fn.env());
+    ptr->checkForIncomingData();
 
     return as_value();
 }
@@ -549,15 +547,6 @@ xmlsocket_onData(const fn_call& fn)
     
     boost::intrusive_ptr<xmlsocket_as_object> ptr = ensureType<xmlsocket_as_object>(fn.this_ptr);
 
-    boost::intrusive_ptr<as_function> onXMLEvent = ptr->getEventHandler("onXML");
-    if ( ! onXMLEvent )
-    {
-            log_msg(_("Builtin XMLSocket.onData doing nothing as no "
-                            "onXML event is defined on XMLSocket %p"),
-                            (void*)ptr.get());
-            return as_value();
-    }
-
     if ( fn.nargs < 1 )
     {
         IF_VERBOSE_ASCODING_ERRORS(
@@ -566,8 +555,7 @@ xmlsocket_onData(const fn_call& fn)
         return as_value();
     }
 
-    as_environment& env = fn.env();
-    const std::string& xmlin = fn.arg(0).to_string(&env);
+    const std::string& xmlin = fn.arg(0).to_string();
 
     if ( xmlin.empty() )
     {
@@ -578,9 +566,13 @@ xmlsocket_onData(const fn_call& fn)
     }
 
     boost::intrusive_ptr<as_object> xml = new XML(xmlin);
+    as_value arg(xml.get());
+
+    VM& vm = VM::get();
+    string_table& st = vm.getStringTable();
+    
+    ptr->callMethod(st.find(PROPNAME("onXML")), arg);
 
-    env.push(xml);
-    call_method(as_value(onXMLEvent.get()), &env, ptr.get(), 1, env.stack_size()-1);
     return as_value();
 
 
@@ -592,7 +584,7 @@ getXMLSocketInterface()
     static boost::intrusive_ptr<as_object> o;
     if ( o == NULL )
     {
-        o = new as_object();
+        o = new as_object(getObjectInterface());
         attachXMLSocketInterface(*o);
     }
     return o.get();
@@ -634,25 +626,22 @@ void xmlsocket_class_init(as_object& global)
 boost::intrusive_ptr<as_function>
 xmlsocket_as_object::getEventHandler(const std::string& name)
 {
-		boost::intrusive_ptr<as_function> ret;
+	boost::intrusive_ptr<as_function> ret;
 
-		std::string key=name;
-		VM& vm = VM::get();
-		if ( vm.getSWFVersion() < 7 ) boost::to_lower(key, vm.getLocale());
-
-		as_value tmp;
-		if ( ! get_member(key, &tmp) ) return ret;
-		ret = tmp.to_as_function();
-		return ret;
+	as_value tmp;
+	string_table& st = getVM().getStringTable();
+	if (!get_member(st.find(PROPNAME(name)), &tmp) ) return ret;
+	ret = tmp.to_as_function();
+	return ret;
 }
 
 void
-xmlsocket_as_object::checkForIncomingData(as_environment& env)
+xmlsocket_as_object::checkForIncomingData()
 {
     assert(obj.connected());
 
     if (obj.processingData()) {
-        log_msg(_("Still processing data"));
+        log_debug(_("Still processing data"));
     }
     
 #ifndef USE_DMALLOC
@@ -662,16 +651,16 @@ xmlsocket_as_object::checkForIncomingData(as_environment& env)
     std::vector<std::string > msgs;
     if (obj.anydata(msgs))
     {
-        log_msg(_("Got " SIZET_FMT " messages: "), msgs.size());
+        log_debug(_("Got " SIZET_FMT " messages: "), msgs.size());
         for (size_t i=0; i<msgs.size(); ++i)
         {
-            log_msg(_(" Message " SIZET_FMT " : %s "), i, msgs[i].c_str());
+            log_debug(_(" Message " SIZET_FMT " : %s "), i, msgs[i].c_str());
         }
 
         boost::intrusive_ptr<as_function> onDataHandler = getEventHandler("onData");
         if ( onDataHandler )
         {
-            //log_msg(_("Got %d messages from XMLsocket"), msgs.size());
+            //log_debug(_("Got %d messages from XMLsocket"), msgs.size());
             for (XMLSocket::MessageList::iterator it=msgs.begin(),
 							itEnd=msgs.end();
 			    it != itEnd; ++it)
@@ -682,8 +671,11 @@ xmlsocket_as_object::checkForIncomingData(as_environment& env)
 #ifndef USE_DMALLOC
                 //dump_memory_stats(__FUNCTION__, __LINE__, "start");
 #endif
-                env.push(datain);
-                call_method(as_value(onDataHandler.get()), &env, this, 1, env.stack_size()-1);
+				as_environment env;
+				env.push(datain);
+				fn_call call(this, &env, 1, env.stack_size() - 1);
+				onDataHandler->call(call);
+//                call_method(as_value(onDataHandler.get()), &env, this, 1, env.stack_size()-1);
 
 #ifndef USE_DMALLOC
                 //dump_memory_stats(__FUNCTION__, __LINE__, "end");
