@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,22 +15,17 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-//
-
-/* $Id: character.h,v 1.86.2.1 2007/08/18 16:13:36 strk Exp $ */
+/* $Id: character.h,v 1.128 2008/02/16 23:12:58 rsavoye Exp $ */
 
 #ifndef GNASH_CHARACTER_H
 #define GNASH_CHARACTER_H
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "gnash.h"
 #include "types.h"
-#include "container.h" // still needed ?
 #include "utility.h"
 #include "event_id.h" // for inlines
 #include "as_object.h" // for inheritance
@@ -54,62 +49,11 @@ namespace gnash {
 	class sprite_instance;
 	class movie_instance;
 	class ExecutableCode;
+	class action_buffer;
 }
 
 namespace gnash {
 
-/// Informations about timeline instances creation
-//
-/// See: http://www.gnashdev.org/wiki/index.php/TimelineControl#Timeline_instances
-///
-class TimelineInfo
-{
-
-public:
-
-	/// Construct a TimelineInfo
-	//
-	/// @param depth
-	///	Depth at which the instance was placed 
-	///
-	/// @param frame
-	///	Frame number in which the instance was placed (0-based)
-	///
-	/// @param replace
-	///	True if this object was placed by a REPLACE tag
-	///	(see PlaceObject2).
-	///
-	TimelineInfo(int depth, int frame, bool replace)
-		:
-		_depth(depth),
-		_frame(frame),
-		_replace(replace)
-	{
-	}
-
-
-	/// Return depth of initial placement 
-	int placedAtDepth() const { return _depth; }
-
-	/// Return frame number of initial placement (0-based)
-	size_t placedInFrame() const { return _frame; }
-
-	/// Return true if this instance replaced an other one at same depth
-	bool placedByReplaceTag() const { return _replace; }
-
-
-private:
-
-	/// Original depth
-	int _depth;
-
-	/// Frame of placement, 0-based
-	size_t _frame;
-
-	/// Placed by Replace tag ?
-	bool _replace;
-};
-
 /// Character is a live, stateful instance of a character_def.
 //
 /// It represents a single active element in a movie.
@@ -134,8 +78,6 @@ private:
 	int 	m_ratio;
 	int	m_clip_depth;
 	Events  _event_handlers;
-	void	(*m_display_callback)(void*);
-	void*	m_display_callback_user_ptr;
 
 	/// Used to assign a name to unnamed instances
 	static unsigned int _lastUnnamedInstanceNum;
@@ -143,17 +85,37 @@ private:
 	/// Set to yes when this instance has been unloaded
 	bool _unloaded;
 
+	/// This flag should be set to true by a call to destroy()
+	bool _destroyed;
+
 	/// Build the _target member recursive on parent
 	std::string computeTargetPath() const;
 
-	/// Timeline info, for timeline instances
-	//
-	/// For dynamically-created instances this is always NULL
-	///
-	std::auto_ptr<TimelineInfo> _timelineInfo;
+	/// The character masking this instance (if any)
+	character* _mask;
+
+	/// The character masked by this instance (if any)
+	character* _maskee;
+
+	/// Original target, as at construction time
+	std::string _origTarget;
+
+	/// Register a character masked by this instance
+	void setMaskee(character* maskee);
 
 protected:
 
+	/// Register currently computable target as
+	/// the "original" one. This will be used by
+	/// soft references (as_value) and should be
+	/// called as soon as the stagePlacementCallback
+	/// is invoked.
+	///
+	void saveOriginalTarget()
+	{
+		_origTarget=getTarget();
+	}
+
 #ifdef GNASH_USE_GC
 	/// Mark all reachable resources, override from as_object.
 	//
@@ -170,20 +132,17 @@ protected:
 
 	/// Mark character-specific reachable resources
 	//
-	/// These are: the character's parent and the defualt
-	/// as_object reachable stuff.
-	void markCharacterReachable() const
-	{
-		if ( m_parent ) m_parent->setReachable();
-		markAsObjectReachable();
-	}
+	/// These are: the character's parent, mask, maskee and the default
+	///             as_object reachable stuff.
+	///
+	void markCharacterReachable() const;
 #endif // GNASH_USE_GC
 
 	const Events& get_event_handlers() const
 	{
 	    return _event_handlers;
 	}
-#ifndef NEW_KEY_LISTENER_LIST_DESIGN
+
 	/// Return a user defined event handler, if any
 	//
 	/// @param name
@@ -196,7 +155,18 @@ protected:
 	///	casts to an as_function. A NULL pointer otherwise.
 	///
 	boost::intrusive_ptr<as_function> getUserDefinedEventHandler(const std::string& name) const;
-#endif	
+
+	/// Return a user defined event handler, if any
+	//
+	/// @param key
+	/// 	Function key to fetch. 
+	///
+	/// @return
+	///	A function if a member with the given key exists and
+	///	casts to an as_function. A NULL pointer otherwise.
+	///
+	boost::intrusive_ptr<as_function> getUserDefinedEventHandler(string_table::key key) const;
+
 	void set_event_handlers(const Events& copyfrom);
 
 	/// Used to assign a name to unnamed instances
@@ -209,11 +179,11 @@ protected:
 
 	boost::intrusive_ptr<character> m_parent;
 
-	/// Implement mouse-dragging for this movie.
-	void do_mouse_drag();
-
 	/// look for '.', 'this',  '..', '_parent', '_level0' and '_root'
-	character* get_relative_target_common(const std::string& name);
+	//
+	/// NOTE: case insensitive up to SWF6, sensitive from SWF7 up
+	///
+	as_object* get_path_element_character(string_table::key key);
 
 	/// \brief
 	/// Set when the visual aspect of this particular character or movie
@@ -224,13 +194,11 @@ protected:
 	/// so it starts in invalidated mode.
 	///
 	bool m_invalidated;
-	
-	
-	/// Just like m_invalidated but set when a child is invalidated instead
-  /// of this character instance. m_invalidated and m_child_invalidated
-  /// can be set at the same time. 
-	bool m_child_invalidated;
 
+	/// Just like m_invalidated but set when a child is invalidated instead
+	/// of this character instance. m_invalidated and m_child_invalidated
+	/// can be set at the same time. 
+	bool m_child_invalidated;
 
 	/// \brief
 	/// Bounds of this character instance before first invalidation
@@ -248,7 +216,7 @@ protected:
 	/// get_invalidated_bounds().
 	///
 	InvalidatedRanges m_old_invalidated_ranges;
-  	
+
 	/// Wheter this character has been transformed by ActionScript code
 	//
 	/// Once we've been moved by ActionScript,
@@ -325,11 +293,14 @@ public:  // TODO: make protected
 	/// Getter-setter for _target 
 	static as_value target_getset(const fn_call& fn);
 
+	/// Getter-setter for _name
+	static as_value name_getset(const fn_call& fn);
+
 	/// @} Common ActionScript getter-setters for characters
 
 public:
 
-    /// This is the amount substracted from displaylist tag defined depths.
+    /// This is the amount added to displaylist tag defined depths.
     /// Character placed by tags (vs. characters instantiated by ActionScript)
     /// always have negative depths by effect of this offset.
     //
@@ -338,12 +309,56 @@ public:
     /// See: http://www.senocular.com/flash/tutorials/depths/?page=2
     ///
     static const int staticDepthOffset = -16384;
+
+    /// This is the offset at which character's depth is
+    /// shifted when a character is removed from stage but
+    /// an onUnload event handler is defined.
+    ///
+    /// Example: a character at depth 60 gets moved to
+    ///          depth -32829 (-32769-60) when unloaded and
+    ///          an onUnload event handler is defined for it
+    ///	         or any of its childs.
+    ///
+    /// So, to recap:
+    ///		1:  -32769 to -16385 are removed
+    ///		2:  -16384 to      0 are statics
+    ///		3:  Max depth for a PlaceoObject call is 16384 (which becomes 0 in the statics)
+    ///	(all of the above correct?)
+    ///
+    ///
+    static const int removedDepthOffset = -32769; 
+
+    /// Return true if the given depth is in the removed zone
+    static bool depthInRemovedZone(int depth)
+    {
+        return depth < staticDepthOffset;
+    }
+
+    /// Return true if this character's depth is in the removed zone
+    bool depthInRemovedZone()
+    {
+        return depthInRemovedZone(get_depth());
+    }
     
-    /// This value is used for m_clip_depth when the value has no meaning, ie.
-    /// the character is not a mask. Depths below -16384 are illegal, so this
+    /// This value is used for m_clip_depth when 
+    /// the character is not a layer mask.
+    //
+    /// Depths below -16384 are illegal, so this
     /// value should not collide with real depths.  
+    ///
     static const int noClipDepthValue = -1000000;
 
+    /// This value is used for m_clip_depth when 
+    /// the character is a dynamic mask.
+    //
+    /// Depths below -16384 are illegal, so this
+    /// value should not collide with real depths.  
+    ///
+    static const int dynClipDepthValue = -2000000;
+
+	// Maybe it's better to move all these constants to DisplayListTag
+	static const int noRatioValue = -1;
+
     ~character();
 
     character(character* parent, int id)
@@ -352,9 +367,11 @@ public:
 	m_depth(0),
 	m_ratio(0),
 	m_clip_depth(noClipDepthValue),
-	m_display_callback(NULL),
-	m_display_callback_user_ptr(NULL),
 	_unloaded(false),
+	_destroyed(false),
+	_mask(0),
+	_maskee(0),
+	_origTarget(),
 	m_visible(true),
 	m_parent(parent),
 	m_invalidated(true),
@@ -362,11 +379,11 @@ public:
 	m_old_invalidated_ranges(),
 	_scriptTransformed(false),
 	_dynamicallyCreated(false)
-	{
+    {
 	    assert((parent == NULL && m_id == -1)
 		   || (parent != NULL && m_id >= 0));
 	    assert(m_old_invalidated_ranges.isNull());
-	}
+    }
 
 	/// Return a reference to the variable scope of this character.
 	//
@@ -391,11 +408,19 @@ public:
 			return m_parent.get();
 	}
 
-    // for extern movie
-    void set_parent(character* parent) { m_parent = parent; }
+	/// for extern movie
+	void set_parent(character* parent)
+	{
+		assert(_origTarget.empty());
+		m_parent = parent;
+	}
+
     int	get_depth() const { return m_depth; }
+
     void	set_depth(int d) { m_depth = d; }
+
     const matrix&	get_matrix() const { return m_matrix; }
+
     void	set_matrix(const matrix& m)
 	{
 	    assert(m.is_valid());
@@ -421,6 +446,7 @@ public:
     void set_y_scale(float factor);
 
     const cxform&	get_cxform() const { return m_color_transform; }
+
     void	set_cxform(const cxform& cx) 
     {       
       if (!(cx == m_color_transform)) {
@@ -428,10 +454,15 @@ public:
       	m_color_transform = cx;
       }
     }
+
     void	concatenate_cxform(const cxform& cx) { m_color_transform.concatenate(cx); }
+
     void	concatenate_matrix(const matrix& m) { m_matrix.concatenate(m); }
+
     int		get_ratio() const { return m_ratio; }
-    void	set_ratio(int r) {
+
+    void	set_ratio(int r)
+    {
       if (r!=m_ratio) set_invalidated(__FILE__, __LINE__); 
       m_ratio = r;       
     }
@@ -440,19 +471,80 @@ public:
     /// tells us to use the character as a mask for all the objects contained 
     /// in the display list from m_depth to m_clipping_depth inclusive.
     /// 
-    /// The value returned by get_clip_depth() is only valid when isMask()
+    /// The value returned by get_clip_depth() is only valid when isMaskLayer()
     /// returns true!
     ///  
     int get_clip_depth() const { return m_clip_depth; }
-    void set_clip_depth(int d) { m_clip_depth = d; }
-    
-    /// Returns true when the character (and it's childs) are used as a mask
-    /// for other characters. isMask() does *not* return true when one of it's
-    /// parents is a mask and the character itself is not.
-    ///   
-    bool isMask() const { return m_clip_depth!=noClipDepthValue; }
 
-    virtual void set_name(const char* name) { _name = name; }
+	/// See get_clip_depth()
+	void set_clip_depth(int d)
+	{
+		m_clip_depth = d;
+	}
+    
+	/// Returns true when the character (and it's childs) is used as a mask
+	/// for other characters at higher depth (up to get_clip_depth).
+	/// isMaskLayer() does *not* return true when one of it's
+	/// parents is a mask and the character itself is not.
+	///
+	/// See also isDynamicMask() and isMask()
+	///   
+	bool isMaskLayer() const
+	{
+		// TODO: is dynClipDepthValue still needed ?
+		//       since we have a _maskee member now, we may use that instead..
+		return (m_clip_depth!=noClipDepthValue && m_clip_depth!=dynClipDepthValue);
+	}
+
+	/// Returns true when the character (and it's childs) is used as a mask
+	/// for another character.
+	/// isDynamicMask() does *not* return true when one of it's
+	/// parents is a mask and the character itself is not.
+	///
+	/// NOTE: there's no way to obtain the maskee from a dynamic mask
+	///
+	/// See also isMaskLayer() and isMask()
+	///   
+	bool isDynamicMask() const
+	{
+		return (m_clip_depth==dynClipDepthValue);
+	}
+
+	character* to_character() { return this; }
+
+	/// Return the character masking this instance (if any)
+	character* getMask() const
+	{
+		if ( ! _mask ) return NULL;
+		if ( _mask->_maskee != this )
+		{
+			// TODO: fix this !
+			log_error("Our mask maskee is not us");
+			return NULL; // for correctness;
+		}
+		return _mask;
+	}
+
+	/// Register a character as a mask for this instance.
+	///
+	/// @param mask The character to use as a mask, possibly NULL.
+	///	A reference to us will be registered with the mask, if
+	///	not null, so it'll know it's a mask for us, and would stop
+	///	being a mask for anything else.
+	///
+	void setMask(character* mask);
+
+	/// Returns true if this character is a mask (either layer or dynamic mask)
+	bool isMask() const
+	{
+		return isDynamicMask() || isMaskLayer();
+	}
+
+	/// Set character name, initializing the original target member
+	void set_name(const std::string& name)
+	{
+		_name = name;
+	}
 
     const std::string& get_name() const { return _name; }
 
@@ -540,8 +632,8 @@ public:
 	///
 	virtual void has_mouse_event() {}
 
-    // Movie interfaces.  By default do nothing.  sprite_instance and some others override these.
-    virtual void	display() {}
+	/// Render this character
+	virtual void	display() {}
 
     	/// Returns local, untransformed height of this character in TWIPS
 	//
@@ -611,7 +703,7 @@ public:
 	/// Point coordinates are in world TWIPS
 	///
 	/// The default implementation returns false if the character is
-	/// not visible, calling pointInBounds() otherwise.
+	/// not visible, calling pointInShape() otherwise.
 	///
 	/// Note that this is good for simple characters but needs
 	/// to be overridden for characters with childs. When a
@@ -620,30 +712,30 @@ public:
 	///
 	virtual bool pointInVisibleShape(float x, float y) const
 	{
-		if ( get_visible() ) return pointInShape(x, y);
-		else return false;
+		if ( ! get_visible() ) return false;
+		if ( isMask() ) return false;
+		return pointInShape(x, y);
 	}
 
-	/// Return the "relative" root of this character
+	/// Return the relative or absolute root of this character
 	//
 	/// The "relative" is the movie_instance created by
 	/// the same SWF definition that contained the
 	/// definition of this character.
 	///
+	/// The "absolute" is top of parent's tree.
+	///
 	/// TODO: what about programmatically created characters ?
 	///	  which would their "relative" root be ?
 	///
-	/// The default implementation is to invoke get_root_movie
+	/// The default implementation is to invoke get_root
 	/// against this character's parent.
 	///
-	virtual sprite_instance* get_root_movie();
-
-	/// By default call get_root on the parent
 	virtual movie_instance* get_root() const {
 		return get_parent()->get_root();
 	}
 
-	/// Find the character which is one degree removed from us,
+	/// Find the object which is one degree removed from us,
 	/// given the relative pathname.
 	///
 	/// If the pathname is "..", then return our parent.
@@ -661,14 +753,12 @@ public:
 	///
 	/// In ActionScript 1.0, everything seems to be CASE
 	/// INSENSITIVE.
-	virtual character* get_relative_target(const std::string& name)
+	///
+	virtual as_object* get_path_element(string_table::key key)
 	{
-		return get_relative_target_common(name);
+		return get_path_element_character(key);
 	}
 
-    //virtual size_t	get_current_frame() const { assert(0); return 0; }
-    //virtual bool	has_looped() const { assert(0); return false; }
-
 	/// Restart the character
 	//
 	/// This is only meaningful for sprite instances, but default
@@ -687,12 +777,13 @@ public:
 	/// 
 	/// Frame advancement include execution of all control tags.
 	/// 
-	virtual void advance(float /*delta_time*/)
+	virtual void advance()
 	{
 		// GNASH_REPORT_FUNCTION 
 	}
 
-    virtual void	goto_frame(size_t /*target_frame*/) {}
+	// TODO: verify if this is really needed (I guess not)
+	virtual void	goto_frame(size_t /*target_frame*/) {}
 
 	/// \brief
 	/// Return true if PlaceObjects tag are allowed to move
@@ -718,39 +809,18 @@ public:
 	///	  NOT set as dynamic (should check if they should)
 	///	- Characters created by attachMovie ARE dynamic
 	///	- Characters created by duplicateMovieClip ARE dynamic
+	/// - Characters created by createEmptyMovieClip ARE dynamic
 	///	- Characters created by new Video ARE dynamic
 	///	- Characters created by createTextField ARE dynamic
 	///
 	///
 	bool isDynamic() const {
-
-		// WARNING: cannot use _timelinInfo for this, unless 
-		// we'll provide a TimelineInfo object for top level movies
-		// (_level#) and dynamically loaded movies too...
-		// which would have no use except implementing
-		// isDynamic(). Note that we have NO automated test for this, but
-		// the "Magical Trevor 2" movie aborts due to a call to getBytesTotal
-		// against the root movie, and bug #19844 show the effect with dynamically
-		// loaded movies.
-
-		// Anyway, any dynamically created character must NOT have a
-		// _timelineInfo object (see setDynamic)
-		assert(_dynamicallyCreated ? (_timelineInfo.get() == 0) : 1 );
-
 		return _dynamicallyCreated;
 	}
 
 	/// Mark this character as dynamically created
-	//
-	/// "Dynamically created" means created trough ActionScript
-	///
-	/// TODO: deprecate this function, all characters should be
-	///	  dynamic by default Unless setTimelineInfo is called.
-	///
 	void setDynamic() {
-		assert(_timelineInfo.get() == NULL);
 		_dynamicallyCreated = true;
-		//assert(get_depth() > 0);
 	}
 
 	/// \brief
@@ -768,31 +838,15 @@ public:
 		_scriptTransformed = true;
 	}
 
-
-    // TODO: why is this virtual ??
-    virtual void	set_visible(bool visible) {
-      if (m_visible!=visible) set_invalidated(__FILE__, __LINE__);  
-      m_visible = visible;      
-    }
-
-    // TODO: why is this virtual ??
-    virtual bool	get_visible() const { return m_visible; }
-
-    virtual void	set_display_callback(void (*callback)(void*), void* user_ptr)
+	// Set whether this character should be rendered
+	void set_visible(bool visible)
 	{
-	    m_display_callback = callback;
-	    m_display_callback_user_ptr = user_ptr;
+		if (m_visible!=visible) set_invalidated(__FILE__, __LINE__);  
+		m_visible = visible;      
 	}
 
-    virtual void	do_display_callback()
-	{
-//			GNASH_REPORT_FUNCTION;
-			
-	    if (m_display_callback)
-		{
-		    (*m_display_callback)(m_display_callback_user_ptr);
-		}
-	}
+	// Return true if this character should be rendered
+	bool get_visible() const { return m_visible; }
 
 	/// Return mouse state in given variables
 	//
@@ -830,8 +884,22 @@ public:
 		return false;
 	}
 
-	/// Queue event handler(s), if any.
-	void queueEventHandler(const event_id& id);
+	/// Queue event in the global action queue.
+	//
+	/// on_event(id) will be called by execution of the queued
+	/// action
+	///
+	void queueEvent(const event_id& id, int lvl);
+
+	/// Return true if an handler for the given event is defined
+	//
+	/// NOTE that we look for both clip-defined and user-defined
+	/// handlers, which is likely error prone since we're doing
+	/// this in a non-virtual function. Main use for this method
+	/// is for being called by ::unload() to verify an Unload handler
+	/// is available.
+	///
+	bool hasEventHandler(const event_id& id) const;
 
 	virtual void on_button_event(const event_id& id)
 	{
@@ -866,16 +934,31 @@ public:
 	{
 		return NULL;
 	}
-	
+
+	/// Find highest depth character whose shape contains the given
+	/// point and is not the character being dragged or any of its childs.
+	//
+	/// Point coordinates in global twips.
+	///
+	virtual const character* findDropTarget(float x, float y, character* dragging) const
+	{
+		if ( this != dragging && get_visible()
+			&& pointInVisibleShape(x, y) )
+		{
+			return this;
+		}
+		else return 0;
+	}
+
 	/// Returns true when the object (type) should get a instance name even 
 	/// if none is provided manually.
-	virtual bool wantsInstanceName()
+	virtual bool wantsInstanceName() const
 	{
 		return false; 
 	}
 
 	/// Returns true when the object (type) can be referenced by ActionScipt
-	bool isActionScriptReferenceable()
+	bool isActionScriptReferenceable() const
 	{
 		// The way around
 		// [ wantsInstanceName() returning isActionScriptReferenceable() ]
@@ -883,6 +966,20 @@ public:
 		return wantsInstanceName();
 	}
 
+	/// Returns the closest as-referenceable ancestor
+	character* getClosestASReferenceableAncestor() 
+	{
+		if ( isActionScriptReferenceable() ) return this;
+		assert(m_parent);
+		return m_parent->getClosestASReferenceableAncestor();
+	}
+
+	const character* getClosestASReferenceableAncestor() const
+	{
+		character* nonconst_this = const_cast<character*>(this);
+		return nonconst_this->getClosestASReferenceableAncestor();
+	}
+
 	/// @}
 
 	/// \brief
@@ -950,35 +1047,67 @@ public:
 	/// force is set.
 	///
 	virtual void add_invalidated_bounds(InvalidatedRanges& ranges, bool force) = 0;
-	
-	/// Construct this instance as an ActionScript object.
+
+	/// Callback invoked whenever a character is placed on stage
 	//
 	/// This function must be called when the character is placed on
-	/// stage for the first time. It will take care of invoking
-	/// the constructor of its associated class and calling the
-	/// 'onConstruct' event handler.
+	/// stage for the first time.
 	///
-	/// Make sure this instance got an instance name before calling
-	/// this method (it's needed for properly setting the "this" pointer
-	/// when calling user-defined constructors).
+	/// The character version of this call sets the original target
+	/// of the character, for soft references to work.
+	/// If you override the method remember to call saveOriginalTarget()
+	/// as the first thing.
 	///
-	virtual void construct()
+	virtual void stagePlacementCallback()
 	{
-		on_event(event_id::CONSTRUCT);
+		saveOriginalTarget();
 	}
 
 	/// Unload this instance from the stage.
 	//
 	/// This function must be called when the character is removed
 	/// from the stage.
-	/// stage for the first time. It will take care of properly
-	/// unloading any child characters and queuing the 'UNLOAD'
-	/// event handler.
+	/// It will take care of properly calling
+	/// unload against any child characters and queuing the
+	/// 'UNLOAD' event handler.
 	///
-	virtual void unload();
+	/// @return true if any onUnload event handler was defined
+	///         by either this or any child characters, false
+	///         otherwise.
+	///
+	virtual bool unload();
 
+	/// Return true if this character was unloaded from the stage
 	bool isUnloaded() { return _unloaded; }
+
+	/// Mark this character as destroyed
+	//
+	/// A character should be destroyed when is removed from the display
+	/// list and is not more needed for names (target) resolutions.
+	/// Sprites are needed for names resolution whenever themselves
+	/// or a contained object has an onUnload event handler defined, 
+	/// in which case we want the event handler to find the 'this'
+	/// variable w/out attempting to rebind it.
+	///
+	/// Note: this function can safely release most memory associated
+	///       with the character as it will not be needed anymore.
+	///
+	virtual void destroy();
+
+	/// Return true if this character was destroyed.
+	//
+	/// See destroy() for more info.
+	///
+	bool isDestroyed() const { return _destroyed; }
 	
+	/// Returns true when the character bounds intersect with the current  
+  /// rendering clipping area.
+  ///
+  /// There is no need to do any rendering for this character when this 
+  /// function returns false because the renderer will not change any pixels
+  /// in the area where this character is placed.  
+	bool boundsInClippingArea() const; 
+
 public: // istn't this 'public' reduntant ?
 
 	/// Return full path to this object, in slash notation
@@ -987,56 +1116,33 @@ public: // istn't this 'public' reduntant ?
 	///
 	std::string getTargetPath() const;
 
+	/// Return original target path to this object, in dot notation
+	/// as of at construction time.
+	//
+	/// This is needed to properly dereference dangling soft-references
+	/// See testcase misc-swfc.all/soft_reference_test1.sc
+	///
+	const std::string& getOrigTarget() const
+	{
+		return _origTarget;
+	}
+
 	/// Return full path to this object, in dot notation
 	//
 	/// e.g. "_level0.sprite1.sprite2.ourSprite"
 	///
 	std::string getTarget() const;
 
-	/// Set timeline information for this instance
-	//
-	/// Timeline info should be set only once, right after creation
-	/// of a timeline instance. If this function is called twice
-	/// Gnash will abort.
-	/// Once timeline informations are added to a character, the
-	/// character become a "timeline instance". If not timeline info
-	/// is available, this is a dynamically-created character.
-	/// See isDynamic();
-	///
-	/// @param depth
-	///	Depth of first placement.
-	///
-	/// @param frame
-	///	Frame of first placement. 0-based.
-	///
-	/// @param replacing
-	///	True if this object was placed by a REPLACE tag
-	///	(see PlaceObject2).
-	///
-	/// NOTE: if we want to compute TimelineInfo records once at
-	/// 	  parse time and reuse pointers we'll just need to take
-	///	  a TimelineInfo pointer as parameter, externally owned.
-	///	  For now, we'll use a new object for each instance.
-	///
-	void setTimelineInfo(int depth, int frame, bool replacing)
-	{
-		assert(_timelineInfo.get()==NULL); // don't call twice !
-		_timelineInfo.reset(new TimelineInfo(depth, frame, replacing));
-	}
-
-	/// Return timeline information, if this is a timeline instance
-	//
-	/// For dynamic instances, NULL is returned.
-	/// Ownership of the returned object belong to this character.
-	///
-	TimelineInfo* getTimelineInfo() { return _timelineInfo.get(); }
-	
-  // override from as_object
-	virtual std::string get_text_value() const;	
-
 #ifdef NEW_KEY_LISTENER_LIST_DESIGN
 	boost::intrusive_ptr<as_function> getUserDefinedEventHandler(const std::string& name) const;
 #endif
+
+	/// Return true if this character is a selectable TextField
+	//
+	/// This method is used by Gui to set up an appropriate cursor
+	/// for input textfields.
+	///
+	virtual bool isSelectableTextField() const { return false; }
 };
 
 
