@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,11 +15,8 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "movie_instance.h"
@@ -39,18 +36,39 @@ namespace gnash {
 
 movie_instance::movie_instance(movie_definition* def, character* parent)
 	:
-	sprite_instance(def, this, parent, -1),
+	sprite_instance(def, this, parent, parent ? 0 : -1),
 	_def(def)
 {
 }
 
-// Advance of an SWF-defined movie instance
 void
-movie_instance::advance(float delta_time)
+movie_instance::stagePlacementCallback()
 {
+	saveOriginalTarget();
+
 	//GNASH_REPORT_FUNCTION;
 
-	assert ( get_root()->get_root_movie() == this );
+	//_def->stopLoader();
+
+	// Load first frame  (1-based index)
+	size_t nextframe = 1;
+	if ( !_def->ensure_frame_loaded(nextframe) )
+	{
+		IF_VERBOSE_MALFORMED_SWF(
+		log_swferror("Frame " SIZET_FMT " never loaded. Total frames: "
+				SIZET_FMT ".", nextframe, get_frame_count());
+		);
+	}
+
+	// Invoke parent placement event handler
+	sprite_instance::stagePlacementCallback();  
+}
+
+// Advance of an SWF-defined movie instance
+void
+movie_instance::advance()
+{
+	//GNASH_REPORT_FUNCTION;
 
 	//_def->stopLoader();
 
@@ -68,34 +86,7 @@ movie_instance::advance(float delta_time)
 		);
 	}
 
-	// The parser might have reset the total frame count
-	// due to SWF malformation, so we check this *after*
-	// the ensure_frame_loaded call above.
-	//
-	if ( get_frame_count() == 0 )
-	{
-		IF_VERBOSE_MALFORMED_SWF(
-		static bool warned=false;
-		if ( ! warned ) {
-			log_swferror(_("The movie with url %s has NO frames!"), m_def->get_url().c_str());
-			warned=true;
-		}
-		);
-		return;
-	}
-
-	if (m_on_event_load_called == false)
-	{
-		construct();
-	}
-
-	advance_sprite(delta_time);
-
-	if (m_on_event_load_called == false)
-	{
-		on_event(event_id::LOAD);	// root onload
-		m_on_event_load_called = true;
-	}
+	advance_sprite(); 
 
 	//_def->resumeLoader();
 }
