@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: movie_root.h,v 1.70 2007/08/02 22:07:26 strk Exp $ */
+/* $Id: movie_root.h,v 1.109 2008/02/20 14:46:28 strk Exp $ */
 
 /// \page events_handling Handling of user events
 ///
@@ -67,7 +67,7 @@
 #define GNASH_MOVIE_ROOT_H
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "tu_config.h"
@@ -75,9 +75,10 @@
 #include "mouse_button_state.h" // for composition
 #include "drag_state.h" // for composition
 #include "movie_instance.h" // for inlines
-#include "timers.h" // for composition
+//#include "timers.h" // for composition
 #include "asobj/Key.h"
 #include "smart_ptr.h" // for memory management
+#include "URL.h" // for loadMovie
 
 #include <vector>
 #include <list>
@@ -88,6 +89,7 @@ namespace gnash {
     class ExecutableCode; // for ActionQueue
     class Stage;
     class URL;
+    class Timer;
 }
 
 namespace gnash
@@ -103,76 +105,6 @@ struct DepthComparator
     }
 };
 
-#ifdef NEW_KEY_LISTENER_LIST_DESIGN
-class KeyListener{
-    public:
-        
-        KeyListener(boost::intrusive_ptr<as_object> obj, int flag=0)
-        : _listener(obj), _registered_type(flag)
-        {}
-
-        boost::intrusive_ptr<as_object> get() const { return _listener; }
-
-        bool operator == (const KeyListener & rhs ) const { return _listener == rhs.get(); }
-        bool operator != (const KeyListener & rhs ) const { return _listener != rhs.get(); }
-        bool operator < (const KeyListener & rhs ) const { return _listener < rhs.get(); }
-        
-        enum
-        {
-            ON_CLIP_DEF = 1 << 0,
-            USER_DEF    = 1 << 1
-        };
-
-        /// \brief
-        /// true if the _listener has OnClip defined key event handlers,
-        /// false if the _listener has no OnClip defined key event handlers.
-        ///
-        /// OnClip defined key event handlers are registered automatically and can not
-        /// be unregistered when they are defined.
-        bool hasOnClipRegistered() const { return _registered_type & ON_CLIP_DEF; }
-        
-        /// \brief
-        /// true if the _listener has been registered by Key.addListener(),
-        /// false if the _listener has not been registered by Key.addListener()
-        /// or unregistered by Key.removeListener().
-        bool hasUserRegistered() const { return _registered_type & USER_DEF; }      
-
-        /// register user defined key handler
-        void registerUserHandler() const { _registered_type |= USER_DEF; }
-        
-        /// register OnClip defined key handler
-        void registerOnClipHandler() const { _registered_type |= ON_CLIP_DEF; }
-
-        /// unregister user defined key handler
-        void unregisterUserHandler() const { _registered_type ^= USER_DEF; }
-
-#ifdef GNASH_USE_GC
-        /// Mark the wrapped object as reachable
-        void setReachable() const
-        {
-            if ( _listener ) _listener->setReachable();
-        }
-#endif
-
-    private:
-
-        /// the listener object, could be a character or a general as_object
-        boost::intrusive_ptr<as_object> _listener;
-
-        /// 0: the listener has no registered event handlers, to be removed;
-        /// ON_CLIP_DEF: the listener has registered onClip event handlers;
-        /// USER_DEF: the listener has registered user defined handlers;
-        //
-        // (1) onClip handlers get registered as soon as they are defined, and 
-        //     will never get unregistered;
-        // (2) user defined handlers get registered by Key.addListener(obj),
-        //     and unregistered by Key.removedListener(obj);
-        // mutable here is a hack for using std::set<>, we might drop this class 
-        // or change to another container later.
-        mutable int _registered_type;
-    };
-#endif 
-
 /// The movie stage (absolute top level node in the characters hierarchy)
 //
 /// This is a wrapper around the set of loaded levels being played.
@@ -243,6 +175,33 @@ public:
     ///
     bool loadLevel(unsigned int num, const URL& url);
 
+    /// Swap depth of a level (or two)
+    //
+    /// Character's depths are updated.
+    ///
+    /// @param sp
+    ///		The level to change depth/level of. A pointer to it is expected
+    ///		to be found in the _level# container, or an error will be printed
+    ///		and the call would result in a no-op.
+    ///
+    /// @param depth
+    ///		New depth to assign to the character. If another level exists at 
+    ///		the target depth the latter is moved in place of the former, with
+    ///		its depth also updated.
+    ///
+    void swapLevels(boost::intrusive_ptr<sprite_instance> sp, int depth);
+
+    /// Drop level at given depth.
+    //
+    /// @param depth
+    ///   Depth of the level to drop. Note that this is -character::staticDepthOffset for
+    ///   the root movie. Must be >=0 and <= 1048575 or an assertion will fail.
+    ///   Note that if the depth evaluates to the original root movie nothing happens
+    ///   (not allowed to remove that). It is not tested if it's allowed to remove 
+    ///   _level0 after loading into it.
+    ///
+    void dropLevel(int depth);
+
     /// @@ should this delegate to _level0?  probably !
     void set_member(
         const std::string& /*name*/,
@@ -322,7 +281,7 @@ public:
 
     /// The host app can use this to tell the movie where the
     /// user's mouse pointer is.
-    void notify_mouse_state(int x, int y, int buttons);
+    //void notify_mouse_state(int x, int y, int buttons);
 
     /// \brief
     /// The host app can use this to tell the movie when
@@ -334,7 +293,7 @@ public:
     ///
     bool notify_key_event(key::code k, bool down);
 
-    /// \brief
+	/// \brief
     /// Use this to retrieve the last state of the mouse, as set via
     /// notify_mouse_state(). 
     //
@@ -346,27 +305,24 @@ public:
 
     void set_drag_state(const drag_state& st);
 
-    /// @return current top-level root sprite (_level0)
-    sprite_instance* get_root_movie()
+    /// @return the originating root movie (not necessarely _level0)
+    movie_instance* getRootMovie() const
     {
-        if ( _movies.empty() ) return NULL;
-        return getLevel(0).get();
+	return _rootMovie.get();
     }
 
     void stop_drag()
     {
-        log_msg("stop_drag called");
         m_drag_state.reset();
     }
 
-    /// Return definition of _level0 
+    /// Return definition of originating root movie 
     //
-    /// TODO: drop this function ?
+    /// TODO: rename to getOriginatingDefinition ?
     ///
     movie_definition* get_movie_definition() const
     {
-        assert(!_movies.empty());
-        return getLevel(0)->get_movie_definition(); 
+	return getRootMovie()->get_movie_definition();
     }
 
     /// Add an interval timer
@@ -389,19 +345,14 @@ public:
     ///
     bool clear_interval_timer(unsigned int x);
 
-    /// Return 0-based frame index of _level0
+    /// Return 0-based frame index of originating root movie
     //
-    /// TODO: drop this function
+    /// TODO: drop this function (currently used by gprocessor)
+    ///       or change it to to delegate to _level0 ?
     ///
     size_t get_current_frame() const
     {
-        assert(!_movies.empty());
-        return getLevel(0)->get_current_frame();
-    }
-
-    // @@ should this be in movie_instance ?
-    float get_frame_rate() const {
-        return get_movie_definition()->get_frame_rate();
+	return getRootMovie()->get_current_frame();
     }
 
     /// \brief
@@ -413,47 +364,46 @@ public:
         return m_pixel_scale;
     }
 
-    // @@ Is this one necessary?
-    //
-    // TODO: drop this
-    character* get_character(int character_id)
-    {
-        assert(!_movies.empty());
-        return getLevel(0)->get_character(character_id);
-    }
+    void set_background_color(const rgba& color);
 
-    void set_background_color(const rgba& color)
-    {
-        m_background_color = color;
-    }
-
-    void set_background_alpha(float alpha)
-    {
-        m_background_color.m_a = iclamp(frnd(alpha * 255.0f), 0, 255);
-    }
+    void set_background_alpha(float alpha);
 
     float get_background_alpha() const
     {
         return m_background_color.m_a / 255.0f;
     }
 
-    /// Restart all levels
-    void restart();
+    /// Entry point for movie advancement
+    //
+    /// This function does:
+    ///     - Execute all timers
+    ///     - Reset the next Random number
+    ///     - Advance all advanceable characters in reverse-placement order
+    ///     - Cleanup key listeners
+    ///     - Process all queued actions
+    ///     - Remove unloaded characters from the advanceable characters list.
+    ///     - Run the GC collector
+    ///
+    void advance();
 
-    void advance(float delta_time);
-
-    /// 0-based!! delegates to _level0
+    /// 0-based!! delegates to originating root movie
+    //
+    /// TODO: drop this method. currently used by gprocessor.
+    ///
     void goto_frame(size_t target_frame_number)
     {
-        getLevel(0)->goto_frame(target_frame_number);
+	getRootMovie()->goto_frame(target_frame_number);
     }
 
     void display();
 
-    /// Delegate to _level0
+    /// Delegate to originating root movie
+    //
+    /// TODO: drop ?
+    ///
     void set_play_state(sprite_instance::play_state s)
     {
-        getLevel(0)->set_play_state(s);
+	getRootMovie()->set_play_state(s);
     }
 
     /// For ActionScript interfacing convenience.
@@ -477,57 +427,109 @@ public:
     void * get_userdata() { return m_userdata; }
     void set_userdata(void * ud ) { m_userdata = ud;  }
 
-    DSOEXPORT void notify_key_listeners(key::code k, bool down);
-#ifdef NEW_KEY_LISTENER_LIST_DESIGN
-    // Push a new key listener to the container if it is not there,
-    // otherwise, just register it.
-    void add_key_listener(const KeyListener& listener);
-    
-    // remove the specified listener from the container if found
-    void remove_key_listener(as_object* listener);
+	/// Notify still loaded character listeners for key events
+	DSOEXPORT void notify_key_listeners(key::code k, bool down);
 
-    typedef std::set<KeyListener> KeyListeners;
-    KeyListeners & getKeyListeners() { return _keyListeners; }
-#else
-    void add_key_listener(as_object* listener);
-    void remove_key_listener(as_object* listener);
-#endif
+	/// Push a new character listener for key events
+	void add_key_listener(character* listener)
+    {
+        add_listener(m_key_listeners, listener);
+    }
 
-    DSOEXPORT void notify_mouse_listeners(const event_id& event);
-    void add_mouse_listener(as_object* listener);
-    void remove_mouse_listener(as_object* listener);
+	/// Remove a character listener for key events
+	void remove_key_listener(character* listener)
+    {
+        remove_listener(m_key_listeners, listener);
+    }
+
+	/// Notify still loaded character listeners for mouse events
+	DSOEXPORT void notify_mouse_listeners(const event_id& event);
+
+	/// Push a new character listener for mouse events
+	void add_mouse_listener(character* listener)
+    {
+        add_listener(m_mouse_listeners, listener);
+    }
+
+	/// Remove a character listener for mouse events
+	void remove_mouse_listener(character* listener)
+    {
+        remove_listener(m_mouse_listeners, listener);
+    }
 
     /// Get the character having focus
     //
+    /// The character having focus will receive mouse button
+    /// and key presses/releases.
+    ///
     /// @return the character having focus or NULL of none.
     ///
-    character* get_active_entity();
+    character* getFocus();
 
     /// Set the character having focus
     //
     /// @param ch
     /// The character having focus. NULL to kill focus.
     ///
-    void set_active_entity(character* ch);
+    void setFocus(character* ch);
     
     DSOEXPORT void add_invalidated_bounds(InvalidatedRanges& ranges, bool force);
 
+    /// Return the topmost active entity under the pointer
+    //
+    /// This method returns cached info, with cache updated
+    /// by notify_mouse_moved (and should be updated also
+    /// by movie advancement or actions execution maybe, not
+    /// currently implmented).
+    ///
+    /// @return the topmost active entity under pointer or NULL if none.
+    ///
+    character* getActiveEntityUnderPointer() const;
+
+    /// Return the topmost non-dragging entity under the pointer
+    //
+    /// This method triggers a displaylist scan
+    ///
+    /// @return the topmost non-dragging entity under pointer or NULL if none
+    ///
+    const character* getEntityUnderPointer() const;
+
+    /// Return the character currently being dragged, if any
+    character* getDraggingCharacter() const;
+
     /// Return true if the mouse pointer is over an active entity
     bool isMouseOverActiveEntity() const;
 
     bool testInvariant() const;
 
-    // Clear invalidated flag for all levels 
-    void clear_invalidated();
+    /// Action priority levels
+    enum ActionPriorityLevel {
+
+        /// Init actions, Init event handlers
+        apINIT=0,
+
+        /// Construct event handlers
+        apCONSTRUCT=1,
+
+        /// EnterFrame event handlers
+        apENTERFRAME=2,
+
+        /// Frame actions, load handlers, unload handlers
+        apDOACTION=3,
+
+        /// Last element used to easy computation of size...
+        apSIZE
+        
+    };
 
     /// Push an executable code to the ActionQueue
-    void pushAction(std::auto_ptr<ExecutableCode> code);
+    void pushAction(std::auto_ptr<ExecutableCode> code, int lvl=apDOACTION);
 
     /// Push an executable code to the ActionQueue
-    void pushAction(const action_buffer& buf, boost::intrusive_ptr<character> target);
+    void pushAction(const action_buffer& buf, boost::intrusive_ptr<character> target, int lvl=apDOACTION);
 
     /// Push a function code to the ActionQueue
-    void pushAction(boost::intrusive_ptr<as_function> func, boost::intrusive_ptr<character> target);
+    void pushAction(boost::intrusive_ptr<as_function> func, boost::intrusive_ptr<character> target, int lvl=apDOACTION);
 
 #ifdef GNASH_USE_GC
     /// Mark all reachable resources (for GC)
@@ -535,35 +537,228 @@ public:
     /// Resources reachable from movie_root are:
     ///
     /// - All _level# movies (_movies)
+    /// - The original root movie (_rootMovie)
     /// - Mouse entities (m_mouse_button_state)
     /// - Timer targets (_intervalTimers)
     /// - Resources reachable by ActionQueue code (_actionQueue)
-    /// - Key listeners (_keyListeners || m_key_listeners)
+    /// - Key listeners (m_key_listeners)
+    /// - Mouse listeners (m_mouse_listeners)
     /// - global Key object (_keyobject)
+    /// - global Mouse object (_mouseobject)
+    /// - Any character being dragged 
     ///
     void markReachableResources() const;
 #endif // GNASH_USE_GC
 
+    /// \brief
+    /// Register a newly born advanceable character to the
+    /// list of characters to be advanced on next ::advance call.
+    //
+    /// The character will only be advanced if not unloaded when
+    /// its turn comes. Characters are advanced in reverse-placement
+    /// order (first registered is advanced last)
+    ///
+    void addLiveChar(boost::intrusive_ptr<character> ch)
+    {
+	// Don't register the object in the list twice 
+	assert(std::find(_liveChars.begin(), _liveChars.end(), ch) == _liveChars.end());
+        _liveChars.push_front(ch);
+    }
+
+    /// Cleanup all resources and run the GC collector
+    //
+    /// This method should be invoked before calling setRootMovie again
+    /// for a clean restart.
+    ///
+    void clear();
+
+    /// Reset stage to it's initial state
+    //
+    void reset();
+
+    /// Call this method for disabling run of actions
+    //
+    /// NOTE: this will only work for queued actions, not
+    ///       for *every* action. Supposedly all actions should
+    ///       be queued, but this is not really always the case.
+    ///       Notable exceptions are:
+    ///         - Actions in callFrame target frame
+    ///           but only executed by execution of the callFrame opcode
+    ///         - on{,Clip}{Initialize,Construct} event handlers
+    ///         - User event handlers (mouse,keyboard)
+    ///
+    void disableScripts();
+
+    /// Return true if scripts execution is disabled
+    bool scriptsDisabled() const { return _disableScripts; };
+
+    /// Process action queues with higher priority then the priority
+    /// of the action queue currently being processed.
+    //
+    /// This is intended to be called at the end of any function call
+    /// and at the end of an action block.
+    ///
+    /// TODO: be aware of infinite loops !
+    ///
+    void flushHigherPriorityActionQueues();
+
+    character* findCharacterByTarget(const std::string& tgtstr) const;
+
+    /// URL access methods
+    enum LoadMethod {
+        NONE=0,
+        GET=1,
+        POST=2
+    };
+
+    /// Queue a request for loading a movie
+    //
+    /// @param url
+    ///		The url to load.
+    ///
+    /// @param target
+    ///	    Target to load into.
+    ///
+    /// @param postdata
+    ///     If not null, the data to POST in an HTTP request.
+    ///     Tests show that if you queue a load request for a target which
+    ///     is unloaded at time of processing, you still get the original
+    ///     target variables posted, not the new ones !
+    ///	    See http://savannah.gnu.org/bugs/index.php?22257
+    ///
+    void loadMovie(const URL& url, const std::string& target, const std::string* postdata=NULL);
+
+    /// Return true if the given string can be interpreted as a _level name
+    //
+    /// @param name
+    ///   The target string.
+    ///   Will be considered case-insensitive if VM version is < 7.
+    ///
+    /// @param levelno
+    ///   Output parameter, will be set to the level number, if true is returned
+    ///
+    bool isLevelTarget(const std::string& name, unsigned int& levelno);
+
+
+    /// Set a filedescriptor to use for host application requests
+    /// (for browser communication mostly)
+    void setHostFD(int fd)
+    {
+        assert(fd > 0);
+        _hostfd = fd;
+    }
+
+    /// Get the filedescriptor to use for host application requests
+    /// (for browser communication mostly)
+    ///
+    /// @return -1 if no filedescriptor is provided by host app.
+    int getHostFD()
+    {
+        return _hostfd;
+    }
+
 private:
 
+    /// A load movie request
+    class LoadMovieRequest {
+    public:
+	/// @param postdata
+	///   If not null POST method will be used for HTTP.
+	///
+        LoadMovieRequest(const URL& u, const std::string& t, const std::string* postdata)
+            :
+            _target(t),
+            _url(u),
+	    _usePost(false)
+        {
+		if ( postdata )
+		{
+			_postData = *postdata;
+			_usePost = true;
+		}
+	}
+
+        const std::string& getTarget() const { return _target; }
+        const URL& getURL() const { return _url; }
+        const std::string& getPostData() const { return _postData; }
+        bool usePost() const { return _usePost; }
+
+    private:
+        std::string _target;
+        URL _url;
+        bool _usePost;
+	std::string _postData;
+    };
+
+    /// Load movie requests
+    typedef std::list<LoadMovieRequest> LoadMovieRequests;
+    LoadMovieRequests _loadMovieRequests;
+
+    /// Process all load movie requests
+    void processLoadMovieRequests();
+
+    /// Process a single load movie request
+    void processLoadMovieRequest(const LoadMovieRequest& r);
+
+    /// Listeners container
+    typedef std::list< boost::intrusive_ptr<character> > CharacterList;
+
+    /// key and mouse listeners container
+    typedef CharacterList KeyListeners;
+    typedef CharacterList MouseListeners;
+
+
+    /// Take care of dragging, if needed
+    void doMouseDrag();
+
+    /// Delete all elements on the action queue and empty it.
+    void clearActionQueue();
+
+    /// Delete all elements on the timers list
+    void clearIntervalTimers();
+
+    /// An element of the advanceable characters
+    typedef boost::intrusive_ptr<character> AdvanceableCharacter;
+
+    /// A list of AdvanceableCharacters
+    //
+    /// This is a list (not a vector) as we want to allow
+    /// ::advance of each element to insert new characters before
+    /// the start w/out invalidating iterators scanning the
+    /// list forward for proper movie advancement
+    ///
+    typedef std::list<AdvanceableCharacter> LiveChars;
+
+    /// The list of advanceable character, in placement order
+    LiveChars _liveChars;
+
     /// Forbid copy 
-    movie_root(const movie_root& ) { assert(0); }
+    movie_root(const movie_root& ) { abort(); }
 
     /// Forbid assignment
-    movie_root& operator=(const movie_root& ) { assert(0); return *this; }
+    movie_root& operator=(const movie_root& ) { abort(); return *this; }
 
     /// Execute expired timers
     void executeTimers();
 
     /// Notify the global Key ActionScript object about a key status change
     key_as_object * notify_global_key(key::code k, bool down);
-    
-    /// Remove all listeners with a ref-count of 1
-    /// (only referenced as key listeners)
-    // in new design:
-    // remove unloaded characters and unregistered as_objects 
-    // from the key listeners container.
-    void cleanup_key_listeners();
+
+    /// Remove unloaded key and mouselisteners.
+    void cleanupUnloadedListeners()
+    {
+        cleanupUnloadedListeners(m_key_listeners);
+        cleanupUnloadedListeners(m_mouse_listeners);
+    }
+
+    /// Erase unloaded characters from the given listeners list
+    static void cleanupUnloadedListeners(CharacterList& ll);
+
+    /// Push a character listener to the front of given container, if not already present
+    static void add_listener(CharacterList& ll, character* elem);
+
+    /// Remove a listener from the list
+    static void remove_listener(CharacterList& ll, character* elem);
 
     /// Return the current Stage object
     //
@@ -574,7 +769,7 @@ private:
 
     typedef std::list<ExecutableCode*> ActionQueue;
 
-    ActionQueue _actionQueue;
+    ActionQueue _actionQueue[apSIZE];
 
     /// Process all actions in the queue
     void processActionQueue();
@@ -604,20 +799,15 @@ private:
     TimerMap _intervalTimers;
     unsigned int _lastTimerId;
 
-    /// A set of as_objects kept by intrusive_ptr
-    typedef std::set< boost::intrusive_ptr<as_object> > ListenerSet;
-
-    /// Objects listening for key events
-#ifdef NEW_KEY_LISTENER_LIST_DESIGN
-    KeyListeners _keyListeners;
-#else
-    ListenerSet m_key_listeners;
-#endif
+    /// Characters for listening key events
+    KeyListeners m_key_listeners;
 
     boost::intrusive_ptr<key_as_object> _keyobject;
 
+    boost::intrusive_ptr<as_object> _mouseobject;
+
     /// Objects listening for mouse events (down,up,move)
-    ListenerSet m_mouse_listeners;
+    MouseListeners m_mouse_listeners;
 
     character*              m_active_input_text;
     float                   m_time_remainder;
@@ -636,9 +826,13 @@ private:
     ///       Appropriate container could be list, set or map (order is important)
     ///
     typedef boost::intrusive_ptr<sprite_instance> LevelMovie;
-    typedef std::map<unsigned int, LevelMovie> Levels;
+    typedef std::map<int, LevelMovie> Levels;
     Levels _movies;
 
+    /// The root movie. This is initially the same as getLevel(0) but might
+    /// change during the run. It will be used to setup and retrive initial stage size
+    boost::intrusive_ptr<movie_instance> _rootMovie;
+
     /// This function should return TRUE iff any action triggered
     /// by the event requires redraw, see \ref events_handling for
     /// more info.
@@ -666,21 +860,26 @@ private:
     ///
     character* getTopmostMouseEntity(float x, float y);
 
-    // Advance all levels
-    void advanceAllLevels(float delta_time);
+    /// Delete characters removed from the stage
+    /// from the display lists
+    void cleanupDisplayList();
 
-    // Advance a given level
-    void advanceMovie(boost::intrusive_ptr<sprite_instance> movie, float delta_time);
+    /// Advance a live character
+    //
+    /// @param ch
+    ///     The character to advance, will NOT be advanced if unloaded
+    ///
+    static void advanceLiveChar(boost::intrusive_ptr<character> ch);
+
+    /// Advance all non-unloaded live chars
+    void advanceLiveChars();
 
     /// Put the given movie at the given level 
     //
-    /// Note that the display viewport will be updated to reflect
-    /// the new layout.
-    ///
     /// @param movie
     /// The movie_instance to store at the given level.
     /// Will be stored in an intrusive_ptr.
-    /// It's depth will be set to <num> and it's name to
+    /// It's depth will be set to <num>+character::staticDepthOffset and it's name to
     /// _level<num>
     ///
     void setLevel(unsigned int num, boost::intrusive_ptr<movie_instance> movie);
@@ -691,6 +890,65 @@ private:
     ///
     boost::intrusive_ptr<key_as_object> getKeyObject();
 
+    /// Return the global Mouse object 
+    //
+    /// TODO: expose the mouse_as_object directly for faster calls ?
+    ///
+    boost::intrusive_ptr<as_object> getMouseObject();
+
+    /// Boundaries of the Stage are always world boundaries
+    /// and are only invalidated by changes in the background
+    /// color.
+    void setInvalidated() { _invalidated=true; }
+
+    /// Every ::display call clears the invalidated flag
+    //
+    /// See setInvalidated();
+    ///
+    void clearInvalidated() { _invalidated=false; }
+
+    /// An invalidated stage will trigger complete redraw
+    //
+    /// So, this method should return true everytime a complete
+    /// redraw is needed. This is tipically only needed when
+    /// the background changes.
+    ///
+    /// See setInvalidated() and clearInvalidated().
+    ///
+    bool isInvalidated() { return _invalidated; }
+
+    /// See setInvalidated
+    bool _invalidated;
+
+    /// This is set to true if execution of scripts
+    /// aborted due to action limit set or whatever else
+    bool _disableScripts;
+
+    /// Return the priority level of first action queue containing actions.
+    //
+    /// Scanned in proprity order (lower first)
+    ///
+    int minPopulatedPriorityQueue() const;
+
+    /// Process all actions in the the given queue, till more actions
+    /// are found in lower levels, in which case we have an earlier
+    /// return.
+    int processActionQueue(int lvl);
+
+    int _processingActionLevel;
+
+    bool processingActions() const
+    {
+        return (_processingActionLevel < apSIZE);
+    }
+
+    const character* findDropTarget(float x, float y, character* dragging) const;
+
+    /// filedescriptor to write to for host application requests
+    //
+    /// -1 if none
+    ///
+    int _hostfd;
 };
 
 
