@@ -5,17 +5,18 @@
 
 // Quadratic bezier outline shapes, the basis for most SWF rendering.
 
-/* $Id: shape_character_def.h,v 1.14 2007/08/06 03:30:19 strk Exp $ */
+/* $Id: shape_character_def.h,v 1.20 2007/12/01 15:40:59 strk Exp $ */
 
 #ifndef GNASH_SHAPE_CHARACTER_DEF_H
 #define GNASH_SHAPE_CHARACTER_DEF_H
 
 
 #include "character_def.h" // for inheritance of shape_character_def
-#include "tesselate.h" 
 #include "shape.h" // for path
 #include "rect.h" // for composition
 
+#include <vector> // for composition
+
 
 namespace gnash {
 	class cxform;
@@ -28,10 +29,7 @@ namespace gnash {
 	/// Represents the outline of one or more shapes, along with
 	/// information on fill and line styles.
 	//
-	/// Inheritance from tesselating_shape is only needed to expose a known interface
-	/// for mesh_set class use.
-	///
-	class shape_character_def : public character_def, public tesselate::tesselating_shape
+	class shape_character_def : public character_def
 	{
 	public:
 
@@ -43,19 +41,42 @@ namespace gnash {
 		virtual ~shape_character_def();
 
 		virtual void	display(character* inst);
-		bool	point_test_local(float x, float y);
+		virtual bool point_test_local(float x, float y);
 
 		float	get_height_local() const;
 		float	get_width_local() const;
 
+		/// \brief
+		/// Read a shape definition as included in DEFINEFONT*,
+		/// DEFINESHAPE* or DEFINEMORPH* tag
+		//
+		/// @param in
+		///	The stream to read the shape from
+		///
+		/// @param tag_type
+		///	The SWF::tag_type this shape definition is read for.
+		///	TODO: change to an actual SWF::tag_type type
+		///
+		/// @param with_style
+		///	If true, this definition includes bounds, fill styles and line styles.
+		///	Tipically, this is only false for DEFINEFONT* tags.
+		///	NOTE: if with_style is false, bounds of the shape will be computed
+		///	      rather then read.
+		///	TODO: drop this function, set based on tag_type ?
+		///
+		/// @param m
+		///	The movie definition corresponding to the SWF we/re parsing.
+		///	This is used to resolve bitmap characters for fill styles, never
+		///	used if with_style is false.
+		///
 		void	read(stream* in, int tag_type, bool with_style, movie_definition* m);
+
 		void	display(
 			const matrix& mat,
 			const cxform& cx,
 			float pixel_scale,
 			const std::vector<fill_style>& fill_styles,
 			const std::vector<line_style>& line_styles) const;
-		virtual void	tesselate(float error_tolerance, tesselate::trapezoid_accepter* accepter) const;
 
 		/// Get cached bounds of this shape.
 		const rect&	get_bound() const { return m_bound; }
@@ -63,10 +84,6 @@ namespace gnash {
 		/// Compute bounds by looking at the component paths
 		void	compute_bound(rect* r) const;
 
-		// deprecated
-		//void	output_cached_data(tu_file* out, const cache_options& options);
-		//void	input_cached_data(tu_file* in);
-
 		const FillStyleVect& get_fill_styles() const { return m_fill_styles; }
 		const LineStyleVect& get_line_styles() const { return m_line_styles; }
 
@@ -75,6 +92,10 @@ namespace gnash {
 		// morph uses this
 		void	set_bound(const rect& r) { m_bound = r; /* should do some verifying */ }
 
+		size_t numEdges() const;
+
+		size_t numPaths() const;
+
 	protected:
 		friend class morph2_character_def;
 
@@ -121,8 +142,6 @@ namespace gnash {
 			return *this;
 		}
 
-		// Cached pre-tesselated meshes. (deprecated)
-		//mutable std::vector<mesh_set*>	m_cached_meshes;
 	};
 
 }	// end namespace gnash
