@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,21 +15,19 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
-/* $Id: rect.h,v 1.21 2007/07/01 10:54:24 bjacques Exp $ */
+/* $Id: rect.h,v 1.23 2008/01/21 20:55:51 rsavoye Exp $ */
 
 #ifndef GNASH_RECT_H
 #define GNASH_RECT_H
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "tu_config.h"
 
 #include "Range2d.h"
+#include "Point2d.h"
 
 #include <cassert> // for inlines
 #include <iostream> // for output operator
@@ -38,7 +36,6 @@
 namespace gnash {
 	class matrix;
 	class stream;
-	class point; // is a forward declaration enough for a return type ?
 }
 
 namespace gnash {
