@@ -37,11 +37,6 @@ namespace gnash {
 	///
 	namespace render
 	{
-
-		/// See render_handler::create_bitmap_info_alpha (in backend/render_handler.h)
-		bitmap_info*	create_bitmap_info_alpha(int w, int h,
-					unsigned char* data);
-
 		/// See render_handler::create_bitmap_info_rgb (in backend/render_handler.h)
 		bitmap_info*	create_bitmap_info_rgb(image::rgb* im);
 
@@ -81,12 +76,6 @@ namespace gnash {
 		/// See render_handler::end_display (in backend/render_handler.h)
 		void	end_display();
 
-		/// See render_handler::set_matrix (in backend/render_handler.h)
-		void	set_matrix(const matrix& m);
-
-		/// See render_handler::set_cxform (in backend/render_handler.h)
-		void	set_cxform(const cxform& cx);
-
 		/// \brief
 		/// Draw triangles using the current fill-style 0.
 		/// Clears the style list after rendering.
@@ -95,16 +84,17 @@ namespace gnash {
 		/// triangle-strip order.  The type of the array should
 		/// be float[vertex_count*2]
 		///
-		void	draw_mesh_strip(const int16_t coords[],
+		void	draw_mesh_strip(const boost::int16_t coords[],
 				int vertex_count);
 
 		/// See render_handler::draw_line_strip (in backend/render_handler.h)
-		void	draw_line_strip(const int16_t coords[],
-				int vertex_count, const rgba& color);
+		void	draw_line_strip(const boost::int16_t coords[],
+				int vertex_count, const rgba& color, const matrix& mat);
 
 		/// See render_handler::draw_poly (in backend/render_handler.h)
 		void  draw_poly(const point* corners, int corner_count,
-				const rgba& fill, const rgba& outline);
+				const rgba& fill, const rgba& outline, const matrix& mat,
+				bool masked);
       
 		/// See render_handler::draw_shape_character (in backend/render_handler.h)
 		void draw_shape_character(shape_character_def *def,
@@ -114,9 +104,6 @@ namespace gnash {
 		void draw_glyph(shape_character_def *def, const matrix& mat,
 				const rgba& color, float pixel_scale);
 
-		/// See render_handler::allow_glyph_textures (in backend/render_handler.h)
-		bool allow_glyph_textures();				
-				
 		/// See render_handler::bounds_in_clipping_area (in backend/render_handler.h)
 		bool bounds_in_clipping_area(const rect& bounds);
 		bool bounds_in_clipping_area(const InvalidatedRanges& ranges);
@@ -131,11 +118,6 @@ namespace gnash {
 		/// See render_handler::disable_mask (in backend/render_handler.h)
 		void	disable_mask();
 
-		/// See render_handler::draw_bitmap (in backend/render_handler.h)
-		void	draw_bitmap(const matrix& m, const bitmap_info* bi,
-				const rect& coords,
-				const rect& uv_coords, const rgba& color);
-
 	}	// end namespace render
 
 }	// end namespace gnash
