@@ -1,6 +1,6 @@
 // sprite_instance.h:  Stateful live Sprite instance, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,17 +16,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
-
 // Stateful live Sprite instance
 
 #ifndef GNASH_SPRITE_INSTANCE_H
 #define GNASH_SPRITE_INSTANCE_H
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "edit_text_character.h" // temp hack
@@ -49,6 +45,7 @@ namespace gnash {
 	class swf_event;
 	class drag_state;
 	class LoadVariablesThread;
+	class gradient_record;
 }
 
 namespace gnash
@@ -68,7 +65,7 @@ public:
 	typedef std::list<const action_buffer*> ActionList;
 
 	// definition must match movie_definition::PlayList
-	typedef std::vector<execute_tag*> PlayList;
+	typedef std::vector<ControlTag*> PlayList;
 
 	typedef std::vector<swf_event*> SWFEventsVector;
 
@@ -95,7 +92,7 @@ public:
 
 	/// Type of execute tags
 	//
-	/// TODO: move to execute_tag.h ?
+	/// TODO: move to ControlTag.h ?
 	///
 	enum control_tag_type
 	{
@@ -106,17 +103,15 @@ public:
 		TAG_DLIST  = 1<<1
 	};
 
-	virtual void has_key_event() {
-		m_has_key_event = true;
-	}
-
-	virtual void has_mouse_event();
-
 	/// \brief
-	/// Return this sprite's relative root as
+	/// Return this sprite's root as
 	/// specified at contruction time
+	//
+	/// Relative or absolute is determined by
+	/// the _lockroot property, see getLockRoot
+	/// and setLockRoot.
 	///
-	virtual sprite_instance* get_root_movie();
+	virtual movie_instance* get_root() const;
 
 	/// \brief
 	/// Return the sprite_definition (or movie_definition)
@@ -235,18 +230,28 @@ public:
 	/// covers that can receive mouse events.  NULL if
 	/// none.  Coords are in parent's frame.
 	virtual character* get_topmost_mouse_entity(float x, float y);
+
+    // see dox in character.h
+	const character* findDropTarget(float x, float y, character* dragging) const;
+
+    void setDropTarget(const std::string& tgt)
+    {
+        _droptarget = tgt;
+    }
+
+    const std::string& getDropTarget() const
+    {
+        return _droptarget;
+    }
 	
-	virtual bool wantsInstanceName()
+	virtual bool wantsInstanceName() const
 	{
 		return true; // sprites can be referenced 
 	}
 
-	virtual void	advance(float delta_time);
-	//virtual void	advance_root(float delta_time);
-	virtual void	advance_sprite(float delta_time);
+	virtual void	advance();
 
-	/// Take care of this frame's actions.
-	void do_actions();
+	void	advance_sprite();
 
 	/// Set the sprite state at the specified frame number.
 	//
@@ -274,6 +279,30 @@ public:
 	///
 	bool get_frame_number(const as_value& frame_spec, size_t& frameno) const;
 
+	/// MovieClip instances need to handle cxform specially 
+	//
+	/// This is to suppor the Color asobject class
+	///
+	cxform	get_world_cxform() const;
+
+	/// Update user-defined color transform
+	//
+	/// This should only be used by the Color AS class
+	///
+	void set_user_cxform(const cxform& cx)
+	{
+		set_invalidated();
+		_userCxform = cx;
+	}
+
+	/// Return the user-defined color transform
+	//
+	/// This should only be used by the Color AS class
+	///
+	cxform  get_user_cxform() const
+	{
+		return _userCxform;
+	}
 
 	/// Look up the labeled frame, and jump to it.
 	bool goto_labeled_frame(const std::string& label);
@@ -314,10 +343,10 @@ public:
 	///
 	/// If the specified depth is already occupied by an instance of a different
 	/// character (including any dynamically-created instance), the behaviour is
-	/// controlled by the replace_if_depth_is_occupied parameter. If false, this
+	/// controlled by the is_jumping_back flag. If false, this
 	/// call will result in a no-op. If true, the previously existing character
 	/// will be replaced by the new one, with unload() method invoked on the
-	/// removed character.
+	/// removed character. 
 	///
 	/// Any successful new placement triggers invokation of the newly created
 	/// instance's LOAD event.
@@ -338,12 +367,6 @@ public:
 	/// @param depth
 	///	The depth to assign to the newly created instance.
 	///
-	/// @param replace_if_depth_is_occupied
-	///	This parameter control behaviour in case an existing instance is found at
-	///	the given depth but is NOT an instance of the given character_id.
-	///	In this case: if this paremeter is true, the old instance will be replaced by
-	///	the new one, otherwise nothing happens and this function returns NULL.
-	///
 	/// @param color_transform
 	///	The color transform to apply to the newly created instance.
 	///
@@ -361,15 +384,13 @@ public:
 	/// @return 
 	///	A pointer to the character being added or NULL
 	///	if this call results in a move of an existing character 
-	///	or in a no-op due to replace_if_depth_is_occupied being
-	///	false.
+	///	or in a no-op due to is_jumping_back being false. 
 	///       
 	character* add_display_object(
-		uint16_t character_id,
-		const char* name,
+		boost::uint16_t character_id,
+		const std::string* name,
 		const SWFEventsVector& event_handlers,
 		int depth,
-		bool replace_if_depth_is_occupied,
 		const cxform& color_transform,
 		const matrix& matrix,
 		int ratio,
@@ -388,26 +409,43 @@ public:
 	///
 	bool attachCharacter(character& newch, int depth);
 
-	/// Construct this instance as an ActionScript object.
+	/// Handle placement event
 	//
-	/// This function must be called when the sprite is placed on
-	/// stage for the first time. It will take care of invoking
-	/// the constructor of its associated class, either MovieClip
-	/// or any user-specified one (see sprite_definition::registerClass).
+	/// This callback will (not known to be a problem):
 	///
-	/// Make sure this sprite got an instance name before calling
-	/// this method (it's needed for properly setting the "this" pointer
-	/// when calling user-defined constructors).
+	/// (1) Register ourselves with the global instance list
+	/// (2) Take note of our original target path
+	/// (3) Register as listener of core broadcasters
+	/// (4) Execute tags of frame 0
 	///
-	/// TODO: check if we only need to construct "named" instances
-	/// TODO: consider moving this function up to the 'character' class.
-	/// TODO: possibly have this function call the onConstruct() event handler
+	/// The callback will also (known to be bogus):
+	//
+	/// (1) Construct this instance as an ActionScript object.
+	///     See constructAsScriptObject() method.
 	///
-	void construct();
+	virtual void stagePlacementCallback();
 
 	/// Unload all contents in the displaylist and this instance
 	/// See character::unload for more info
-	void unload();
+	bool unload();
+
+	/// Mark this sprite as destroyed
+	//
+	/// This is an override of character::destroy()
+	///
+	/// A sprite should be destroyed when is removed from the display
+	/// list and is not more needed for names (target) resolutions.
+	/// Sprites are needed for names resolution whenever themselves
+	/// or a contained object has an onUnload event handler defined, 
+	/// in which case we want the event handler to find the 'this'
+	/// variable w/out attempting to rebind it.
+	///
+	/// When a sprite is destroyed, all its children are also destroyed.
+	/// 
+	/// Note: this function will release most memory associated with
+	/// the sprite as no members or drawable should be needed anymore.
+	///
+	void destroy();
 
 	/// See DisplayList::move_display_object, this method is just a proxy to that...
 	//
@@ -426,7 +464,8 @@ public:
 			int ratio,
 			int clip_depth)
 	{
-	    m_display_list.move_display_object(depth, color_xform, mat, ratio, clip_depth);
+		DisplayList& dlist = const_cast<DisplayList &>( getDisplayList() );
+	    dlist.move_display_object(depth, color_xform, mat, ratio, clip_depth);
 	}
 
 
@@ -435,13 +474,19 @@ public:
 	///	The color tranform to assign to the new character.
 	///	If NULL the default color transform will be kept.
 	///
+	/// @param name
+	///	The name to give to the newly created instance.
+	///	If NULL, the new instance will be assigned a sequential
+	///	name in the form 'instanceN', where N is incremented
+	///	at each call, starting from 1.
+	///
 	/// @param mat
 	///	The matrix tranform to assign to the new character.
 	///	If NULL the default matrix will be kept.
 	///
 	void	replace_display_object(
-			uint16_t character_id,
-			const char* name,
+			boost::uint16_t character_id,
+			const std::string* name,
 			int depth,
 			const cxform* color_xform,
 			const matrix* mat,
@@ -453,14 +498,23 @@ public:
 	/// @param color_xform
 	///	The color tranform to assign to the new character.
 	///	If NULL the default color transform will be kept.
-	//
+	///
+	/// @param ch
+	///	The character instance that should replace the old one.
+	///
+	/// @param name
+	///	The name to give to the new character (ch).
+	///	If NULL, the new instance will be assigned a sequential
+	///	name in the form 'instanceN', where N is incremented
+	///	at each call, starting from 1.
+	///
 	/// @param mat
 	///	The matrix tranform to assign to the new character.
 	///	If NULL the default matrix will be kept.
 	///
 	void	replace_display_object(
 			character* ch,
-			const char* name,
+			const std::string* name,
 			int depth,
 			const cxform* color_xform,
 			const matrix* mat,
@@ -476,19 +530,9 @@ public:
 	///
 	void	remove_display_object(int depth, int /* id */)
 	{
-      	character* existing_char = m_display_list.get_character_at_depth(depth);
-		if(existing_char && existing_char->getTimelineInfo())
-		{
-  			// Don't remove characters placed after target frame
-  			/// TODO: Don't remove character placed after the this RemoveObject tag
-			if(m_current_frame <= existing_char->getTimelineInfo()->placedInFrame() )
-			{
-				return;
-			}
-		}
-
 	    set_invalidated();
-	    m_display_list.remove_display_object(depth);
+		DisplayList& dlist = const_cast<DisplayList &>( getDisplayList() );
+	    dlist.remove_display_object(depth);
 	}
 
 
@@ -497,9 +541,29 @@ public:
 	/// frame advance.
 	void	add_action_buffer(const action_buffer* a)
 	{
-	    m_action_list.push_back(a);
+		if ( ! _callingFrameActions ) queueAction(*a);
+		else execute_action(*a);
 	}
 
+	
+	/// \brief
+	/// Execute the given init action buffer, if not done yet
+	/// for the target character id.
+	//
+	/// The action will normally be pushed on queue, but will
+	/// be executed immediately if we are executing actions
+	/// resulting from a callFame instead.
+	///
+	/// @param a
+	///	The action buffer to execute
+	///
+	/// @param cid
+	///	The referenced character id
+	///
+	void execute_init_action_buffer(const action_buffer& a, int cid);
+
+	/// Execute a single action buffer (DOACTION block)
+	void execute_action(const action_buffer& ab);
 
 	/// For debugging -- return the id of the character
 	/// at the specified depth.
@@ -510,9 +574,15 @@ public:
 
 	/// Load a movie in this sprite, replacing it
 	//
+	/// @param url
+	///  The url to load the movie from. Can be a bitmap or an SWF.
+	///
+	/// @param postdata
+	///  IF not NULL, use as the POST body for HTTP requests
+	///
 	/// Return: true if it succeeded, false otherwise
 	///
-	bool loadMovie(const URL& url);
+	bool loadMovie(const URL& url, const std::string* postdata=NULL);
 
 	/// \brief
 	/// Load url-encoded variables from the given url, optionally
@@ -535,23 +605,25 @@ public:
 	///	If 1, GET will be used.
 	///	If 2, POST will be used.
 	///
-	void loadVariables(const URL& url, short sendVarsMethod=0);
+	void loadVariables(URL url, short sendVarsMethod=0);
 
 	//
 	// ActionScript support
 	//
 
 	// See dox in as_object.h
-	bool get_member(const std::string& name, as_value* val);
+	bool get_member(string_table::key name, as_value* val, 
+		string_table::key nsname = 0);
 		
 	/// Set the named member to the value. 
 	//
 	/// Return true if we have that member; false otherwise.
 	///
-	virtual void set_member(const std::string& name, const as_value& val);
+	virtual void set_member(string_table::key name, const as_value& val,
+		string_table::key nsname = 0);
 
 	/// Overridden to look in DisplayList for a match
-	virtual character* get_relative_target(const std::string& name);
+	as_object* get_path_element(string_table::key key);
 
 	/// Execute the actions for the specified frame. 
 	//
@@ -600,7 +672,7 @@ public:
 	/// FIXME: unimplemented
 	virtual void	on_event_xmlsocket_onxml()
 	{
-	    log_msg("FIXME: %s: unimplemented\n", __FUNCTION__);
+	    log_debug("FIXME: %s: unimplemented\n", __FUNCTION__);
 	    on_event(event_id::SOCK_XML);
 	}
 		
@@ -608,14 +680,14 @@ public:
 	/// specified interval.
 	virtual void	on_event_interval_timer()
 	{
-	    log_msg("FIXME: %s: unimplemented\n", __FUNCTION__);
+	    log_debug("FIXME: %s: unimplemented\n", __FUNCTION__);
 	    on_event(event_id::TIMER);
 	}
 
 	/// Do the events that happen as a MovieClip (swf 7 only) loads.
 	virtual void	on_event_load_progress()
 	{
-	    log_msg("FIXME: %s: unimplemented\n", __FUNCTION__);
+	    log_debug("FIXME: %s: unimplemented\n", __FUNCTION__);
 	    on_event(event_id::LOAD_PROGRESS);
 	}
 
@@ -641,7 +713,11 @@ public:
 			
 
 	const DisplayList& getDisplayList() const {
-		return m_display_list;
+		if(! is_jumping_back)	{
+			return m_display_list;
+		} else {
+			return m_tmp_display_list;
+		}
 	}
 
 	/// Return the next highest available depth
@@ -656,7 +732,11 @@ public:
 
 	void testInvariant() const {
 		assert(m_play_state == PLAY || m_play_state == STOP);
-		assert(m_current_frame < m_def->get_frame_count());
+
+		// m_current_frame may be 0, since this is our initial
+		// condition. Still, frame count might be 0 as well, and
+		// loaded frames too !
+		//assert(m_current_frame < m_def->get_frame_count());
 #ifndef GNASH_USE_GC 
 		assert(get_ref_count() > 0); // or we're constructed but
 		                             // not stored in a boost::intrusive_ptr
@@ -669,10 +749,6 @@ public:
 	/// Get the current m_sound_stream_id
 	virtual int get_sound_stream_id() { return m_sound_stream_id;}
 
-	/// Override for character::set_name to proprely update
-	/// _target and _target_dot.
-	virtual void set_name(const char* name);
-
 	/// Remove this sprite from the stage.
 	//
 	/// This function is intended to be called by 
@@ -699,7 +775,7 @@ public:
 
 	/// @{ Drawing API
 	
-	void lineStyle(uint16_t thickness, const rgba& color)
+	void lineStyle(boost::uint16_t thickness, const rgba& color)
 	{
 		_drawable->lineStyle(thickness, color);
 	}
@@ -714,6 +790,16 @@ public:
 		_drawable->beginFill(color);
 	}
 
+	void beginLinearGradientFill(const std::vector<gradient_record>& grad, const matrix& mat)
+	{
+		_drawable->beginLinearGradientFill(grad, mat);
+	}
+
+	void beginRadialGradientFill(const std::vector<gradient_record>& grad, const matrix& mat)
+	{
+		_drawable->beginRadialGradientFill(grad, mat);
+	}
+
 	void endFill()
 	{
 		_drawable->endFill();
@@ -756,8 +842,49 @@ public:
 	///
 	virtual void enumerateNonProperties(as_environment&) const;
 
+	/// Delete characters removed from the stage
+	/// from the display lists
+	void cleanupDisplayList();
+
+        /// Queue the given action buffer
+        //
+        /// The action will be pushed on the current
+        /// global list (see movie_root).
+        ///
+        void queueAction(const action_buffer& buf);
+
+	/// Construct this instance as an ActionScript object
+	//
+	/// This method invokes the constructor associated with our
+	/// definition, either MovieClip or any user-speficied one
+	/// (see sprite_definition::registerClass). 
+	/// It will also invoke the onClipConstruct and onConstruct handlers.
+	///
+	void constructAsScriptObject();
+
+
+	/// Return true if get_root() should return the *relative* root,
+	/// false otherwise.
+	bool getLockRoot() const { return _lockroot; }
+
+	/// Set whether get_root() should return the *relative* root,
+	/// false otherwise. True for relative root.
+	///
+	void setLockRoot(bool lr) { _lockroot=lr; }
+
+	/// Getter-setter for MovieClip._lockroot
+	static as_value lockroot_getset(const fn_call& fn);
+
 private:
 
+	/// Register this sprite as a listener of core broadcasters
+	//
+	/// This is currently only used for key events broadcaster
+	/// and it's supposed to be called only once (or should we
+	/// also call it whenever onKey* user-defined function is defined ?
+	///
+	void registerAsListener();
+
 	/// \brief
 	/// Return value of the 'enabled' property, casted to a boolean value.
 	/// True if not found (undefined to bool evaluates to false).
@@ -775,43 +902,31 @@ private:
 		:
 		character(NULL, 0)
 	{
-		assert(0);
+		abort();
 	}
 
 	/// Forbid assignment
-	sprite_instance& operator=(const sprite_instance&) { assert(0); return *this; }
+	sprite_instance& operator=(const sprite_instance&) { abort(); return *this; }
 
-	/// \brief
-	/// Call has_key_event() or has_mouse_event()
-	/// if the given string correspond to an event handler
-	/// for which registering as a listener of Mouse or Key is needed
+	/// Advance to a previous frame.
 	//
+	/// This function will basically restore the DisplayList as it supposedly
+	/// was *before* executing tags in target frame and then execute target
+	/// frame tags (both DLIST and ACTION ones).
 	///
-	/// @param name
-	///	Member name. 
-	///
-	void checkForKeyOrMouseEvent(const std::string& name);
-
-	/// Duplicate the object with the specified name
-	/// and add it with a new name  at a new depth.
-	void clone_display_object(const std::string& name,
-		const std::string& newname, int depth);
-
-	/// Reset the DisplayList for proper loop-back or goto_frame
-	//
-	/// The DisplayList is cleared by all but dynamic characters
-	///
-	void resetDisplayList();
-
-	/// Reconstruct the DisplayList for proper loop-back operations
-	//
-	/// This function will:
+	/// In practice, it will:
 	///
 	/// - Remove from current DisplayList:
 	///	- Timeline instances constructed after target frame 
 	///	- Timeline instances constructed before or at the target frame but no more at the original depth
 	///	- Dynamic instances found in the static depth zone
-	/// - Execute all displaylist tags from first to target frame, incrementing m_current_frame as it goes
+	/// - Execute all displaylist tags from first to one-before target frame,
+	///   appropriately setting m_current_frame as it goes, finally execute both displaylist and action
+	///   tags for target frame.
+	///
+	/// Callers of this methods are:
+	///	- goto_frame (for jump-backs)
+	///	- advance_sprite (for loop-back)
 	///
 	/// See: http://www.gnashdev.org/wiki/index.php/TimelineControl#Timeline_instances
 	///
@@ -823,8 +938,6 @@ private:
 	///
 	///	- m_current_frame == targetFrame
 	///
-	/// NOTES: resetDisplayList() above should likely just call restoreDisplayList(0);
-	///
 	/// TODO: consider using this same function for jump-forward too,
 	///       with some modifications...
 	///
@@ -837,9 +950,6 @@ private:
 	///
 	void queueActions(ActionList& action_list);
 
-	/// Execute a single action buffer (DOACTION block)
-	void execute_action(const action_buffer& ab);
-
 	/// Execute the actions in the action list
 	//
 	/// The list of action will be consumed starting from the first
@@ -853,12 +963,8 @@ private:
 	/// Current Display List contents.
 	DisplayList	m_display_list;
 
-	/// oldDisplayList is a backup of current DisplayList
-	/// (m_display_list) updated at each call to ::advance
-	/// and at first call to ::construct (it's empty in
-	/// this latter case).
-	/// It will be used to control actions execution order.
-	DisplayList	oldDisplayList;
+	/// temporary display list used for timeline construction during jump back
+	DisplayList m_tmp_display_list;
 
 	/// The canvas for dynamic drawing
 	//
@@ -878,8 +984,6 @@ private:
 	///
 	boost::intrusive_ptr<character> _drawable_inst;
 
-	ActionList	m_action_list;
-
 	// this is deprecated, we'll be pushing gotoframe target
 	// actions to the global action queue
 	//ActionList	m_goto_frame_action_list;
@@ -892,8 +996,11 @@ private:
 	// true if this sprite reached the last frame and restarted
 	bool		m_has_looped;
 
-	// a bit-array class would be ideal for this
-	std::vector<bool>	m_init_actions_executed;
+	// true is the sprite is jumping back.
+	bool		is_jumping_back;
+
+	// true is we're calling frame actions
+	bool _callingFrameActions;
 
 	/// This timeline's variable scope
 	as_environment	m_as_environment;
@@ -901,21 +1008,19 @@ private:
 	/// Increment m_current_frame, and take care of looping.
 	void increment_frame_and_check_for_loop();
 
-	/// key events are: KEY_DOWN, KEY_PRESS, KEY_UP
-	bool m_has_key_event;
-
-	bool m_has_mouse_event;
+	typedef boost::intrusive_ptr< edit_text_character > TextFieldPtr;
+	typedef std::vector< TextFieldPtr > TextFieldPtrVect;
 
 	/// A container for textfields, indexed by their variable name
-	typedef std::map< std::string, boost::intrusive_ptr<edit_text_character> > TextfieldMap;
+	typedef std::map< std::string, TextFieldPtrVect > TextFieldMap;
 
 	/// We'll only allocate Textfield variables map if
 	/// we need them (ie: anyone calls set_textfield_variable)
 	///
-	std::auto_ptr<TextfieldMap> _text_variables;
+	std::auto_ptr<TextFieldMap> _text_variables;
 
 	/// \brief
-	/// Returns a TextField given it's variable name,
+	/// Returns a vector of TextField associated with the given variable name,
 	/// or NULL if no such variable name is known.
 	//
 	/// A TextField variable is a variable that acts
@@ -925,18 +1030,19 @@ private:
 	///
 	/// @todo find out wheter we should be case sensitive or not
 	///
-	edit_text_character* get_textfield_variable(const std::string& name);
+	/// @return a pointer inside a vector, will be invalidated by modifications
+	///         of the vector (set_textfield_variable)
+	///
+	TextFieldPtrVect* get_textfield_variable(const std::string& name);
 
 	/// soundid for current playing stream. If no stream set to -1
 	int m_sound_stream_id;
 
+	cxform _userCxform;
 
-	/// The DisplayList resulting by execution of tags in first frame.
-	//
-	/// It will be used to reinitialized actual DisplayList on restart.
-	/// See execute_frame_tags.
-	///
-	DisplayList _frame0_chars;
+	std::string _droptarget;
+
+	bool _lockroot;
 
 protected:
 
@@ -948,14 +1054,6 @@ protected:
 	}
 
 	/// Execute the tags associated with the specified frame.
-	//
-	/// Execution of 1st frame tags is specially handled:
-	///
-	/// - After executing them for the first time
-	///   the DisplayList state is saved into the _frame0_chars
-	///   member, which is used by resetDisplayList() on loop-back
-	///   TODO: drop this _frame0_chars thing ? See comments in
-	///   resetDisplayList()
 	///
 	/// @param frame
 	///	Frame number. 0-based
@@ -964,15 +1062,13 @@ protected:
 	///     Which kind of control tags we want to execute. 
 	///	See control_tag_type enum.
 	///
-	void execute_frame_tags(size_t frame, int typeflags=TAG_DLIST|TAG_ACTION); // bool state_only = false;
+	void execute_frame_tags(size_t frame, int typeflags=TAG_DLIST|TAG_ACTION); 
 
 	/// \brief
 	/// This is either sprite_definition (for sprites defined by
 	/// DefineSprite tag) or movie_def_impl (for the top-level movie).
 	boost::intrusive_ptr<movie_definition>	m_def;
 
-	bool m_on_event_load_called;
-
 	/// List of loadVariables requests
 	typedef std::list<LoadVariablesThread*> LoadVariablesThreads;
 
