@@ -5,7 +5,7 @@
 
 // line style types.
 
-/* $Id: styles.h,v 1.20 2007/08/07 20:53:10 strk Exp $ */
+/* $Id: styles.h,v 1.23 2007/12/04 11:45:29 strk Exp $ */
 
 #ifndef GNASH_STYLES_H
 #define GNASH_STYLES_H
@@ -18,7 +18,7 @@
 namespace gnash {
 
 class stream;
-
+class movie_definition;
 
 class base_line_style
 {
@@ -42,7 +42,7 @@ public:
 	/// @param color
 	///	Line color
 	///
-	line_style(uint16_t width, const rgba& color)
+	line_style(boost::uint16_t width, const rgba& color)
 		:
 		m_width(width),
 		m_color(color)
@@ -57,10 +57,15 @@ public:
 	/// Throw a ParserException if there's no enough bytes in the
 	/// currently opened tag for reading. See stream::ensureBytes()
 	///
-	void	read(stream* in, int tag_type);
+	void	read(stream* in, int tag_type, movie_definition *md);
 	
+	/// Read two lines styles from the SWF stream
+	/// at the same time -- this is used in morphing.
+	void read_morph(stream* in, int tag_type, movie_definition *md,
+		line_style *pOther);
+
 	/// Return thickness of the line, in TWIPS
-	uint16_t	get_width() const { return m_width; }
+	boost::uint16_t	get_width() const { return m_width; }
 
 	/// Return line color and alpha
 	const rgba&	get_color() const { return m_color; }
@@ -69,7 +74,7 @@ private:
 	friend class morph2_character_def;
 	friend class triangulating_render_handler;
 	
-	uint16_t	m_width;	// in TWIPS
+	boost::uint16_t	m_width;	// in TWIPS
 	rgba	m_color;
 };
 
@@ -82,7 +87,7 @@ public:
 	void read(stream* in);
 	
 private:
-	uint16_t m_width[2];
+	boost::uint16_t m_width[2];
 	rgba   m_color[2];
 };
 
