@@ -1,9 +1,9 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -19,14 +19,16 @@
 #define __GNASH_SWF_FUNCTION_H__
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "impl.h"
 #include "as_function.h" // for inheritance
 #include "as_object.h" // for composition (vector element)
+#include "VM.h" //fow SWF version
 //#include "with_stack_entry.h" // for composition (vector element)
 
+#include <boost/algorithm/string/case_conv.hpp> 
 #include <cassert>
 #include <string>
 
@@ -75,23 +77,12 @@ private:
 	};
 	std::vector<arg_spec>	m_args;
 	bool	m_is_function2;
-	uint8_t	m_local_register_count;
+	boost::uint8_t	m_local_register_count;
 
 	/// used by function2 to control implicit arg register assignments
 	// 
 	/// See http://sswf.sourceforge.net/SWFalexref.html#action_declare_function2
-	uint16_t	m_function2_flags;
-
-	/// Return a pointer to the given object's superclass interface
-	//
-	/// Super class prototype is : obj.__proto__.constructor.prototype
-	/// If any of the above element is undefined NULL is returned.
-	///
-	/// TODO: cleanup and optimize this function, probably delegating
-	///	  parts of it to the as_object class
-	///	  (getConstructor, for example)
-	///
-	static boost::intrusive_ptr<as_object> getSuper(as_object& obj);
+	boost::uint16_t	m_function2_flags;
 
 	/// Return an 'arguments' object.
 	//
@@ -209,16 +200,16 @@ public:
 
 	void	set_is_function2() { m_is_function2 = true; }
 
-	void	set_local_register_count(uint8_t ct) { assert(m_is_function2); m_local_register_count = ct; }
+	void	set_local_register_count(boost::uint8_t ct) { assert(m_is_function2); m_local_register_count = ct; }
 
-	void	set_function2_flags(uint16_t flags) { assert(m_is_function2); m_function2_flags = flags; }
+	void	set_function2_flags(boost::uint16_t flags) { assert(m_is_function2); m_function2_flags = flags; }
 
 	void	add_arg(int arg_register, const char* name)
 	{
 		assert(arg_register == 0 || m_is_function2 == true);
 		m_args.resize(m_args.size() + 1);
 		m_args.back().m_register = arg_register;
-		m_args.back().m_name = name;
+		m_args.back().m_name = PROPNAME(name);
 	}
 
 	void	set_length(int len);
@@ -227,6 +218,15 @@ public:
 	as_value	operator()(const fn_call& fn);
 
 	//void	lazy_create_properties();
+
+#ifdef GNASH_USE_GC
+	/// Mark reachable resources. Override from as_function.
+	//
+	/// Reachable resources from this object is it's scope stack
+	/// and the prototype.
+	///
+	virtual void markReachableResources() const;
+#endif // GNASH_USE_GC
 };
 
 
