@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,8 @@
 #include "sound_handler.h" // for creating the "test" sound handlers
 #include "types.h" // for rgba class
 #include "render_handler.h" // for dtor visibility by auto_ptr
+#include "movie_instance.h" 
+#include "ManualClock.h" // for composition
 
 #include <memory> // for auto_ptr
 #include <string> 
@@ -52,6 +54,7 @@ namespace gnash {
 	class sprite_instance;
 	class character;
 	class FuzzyPixel;
+	class VirtualClock;
 }
 
 namespace gnash {
@@ -104,7 +107,17 @@ public:
 	MovieTester(const std::string& filespec);
 
 	/// Advance the movie by one frame
-	void advance();
+	//
+	/// @param updateClock
+	///	If true (the default), this method also
+	///     advances the clock by the nominal delay expected
+	///     between frame advancements before performing the
+	///     actual playhead advancement.
+	///
+	void advance(bool updateClock=true);
+
+	/// Advance the clock by the given amount of milliseconds
+	void advanceClock(unsigned long ms);
 
 	/// Fully redraw of current frame
 	//
@@ -239,6 +252,14 @@ public:
 	/// Return true if this build of gnash supports video
 	bool canTestVideo() const;
 
+	/// Restart the movie
+	//
+	/// NOTE: the movie returned by getRootMovie() will likely be
+	///       NOT the real root movie anymore, so call getRootMovie
+	///	  again after this call.
+	///
+	void restart();
+
 private:
 
 	/// Initialize testing renderers
@@ -276,9 +297,9 @@ private:
 
 	gnash::movie_definition* _movie_def;
 
-	gnash::sprite_instance* _movie;
+	gnash::movie_instance* _movie;
 
-	std::auto_ptr<sound_handler> _sound_handler;
+	std::auto_ptr<media::sound_handler> _sound_handler;
 
 	/// Current pointer position - X ordinate
 	int _x;
@@ -309,6 +330,11 @@ private:
 	// When true, pass world invalidated ranges
 	// to the renderer(s) at ::render time.
 	bool _forceRedraw;
+
+	/// Virtual clock to use to let test runners
+	/// control time flow
+	ManualClock _clock;
+	//std::auto_ptr<VirtualClock> _clock;
 };
 
 } // namespace gnash
