@@ -1,6 +1,6 @@
 // processor.cpp:  Flash movie processor (gprocessor command), for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,8 +16,10 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+/* $Id: processor.cpp,v 1.84.2.1 2008/03/03 16:53:25 strk Exp $ */
+
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include <iostream>
@@ -25,9 +27,12 @@
 #include <sys/time.h>
 #include <time.h>
 
+#ifdef ENABLE_NLS
+#include <locale.h>
+#endif
+
 #include "gettext.h"
 #include "tu_file.h"
-#include "container.h"
 #include "gnash.h"
 #include "movie_definition.h"
 #include "sprite_instance.h"
@@ -38,6 +43,9 @@
 #include "GnashException.h"
 #include "debugger.h"
 #include "VM.h"
+#include "noseek_fd_adapter.h"
+#include "ManualClock.h"
+#include "tu_timer.h"
 
 extern "C"{
 	#include <unistd.h>
@@ -65,22 +73,9 @@ static const size_t allowloopbacks = 10;
 // TODO: add a command-line switch to control this
 static size_t limit_advances = 0;
 
-bool gofast = false;		// FIXME: this flag gets set based on
-				// an XML message written using
-				// SendCommand(""). This way a movie
-				// can optimize it's own performance
-				// when needed,
-bool nodelay = false;           // FIXME: this flag gets set based on
-				// an XML message written using
-				// SendCommand(""). This way a movie
-				// can optimize it's own performance
-				// when needed,
-
-extern int xml_fd;		// FIXME: this is the file descriptor
-				// from XMLSocket::connect(). This
-				// needs to be propogated up through
-				// the layers properly, but first I
-				// want to make sure it all works.
+// How much time to sleep between advances ?
+// If set to -1 it will be computed based on FPS.
+static long int delay = 0;
 
 const char *GPROC_VERSION = "1.0";
 
@@ -104,7 +99,6 @@ struct movie_data
 };
 
 static gnash::movie_definition*	play_movie(const char* filename);
-static int write_cache_file(const movie_data& md);
 
 static bool s_do_output = false;
 static bool s_stop_on_errors = true;
@@ -129,32 +123,41 @@ secondsSinceLastAdvance()
 	return ( now - lastAdvanceTimer);
 }
 
+// A flag which will be used to interrupt playback
+// by effect of a "quit" fscommand
+//
+static int quitrequested = false;
+
+void execFsCommand(sprite_instance* movie, const char* command, const char* args)
+{
+    log_debug(_("fs_callback(%p): %s %s"), (void*)movie, command, args);
+
+    if ( ! strcasecmp(command, "quit") ) quitrequested=true;
+}
+
 int
 main(int argc, char *argv[])
 {
-    assert(tu_types_validate());
-
     /// Initialize gnash core library
     gnashInit();
 
     // Enable native language support, i.e. internationalization
-    setlocale (LC_MESSAGES, "");
+#ifdef ENABLE_NLS
+    setlocale (LC_ALL, "");
     bindtextdomain (PACKAGE, LOCALEDIR);
     textdomain (PACKAGE);
-
+#endif
     int c;
 
     // scan for the two main standard GNU options
     for (c = 0; c < argc; c++) {
       if (strcmp("--help", argv[c]) == 0) {
         usage(argv[0]);
-	dbglogfile.removeLog();
         exit(0);
       }
       if (strcmp("--version", argv[c]) == 0) {
-        log_msg (_("Gnash gprocessor version: %s, Gnash version: %s"),
+        printf (_("Gnash gprocessor version: %s, Gnash version: %s\n"),
 		   GPROC_VERSION, VERSION);
-        dbglogfile.removeLog();
         exit(0);
       }
     }
@@ -178,7 +181,7 @@ main(int argc, char *argv[])
         dbglogfile.setVerbosity();
     }
 
-    while ((c = getopt (argc, argv, ":hwvapr:gf:")) != -1) {
+    while ((c = getopt (argc, argv, ":hwvapr:gf:d:")) != -1) {
 	switch (c) {
 	  case 'h':
 	      usage (argv[0]);
@@ -189,13 +192,13 @@ main(int argc, char *argv[])
 	      break;
 	  case 'v':
 	      dbglogfile.setVerbosity();
-	      log_msg (_("Verbose output turned on"));
+	      log_debug (_("Verbose output turned on"));
 	      break;
           case 'g':
 #ifdef USE_DEBUGGER
               debugger.enabled(true);
               debugger.console();
-              log_msg (_("Setting debugger ON"));
+              log_debug (_("Setting debugger ON"));
 #else
               log_error (_("The debugger has been disabled at configuration time"));
 #endif
@@ -216,6 +219,11 @@ main(int argc, char *argv[])
 	  case 'r':
               allowed_end_hits = strtol(optarg, NULL, 0);
 	      break;
+	  case 'd':
+              delay = atoi(optarg)*1000; // delay is in microseconds
+              // this will be recognized as a request to run at FPS speed
+              if ( delay < 0 ) delay = -1;
+	      break;
 	  case 'f':
               limit_advances = strtol(optarg, NULL, 0);
 	      break;
@@ -257,6 +265,8 @@ main(int argc, char *argv[])
 	exit(1);
     }
 
+    register_fscommand_callback(execFsCommand);
+
     // Play through all the movies.
     for (int i = 0, n = infiles.size(); i < n; i++) {
 
@@ -277,20 +287,6 @@ main(int argc, char *argv[])
 	data.push_back(md);
     }
     
-    // Now append processed data.
-    if (s_do_output) {
-	for (int i = 0, n = data.size(); i < n; i++) {
-	    int	error = write_cache_file(data[i]);
-	    if (error) {
-		if (s_stop_on_errors) {
-		    // Fail.
-		    fprintf(stderr, "error processing movie '%s', quitting\n", data[i].m_filename.c_str());
-		    exit(1);
-		}
-	    }
-	}
-    }
-
     // Signal core lib we're willing to quit.
     gnash::clear();
     
@@ -312,7 +308,29 @@ play_movie(const char* filename)
     gnash::movie_definition* md;
     try
     {
-      md = gnash::create_library_movie(URL(filename), NULL, false);
+      if ( ! strcmp(filename, "-") )
+      {
+         std::auto_ptr<tu_file> in ( noseek_fd_adapter::make_stream(fileno(stdin)) );
+         md = gnash::create_movie(in, filename, false);
+      }
+      else
+      {
+         URL url(filename);
+         if ( url.protocol() == "file" )
+         {
+             const std::string& path = url.path();
+#if 1 // add the *directory* the movie was loaded from to the local sandbox path
+             size_t lastSlash = path.find_last_of('/');
+             std::string dir = path.substr(0, lastSlash+1);
+             rcfile.addLocalSandboxPath(dir);
+             log_debug(_("%s appended to local sandboxes"), dir.c_str());
+#else // add the *file* to be loaded to the local sandbox path
+             rcfile.addLocalSandboxPath(path);
+             log_debug(_("%s appended to local sandboxes"), path.c_str());
+#endif
+         }
+         md = gnash::create_library_movie(url, NULL, false);
+      }
     }
     catch (GnashException& ge)
     {
@@ -324,9 +342,30 @@ play_movie(const char* filename)
 	exit(1);
     }
 
-    gnash::movie_root& m = VM::init(*md).getRoot();
+    float fps = md->get_frame_rate();
+    long fpsDelay = long(1000000/fps);
+    long clockAdvance = fpsDelay/1000;
+    long localDelay = delay == -1 ? fpsDelay : delay; // microseconds
+
+    printf("Will sleep %ld microseconds between iterations - fps is %g, clockAdvance is %lu\n", localDelay, fps, clockAdvance);
+
+    // Use a clock advanced at every iteration to match exact FPS speed.
+    ManualClock cl;
+    gnash::movie_root& m = VM::init(*md, cl).getRoot();
 
     md->completeLoad();
+
+    std::auto_ptr<movie_instance> mi ( md->create_movie_instance() );
+
+    m.setRootMovie( mi.release() );
+    if ( quitrequested )  // setRootMovie would execute actions in first frame
+    {
+        quitrequested = false;
+        return md;
+    }
+
+    printf("iteration, timer: %lu, localDelay: %ld\n", cl.elapsed(), localDelay);
+    usleep(localDelay);
     
     resetLastAdvanceTimer();
     int	kick_count = 0;
@@ -348,8 +387,17 @@ play_movie(const char* filename)
 	// safety margin on scaled shapes.
 	
 	size_t	last_frame = m.get_current_frame();
-	m.advance(0.010f);
-	m.display();
+	//printf("advancing clock by %lu\n", clockAdvance);
+	cl.advance(clockAdvance);
+	m.advance();
+
+	if ( quitrequested ) 
+	{
+		quitrequested = false;
+		return md;
+	}
+
+	m.display(); // FIXME: for which reason are we calling display here ??
 	++nadvances;
 	if ( limit_advances && nadvances >= limit_advances)
 	{
@@ -418,43 +466,14 @@ play_movie(const char* filename)
 	    stop_count = 0;
 	    resetLastAdvanceTimer();
 	}
+
+	printf("iteration, timer: %lu, localDelay: %ld\n", cl.elapsed(), localDelay);
+	usleep(localDelay);
     }
     
     return md;
 }
 
-int
-// Write a cache file for the given movie.
-write_cache_file(const movie_data& md)
-{
-    // Open cache file.
-    std::string	cache_filename(md.m_filename);
-    cache_filename += ".gsc";
-    tu_file	out(cache_filename.c_str(), "wb");	// "gsc" == "gnash cache"
-    if (out.get_error() == TU_FILE_NO_ERROR) {
-	// Write out the data.
-	gnash::cache_options	opt;
-	md.m_movie->output_cached_data(&out, opt);
-	if (out.get_error() == TU_FILE_NO_ERROR) {
-	    printf(
-		"wrote '%s'\n",
-		cache_filename.c_str());
-	} else {
-	    fprintf(stderr, "error: write failure to '%s'\n", cache_filename.c_str());
-	}
-    } else {
-	fprintf(stderr, "error: can't open '%s' for cache file output\n", cache_filename.c_str());
-	return 1;
-    }
-    
-    // // xxx temp debug code: dump cached data to stdout
-    // tu_file	tu_stdout(stdout, false);
-    // tu_stdout.copy_from(&cached_data);
-    
-    return 0;
-}
-
-
 static void
 usage (const char *name)
 {
@@ -486,6 +505,9 @@ usage (const char *name)
 	"",
 #endif
 	_(
+	"  -d [<ms>]\n"
+	"              Milliseconds delay between advances (0 by default).\n"
+	"              If '-1' the delay will be computed from the FPS.\n"
 	"  -r <times>  Allow the given number of complete runs.\n"
 	"              Keep looping undefinitely if set to 0.\n"
 	"              Default is 1 (end as soon as the last frame is reached).\n"
